@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,37 +13,51 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "buffy.h"
-#include "mx.h"
 #include "mailbox.h"
+#include "mx.h"
+
+#include "mutt_curses.h"
+
+#ifdef USE_IMAP
+#include "imap.h"
+#endif
 
 #include <string.h>
 #include <sys/stat.h>
 #include <dirent.h>
 #include <utime.h>
 #include <ctype.h>
+#include <unistd.h>
 
 #include <stdio.h>
 
-static time_t BuffyTime = 0;	/* last time we checked for new mail */
+static time_t BuffyTime = 0;	/* last time we started checking for mail */
+time_t BuffyDoneTime = 0;	/* last time we knew for sure how much mail there was. */
 static short BuffyCount = 0;	/* how many boxes with new mail */
 static short BuffyNotify = 0;	/* # of unnotified new boxes */
 
 #ifdef BUFFY_SIZE
 
-/* Find the last message in the file. * upon success return 0. * If no
- * message found - return -1 */
+/* Find the last message in the file. 
+ * upon success return 0. If no message found - return -1 */
+
 int fseek_last_message (FILE * f)
 {
   long int pos;
-  char buffer[BUFSIZ + 7];	/* 7 for "\n\nFrom " */
+  char buffer[BUFSIZ + 9];	/* 7 for "\n\nFrom " */
   int bytes_read;
   int i;			/* Index into `buffer' for scanning.  */
-  memset (buffer, 0, BUFSIZ+7);
+
+  memset (buffer, 0, sizeof(buffer));
   fseek (f, 0, SEEK_END);
   pos = ftell (f);
 
@@ -57,13 +71,13 @@ int fseek_last_message (FILE * f)
   while ((pos -= bytes_read) >= 0)
   {
     /* we save in the buffer at the end the first 7 chars from the last read */
-    strncpy (buffer + BUFSIZ, buffer, 7);
+    strncpy (buffer + BUFSIZ, buffer, 5+2); /* 2 == 2 * mutt_strlen(CRLF) */
     fseek (f, pos, SEEK_SET);
     bytes_read = fread (buffer, sizeof (char), bytes_read, f);
     if (bytes_read == -1)
       return -1;
     for (i = bytes_read; --i >= 0;)
-      if (!strncmp (buffer + i, "\n\nFrom ", strlen ("\n\nFrom ")))
+      if (!mutt_strncmp (buffer + i, "\n\nFrom ", mutt_strlen ("\n\nFrom ")))
       {				/* found it - go to the beginning of the From */
 	fseek (f, pos + i + 2, SEEK_SET);
 	return 0;
@@ -72,7 +86,7 @@ int fseek_last_message (FILE * f)
   }
 
   /* here we are at the beginning of the file */
-  if (!strncmp ("From ", buffer, 5))
+  if (!mutt_strncmp ("From ", buffer, 5))
   {
     fseek (f, 0, 0);
     return (0);
@@ -85,15 +99,18 @@ int fseek_last_message (FILE * f)
 int test_last_status_new (FILE * f)
 {
   HEADER *hdr;
+  ENVELOPE* tmp_envelope;
   int result = 0;
 
   if (fseek_last_message (f) == -1)
     return (0);
 
   hdr = mutt_new_header ();
-  mutt_read_rfc822_header (f, hdr);
+  tmp_envelope = mutt_read_rfc822_header (f, hdr, 0, 0);
   if (!(hdr->read || hdr->old))
     result = 1;
+
+  mutt_free_envelope(&tmp_envelope);
   mutt_free_header (&hdr);
 
   return result;
@@ -110,9 +127,11 @@ int test_new_folder (const char *path)
   if (typ != M_MBOX && typ != M_MMDF)
     return 0;
 
-  f = fopen (path, "rb");
-  rc = test_last_status_new (f);
-  fclose (f);
+  if ((f = fopen (path, "rb")))
+  {
+    rc = test_last_status_new (f);
+    fclose (f);
+  }
 
   return rc;
 }
@@ -150,31 +169,63 @@ void mutt_update_mailbox (BUFFY * b)
 }
 #endif
 
-int mutt_parse_mailboxes (const char *s, unsigned long data, char *err, size_t errlen)
+int mutt_parse_mailboxes (BUFFER *path, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  BUFFY **tmp;
+  BUFFY **tmp,*tmp1;
   char buf[_POSIX_PATH_MAX];
-  char expn[LONG_STRING];
 #ifdef BUFFY_SIZE
   struct stat sb;
 #endif /* BUFFY_SIZE */
 
-  do
+  while (MoreArgs (s))
   {
-    s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
+    mutt_extract_token (path, s, 0);
+    strfcpy (buf, path->data, sizeof (buf));
+
+    if(data == M_UNMAILBOXES && mutt_strcmp(buf,"*") == 0)
+    {
+      for (tmp = &Incoming; *tmp;)
+      {
+        FREE (&((*tmp)->path));
+        tmp1=(*tmp)->next;
+        FREE (tmp);
+        *tmp=tmp1;
+      }
+      return 0;
+    }
+
     mutt_expand_path (buf, sizeof (buf));
+
+    /* Skip empty tokens. */
+    if(!*buf) continue;
+
     /* simple check to avoid duplicates */
     for (tmp = &Incoming; *tmp; tmp = &((*tmp)->next))
     {
-      if (strcmp (buf, (*tmp)->path) == 0)
+      if (mutt_strcmp (buf, (*tmp)->path) == 0)
 	break;
     }
 
+    if(data == M_UNMAILBOXES)
+    {
+      if(*tmp)
+      {
+        FREE (&((*tmp)->path));
+        tmp1=(*tmp)->next;
+        FREE (tmp);
+        *tmp=tmp1;
+      }
+      continue;
+    }
+
     if (!*tmp)
     {
       *tmp = (BUFFY *) safe_calloc (1, sizeof (BUFFY));
       (*tmp)->path = safe_strdup (buf);
       (*tmp)->next = NULL;
+      /* it is tempting to set magic right here */
+      (*tmp)->magic = 0;
+      
     }
 
     (*tmp)->new = 0;
@@ -195,7 +246,6 @@ int mutt_parse_mailboxes (const char *s, unsigned long data, char *err, size_t e
       (*tmp)->size = 0;
 #endif /* BUFFY_SIZE */
   }
-  while (s);
   return 0;
 }
 
@@ -218,17 +268,29 @@ int mutt_buffy_check (int force)
   struct stat contex_sb;
   time_t t;
 
+#ifdef USE_IMAP
+  /* update postponed count as well, on force */
+  if (force)
+    mutt_update_num_postponed ();
+#endif
+
   /* fastest return if there are no mailboxes */
   if (!Incoming)
     return 0;
   t = time (NULL);
-  if (!force && t - BuffyTime < BuffyTimeout)
+  if (!force && (t - BuffyTime < BuffyTimeout))
     return BuffyCount;
  
   BuffyTime = t;
   BuffyCount = 0;
   BuffyNotify = 0;
 
+#ifdef USE_IMAP
+  if (!Context || Context->magic != M_IMAP)
+#endif
+#ifdef USE_POP
+  if (!Context || Context->magic != M_POP)
+#endif
   /* check device ID and serial number instead of comparing paths */
   if (!Context || !Context->path || stat (Context->path, &contex_sb) != 0)
   {
@@ -240,12 +302,21 @@ int mutt_buffy_check (int force)
   {
     tmp->new = 0;
 
-    if (stat (tmp->path, &sb) != 0 ||
+#ifdef USE_IMAP
+    if (mx_is_imap (tmp->path))
+      tmp->magic = M_IMAP;
+    else
+#endif
+#ifdef USE_POP
+    if (mx_is_pop (tmp->path))
+      tmp->magic = M_POP;
+    else
+#endif
+    if (stat (tmp->path, &sb) != 0 || sb.st_size == 0 ||
 	(!tmp->magic && (tmp->magic = mx_get_magic (tmp->path)) <= 0))
     {
       /* if the mailbox still doesn't exist, set the newly created flag to
-       * be ready for when it does.
-       */
+       * be ready for when it does. */
       tmp->newly_created = 1;
       tmp->magic = 0;
 #ifdef BUFFY_SIZE
@@ -254,8 +325,28 @@ int mutt_buffy_check (int force)
       continue;
     }
 
-    if (!Context || !Context->path || 
-	sb.st_dev != contex_sb.st_dev || sb.st_ino != contex_sb.st_ino)
+    /* check to see if the folder is the currently selected folder
+     * before polling */
+    if (!Context || !Context->path ||
+#if defined USE_IMAP || defined USE_POP
+	((
+#ifdef USE_IMAP
+	tmp->magic == M_IMAP
+#endif
+#ifdef USE_POP
+#ifdef USE_IMAP
+	||
+#endif
+	tmp->magic == M_POP
+#endif
+	) ? mutt_strcmp (tmp->path, Context->path) :
+#endif
+	 (sb.st_dev != contex_sb.st_dev || sb.st_ino != contex_sb.st_ino)
+#if defined USE_IMAP || defined USE_POP	 
+	    )
+#endif
+	)
+	
     {
       switch (tmp->magic)
       {
@@ -290,9 +381,11 @@ int mutt_buffy_check (int force)
 	}
 	while ((de = readdir (dirp)) != NULL)
 	{
-	  if (*de->d_name != '.')
+	  char *p;
+	  if (*de->d_name != '.' && 
+	      (!(p = strstr (de->d_name, ":2,")) || !strchr (p + 3, 'T')))
 	  {
-	    /* one new message is enough */
+	    /* one new and undeleted message is enough */
 	    BuffyCount++;
 	    tmp->new = 1;
 	    break;
@@ -302,26 +395,24 @@ int mutt_buffy_check (int force)
 	break;
 
       case M_MH:
-
-	if ((dirp = opendir (tmp->path)) == NULL)
-	{
-	  tmp->magic = 0;
-	  break;
-	}
-	while ((de = readdir (dirp)) != NULL)
-	{
-	  if (!mh_valid_message (de->d_name))
-	    continue;
-	  snprintf (path, sizeof (path), "%s/%s", tmp->path, de->d_name);
-	  if (stat (path, &sb) != -1 && sb.st_mtime >= sb.st_atime)
-	  {
-	    BuffyCount++;
-	    tmp->new = 1;
-	    break;
-	  }
-	}
-	closedir (dirp);
+	if ((tmp->new = mh_buffy (tmp->path)) > 0)
+	  BuffyCount++;
 	break;
+	
+#ifdef USE_IMAP
+      case M_IMAP:
+	if ((tmp->new = imap_mailbox_check (tmp->path, 1)) > 0)
+	  BuffyCount++;
+	else
+	  tmp->new = 0;
+
+	break;
+#endif
+
+#ifdef USE_POP
+      case M_POP:
+	break;
+#endif
       }
     }
 #ifdef BUFFY_SIZE
@@ -334,32 +425,71 @@ int mutt_buffy_check (int force)
     else if (!tmp->notified)
       BuffyNotify++;
   }
+
+  BuffyDoneTime = BuffyTime;
   return (BuffyCount);
 }
 
+int mutt_buffy_list (void)
+{
+  BUFFY *tmp;
+  char path[_POSIX_PATH_MAX];
+  char buffylist[160];
+  int pos;
+  int first;
+
+  int have_unnotified = BuffyNotify;
+  
+  pos = 0;
+  first = 1;
+  buffylist[0] = 0;
+  pos += strlen (strncat (buffylist, _("New mail in "), sizeof (buffylist) - 1 - pos)); /* __STRNCAT_CHECKED__ */
+  for (tmp = Incoming; tmp; tmp = tmp->next)
+  {
+    /* Is there new mail in this mailbox? */
+    if (!tmp->new || (have_unnotified && tmp->notified))
+      continue;
+
+    strfcpy (path, tmp->path, sizeof (path));
+    mutt_pretty_mailbox (path);
+    
+    if (!first && pos + strlen (path) >= COLS - 7)
+      break;
+    
+    if (!first)
+      pos += strlen (strncat(buffylist + pos, ", ", sizeof(buffylist)-1-pos)); /* __STRNCAT_CHECKED__ */
+
+    /* Prepend an asterisk to mailboxes not already notified */
+    if (!tmp->notified)
+    {
+      /* pos += strlen (strncat(buffylist + pos, "*", sizeof(buffylist)-1-pos));  __STRNCAT_CHECKED__ */
+      tmp->notified = 1;
+      BuffyNotify--;
+    }
+    pos += strlen (strncat(buffylist + pos, path, sizeof(buffylist)-1-pos)); /* __STRNCAT_CHECKED__ */
+    first = 0;
+  }
+  if (!first && tmp)
+  {
+    strncat (buffylist + pos, ", ...", sizeof (buffylist) - 1 - pos); /* __STRNCAT_CHECKED__ */
+  }
+  if (!first)
+  {
+    mutt_message ("%s", buffylist);
+    return (1);
+  }
+  /* there were no mailboxes needing to be notified, so clean up since 
+   * BuffyNotify has somehow gotten out of sync
+   */
+  BuffyNotify = 0;
+  return (0);
+}
+
 int mutt_buffy_notify (void)
 {
-  BUFFY *tmp;
-  char path[_POSIX_PATH_MAX];
-
   if (mutt_buffy_check (0) && BuffyNotify)
   {
-    for (tmp = Incoming; tmp; tmp = tmp->next)
-    {
-      if (tmp->new && !tmp->notified)
-      {
-	strfcpy (path, tmp->path, sizeof (path));
-	mutt_pretty_mailbox (path);
-	mutt_message ("New mail in %s.", path);
-	tmp->notified = 1;
-	BuffyNotify--;
-	return (1);
-      }
-    }
-    /* there were no mailboxes needing to be notified, so clean up since 
-     * BuffyNotify has somehow gottten out of sync
-     */
-    BuffyNotify = 0;
+    return (mutt_buffy_list ());
   }
   return (0);
 }
@@ -370,7 +500,7 @@ int mutt_buffy_notify (void)
  * given a folder name, this routine gives the next incoming folder with new
  * new mail.
  */
-void mutt_buffy (char *s)
+void mutt_buffy (char *s, size_t slen)
 {
   int count;
   BUFFY *tmp = Incoming;
@@ -380,7 +510,7 @@ void mutt_buffy (char *s)
   {
   case 0:
 
-    s = '\0';
+    *s = '\0';
     break;
 
   case 1:
@@ -389,11 +519,11 @@ void mutt_buffy (char *s)
       tmp = tmp->next;
     if (!tmp)
     {
-      s = '\0';
+      *s = '\0';
       mutt_buffy_check (1); /* buffy was wrong - resync things */
       break;
     }
-    strcpy (s, tmp->path);
+    strfcpy (s, tmp->path, slen);
     mutt_pretty_mailbox (s);
     break;
 
@@ -402,7 +532,7 @@ void mutt_buffy (char *s)
     count = 0;
     while (count < 3)
     {
-      if (strcmp (s, tmp->path) == 0)
+      if (mutt_strcmp (s, tmp->path) == 0)
 	count++;
       else if (count && tmp->new)
 	break;
@@ -415,11 +545,11 @@ void mutt_buffy (char *s)
     }
     if (count >= 3)
     {
-      s = '\0';
+      *s = '\0';
       mutt_buffy_check (1); /* buffy was wrong - resync things */
       break;
     }
-    strcpy (s, tmp->path);
+    strfcpy (s, tmp->path, slen);
     mutt_pretty_mailbox (s);
     break;
   }
