@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,10 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #ifdef USE_IMAP
 #include "mailbox.h"
@@ -48,19 +52,14 @@ int mutt_complete (char *s, size_t slen)
   dprint (2, (debugfile, "mutt_complete: completing %s\n", s));
 
   /* we can use '/' as a delimiter, imap_complete rewrites it */
-  if (*s == '=' || *s == '+')
+  if (*s == '=' || *s == '+' || *s == '!')
   {
-    if (s[1])
-    {
-      /* don't append '/' if Maildir is {host} only */
-      if (mx_is_imap (NONULL (Maildir)) && Maildir[strlen (Maildir)-1] == '}')
-        snprintf (imap_path, sizeof (imap_path), "%s%s", Maildir, s+1);
-      else
-        snprintf (imap_path, sizeof (imap_path), "%s/%s", NONULL (Maildir),
-          s+1);
-    }
+    if (*s == '!')
+      p = NONULL (Spoolfile);
     else
-      strfcpy (imap_path, NONULL(Maildir), sizeof(imap_path));
+      p = NONULL (Maildir);
+
+    mutt_concat_path (imap_path, p, s+1, sizeof (imap_path));
   }
   else
     strfcpy (imap_path, s, sizeof(imap_path));
@@ -69,16 +68,22 @@ int mutt_complete (char *s, size_t slen)
     return imap_complete (s, slen, imap_path);
 #endif
   
-  if (*s == '=' || *s == '+')
+  if (*s == '=' || *s == '+' || *s == '!')
   {
     dirpart[0] = *s;
     dirpart[1] = 0;
-    strfcpy (exp_dirpart, NONULL (Maildir), sizeof (exp_dirpart));
+    if (*s == '!')
+      strfcpy (exp_dirpart, NONULL (Spoolfile), sizeof (exp_dirpart));
+    else
+      strfcpy (exp_dirpart, NONULL (Maildir), sizeof (exp_dirpart));
     if ((p = strrchr (s, '/')))
     {
+      char buf[_POSIX_PATH_MAX];
       *p++ = 0;
-      sprintf (exp_dirpart + mutt_strlen (exp_dirpart), "/%s", s+1);
-      sprintf (dirpart + mutt_strlen (dirpart), "%s/", s+1);
+      mutt_concat_path (buf, exp_dirpart, s + 1, sizeof (buf));
+      strfcpy (exp_dirpart, buf, sizeof (exp_dirpart));
+      snprintf (buf, sizeof (buf), "%s%s/", dirpart, s+1);
+      strfcpy (dirpart, buf, sizeof (dirpart));
       strfcpy (filepart, p, sizeof (filepart));
     }
     else
@@ -169,13 +174,14 @@ int mutt_complete (char *s, size_t slen)
 	if (dirpart[0])
 	{
 	  strfcpy (buf, exp_dirpart, sizeof (buf));
-	  strcat (buf, "/");
+	  strfcpy (buf + strlen (buf), "/", sizeof (buf) - strlen (buf));
 	}
 	else
 	  buf[0] = 0;
-	strcat (buf, filepart);
+	strfcpy (buf + strlen (buf), filepart, sizeof (buf) - strlen (buf));
 	if (stat (buf, &st) != -1 && (st.st_mode & S_IFDIR))
-	  strcat (filepart, "/");
+	  strfcpy (filepart + strlen (filepart), "/",
+		   sizeof (filepart) - strlen (filepart));
 	init = 1;
       }
     }
@@ -184,13 +190,13 @@ int mutt_complete (char *s, size_t slen)
 
   if (dirpart[0])
   {
-    strcpy (s, dirpart);
+    strfcpy (s, dirpart, slen);
     if (mutt_strcmp ("/", dirpart) != 0 && dirpart[0] != '=' && dirpart[0] != '+')
-      strcat (s, "/");
-    strcat (s, filepart);
+      strfcpy (s + strlen (s), "/", slen - strlen (s));
+    strfcpy (s + strlen (s), filepart, slen - strlen (s));
   }
   else
-    strcpy (s, filepart);
+    strfcpy (s, filepart, slen);
 
   return (init ? 0 : -1);
 }
