@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,10 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mutt_menu.h"
@@ -23,25 +27,17 @@
 #include "mapping.h"
 #include "sort.h"
 #include "buffy.h"
+#include "mx.h"
+
+#ifdef USE_POP
+#include "pop.h"
+#endif
 
 #ifdef USE_IMAP
-#include "mx.h"
-#include "imap.h"
+#include "imap_private.h"
 #endif
 
-
-
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
-
-
-
-
-
-
-
-
+#include "mutt_crypt.h"
 
 
 #include <ctype.h>
@@ -56,6 +52,7 @@ static const char *No_mailbox_is_open = N_("No mailbox is open.");
 static const char *There_are_no_messages = N_("There are no messages.");
 static const char *Mailbox_is_read_only = N_("Mailbox is read-only.");
 static const char *Function_not_permitted_in_attach_message_mode = N_("Function not permitted in attach-message mode.");
+static const char *No_visible = N_("No visible messages.");
 
 #define CHECK_MSGCOUNT if (!Context) \
 	{ \
@@ -70,6 +67,14 @@ static const char *Function_not_permitted_in_attach_message_mode = N_("Function
 		break; \
 	}
 
+#define CHECK_VISIBLE if (Context && menu->current >= Context->vcount) \
+  	{\
+	  	mutt_flushinp (); \
+	  	mutt_error _(No_visible); \
+	  	break; \
+	}
+    
+
 #define CHECK_READONLY if (Context->readonly) \
 			{ \
 			  	mutt_flushinp (); \
@@ -77,6 +82,17 @@ static const char *Function_not_permitted_in_attach_message_mode = N_("Function
 				break; \
 			}
 
+#ifdef USE_IMAP 
+/* the error message returned here could be better. */
+#define CHECK_IMAP_ACL(aclbit) if (Context->magic == M_IMAP) \
+		if (mutt_bit_isset (((IMAP_DATA *)Context->data)->capabilities, ACL) \
+		&& !mutt_bit_isset(((IMAP_DATA *)Context->data)->rights,aclbit)){ \
+			mutt_flushinp(); \
+			mutt_error ("Operation not permitted by the IMAP ACL for this mailbox"); \
+			break; \
+		}
+#endif
+
 #define CHECK_ATTACH if(option(OPTATTACHMSG)) \
 		     {\
 			mutt_flushinp (); \
@@ -89,17 +105,18 @@ static const char *Function_not_permitted_in_attach_message_mode = N_("Function
 #define UNREAD(h) mutt_thread_contains_unread (Context, h)
 
 extern const char *ReleaseDate;
+extern size_t UngetCount;
 
 void index_make_entry (char *s, size_t l, MUTTMENU *menu, int num)
 {
   format_flag flag = M_FORMAT_MAKEPRINT | M_FORMAT_ARROWCURSOR | M_FORMAT_INDEX;
   int edgemsgno, reverse = Sort & SORT_REVERSE;
-  HEADER *tmp, *h = Context->hdrs[Context->v2r[num]];
+  HEADER *h = Context->hdrs[Context->v2r[num]];
+  THREAD *tmp;
 
   if ((Sort & SORT_MASK) == SORT_THREADS && h->tree)
   {
     flag |= M_FORMAT_TREE; /* display the thread tree */
-
     if (h->display_subject)
       flag |= M_FORMAT_FORCESUBJ;
     else
@@ -114,26 +131,32 @@ void index_make_entry (char *s, size_t l, MUTTMENU *menu, int num)
       else
 	edgemsgno = Context->v2r[menu->top];
 
-      for (tmp = h->parent; tmp; tmp = tmp->parent)
+      for (tmp = h->thread->parent; tmp; tmp = tmp->parent)
       {
+	if (!tmp->message)
+	  continue;
+
 	/* if no ancestor is visible on current screen, provisionally force
 	 * subject... */
-	if (reverse ? tmp->msgno > edgemsgno : tmp->msgno < edgemsgno)
+	if (reverse ? tmp->message->msgno > edgemsgno : tmp->message->msgno < edgemsgno)
 	{
 	  flag |= M_FORMAT_FORCESUBJ;
 	  break;
 	}
-	else if (tmp->virtual >= 0)
+	else if (tmp->message->virtual >= 0)
 	  break;
       }
-      if ((flag & M_FORMAT_FORCESUBJ) && h->prev)
+      if (flag & M_FORMAT_FORCESUBJ)
       {
-	for (tmp = h->prev; tmp; tmp = tmp->prev)
+	for (tmp = h->thread->prev; tmp; tmp = tmp->prev)
 	{
+	  if (!tmp->message)
+	    continue;
+
 	  /* ...but if a previous sibling is available, don't force it */
-	  if (reverse ? tmp->msgno > edgemsgno : tmp->msgno < edgemsgno)
+	  if (reverse ? tmp->message->msgno > edgemsgno : tmp->message->msgno < edgemsgno)
 	    break;
-	  else if (tmp->virtual >= 0)
+	  else if (tmp->message->virtual >= 0)
 	  {
 	    flag &= ~M_FORMAT_FORCESUBJ;
 	    break;
@@ -150,7 +173,7 @@ int index_color (int index_no)
 {
   HEADER *h = Context->hdrs[Context->v2r[index_no]];
 
-  if (h->pair)
+  if (h && h->pair)
     return h->pair;
 
   mutt_set_header_color (Context, h);
@@ -292,21 +315,21 @@ static void update_index (MUTTMENU *menu, CONTEXT *ctx, int check,
   
   /* if the mailbox was reopened, need to rethread from scratch */
   mutt_sort_headers (Context, (check == M_REOPENED));
-  
+
   /* uncollapse threads with new mail */
   if ((Sort & SORT_MASK) == SORT_THREADS)
   {
     if (check == M_REOPENED)
     {
-      HEADER *h;
+      THREAD *h, *j;
       
-      h = Context->tree;
       Context->collapsed = 0;
       
-      while (h)
+      for (h = Context->tree; h; h = h->next)
       {
-	mutt_uncollapse_thread (Context, h);
-	h = h->next;
+	for (j = h; !j->message; j = j->child)
+	  ;
+	mutt_uncollapse_thread (Context, j->message);
       }
       mutt_set_virtual (Context);
     }
@@ -350,10 +373,12 @@ static void update_index (MUTTMENU *menu, CONTEXT *ctx, int check,
 static void resort_index (MUTTMENU *menu)
 {
   int i;
-  const HEADER *current = CURHDR;
+  HEADER *current = CURHDR;
 
+  menu->current = -1;
   mutt_sort_headers (Context, 0);
   /* Restore the current message */
+
   for (i = 0; i < Context->vcount; i++)
   {
     if (Context->hdrs[Context->v2r[i]] == current)
@@ -362,6 +387,13 @@ static void resort_index (MUTTMENU *menu)
       break;
     }
   }
+  
+  if ((Sort & SORT_MASK) == SORT_THREADS && menu->current < 0)
+    menu->current = mutt_parent_message (Context, current);
+  
+  if (menu->current < 0)
+    menu->current = ci_first_message ();
+  
   menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 }
 
@@ -422,6 +454,13 @@ int mutt_index_menu (void)
      */
     if (option (OPTNEEDRESORT) && Context && Context->msgcount)
       resort_index (menu);
+    
+    if (option (OPTREDRAWTREE) && Context && Context->msgcount && (Sort & SORT_MASK) == SORT_THREADS)
+    {
+      mutt_draw_tree (Context);
+      menu->redraw |= REDRAW_STATUS;
+      unset_option (OPTREDRAWTREE);
+    }
 
     if (Context && !attach_msg)
     {
@@ -435,32 +474,34 @@ int mutt_index_menu (void)
       imap_allow_reopen (Context);
 #endif
     
-      index_hint = (Context->vcount && menu->current < Context->vcount) ? CURHDR->index : 0;
+      index_hint = (Context->vcount && menu->current >= 0 && menu->current < Context->vcount) ? CURHDR->index : 0;
 
       if ((check = mx_check_mailbox (Context, &index_hint, 0)) < 0)
       {
 	if (!Context->path)
 	{
 	  /* fatal error occurred */
-	  safe_free ((void **) &Context);
+	  FREE (&Context);
 	  menu->redraw = REDRAW_FULL;
 	}
 
 	set_option (OPTSEARCHINVALID);
       }
-      else if (check == M_NEW_MAIL || check == M_REOPENED)
+      else if (check == M_NEW_MAIL || check == M_REOPENED || check == M_FLAGS)
       {
 	update_index (menu, Context, check, oldcount, index_hint);
 	
 	/* notify the user of new mail */
 	if (check == M_REOPENED)
 	  mutt_error _("Mailbox was externally modified.  Flags may be wrong.");
-	else
+	else if (check == M_NEW_MAIL)
 	{
 	  mutt_message _("New mail in this mailbox.");
 	  if (option (OPTBEEPNEW))
 	    beep ();
-	}
+	} else if (check == M_FLAGS)
+	  mutt_message _("Mailbox was externally modified.");
+
 	/* avoid the message being overwritten by buffy */
 	do_buffy_notify = 0;
 	
@@ -472,6 +513,7 @@ int mutt_index_menu (void)
     }
 
 #ifdef USE_IMAP
+    imap_keepalive ();
     imap_disallow_reopen (Context);
 #endif
 
@@ -501,7 +543,7 @@ int mutt_index_menu (void)
 
     if (menu->menu == MENU_MAIN)
     {
-      if (Context)
+      if (Context && Context->hdrs && !(menu->current >= Context->vcount))
       {
 	menu_check_recenter (menu);
 
@@ -521,7 +563,7 @@ int mutt_index_menu (void)
 	menu_status_line (buf, sizeof (buf), menu, NONULL (Status));
 	CLEARLINE (option (OPTSTATUSONTOP) ? 0 : LINES-2);
 	SETCOLOR (MT_COLOR_STATUS);
-	printw ("%-*.*s", COLS, COLS, buf);
+	mutt_paddstr (COLS, buf);
 	SETCOLOR (MT_COLOR_NORMAL);
 	menu->redraw &= ~REDRAW_STATUS;
       }
@@ -551,6 +593,11 @@ int mutt_index_menu (void)
 	menu->menu = MENU_MAIN;
 	SigWinch = 0;
 	menu->top = 0; /* so we scroll the right amount */
+	/*
+	 * force a real complete redraw.  clrtobot() doesn't seem to be able
+	 * to handle every case without this.
+	 */
+	clearok(stdscr,TRUE);
 	continue;
       }
 #endif
@@ -591,10 +638,51 @@ int mutt_index_menu (void)
       else if (option (OPTAUTOTAG) && Context && Context->tagged)
 	tag = 1;
 
+      if (op == OP_TAG_PREFIX_COND)
+      {
+	if (!Context)
+	{
+	  mutt_error _("No mailbox is open.");
+	  continue;
+	}
+
+	if (!Context->tagged)
+	{
+	  event_t tmp;
+	  while(UngetCount>0)
+	  {
+	    tmp=mutt_getch();
+	    if(tmp.op==OP_END_COND)break;
+	  }
+	  mutt_message  _("Nothing to do.");
+	  continue;
+	}
+	tag = 1;
+
+	/* give visual indication that the next command is a tag- command */
+	mvaddstr (LINES - 1, 0, "tag-");
+	clrtoeol ();
+
+	/* get the real command */
+	if ((op = km_dokey (MENU_MAIN)) == OP_TAG_PREFIX)
+	{
+	  /* abort tag sequence */
+	  CLEARLINE (LINES-1);
+	  continue;
+	}
+      }
+
       mutt_clear_error ();
     }
     else
+    {
+      if (menu->current < menu->max)
+	menu->oldcurrent = menu->current;
+      else
+	menu->oldcurrent = -1;
+      
       mutt_curs_set (1);	/* fallback from the pager */
+    }
 
     switch (op)
     {
@@ -649,6 +737,7 @@ int mutt_index_menu (void)
       case OP_JUMP:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
         if (isdigit (LastKey)) mutt_ungetch (LastKey, 0);
 	buf[0] = 0;
 	if (mutt_get_field (_("Jump to message: "), buf, sizeof (buf), 0) != 0
@@ -704,7 +793,13 @@ int mutt_index_menu (void)
       case OP_MAIN_DELETE_PATTERN:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_READONLY;
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	CHECK_ATTACH;
 	mutt_pattern_func (M_DELETE, _("Delete messages matching: "));
 	menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
@@ -714,7 +809,7 @@ int mutt_index_menu (void)
       case OP_MAIN_FETCH_MAIL:
 
 	CHECK_ATTACH;
-	mutt_fetchPopMail ();
+	pop_fetch_mail ();
 	menu->redraw = REDRAW_FULL;
 	break;
 #endif /* USE_POP */
@@ -741,7 +836,7 @@ int mutt_index_menu (void)
       case OP_MAIN_LIMIT:
 
 	CHECK_MSGCOUNT;
-	menu->oldcurrent = (Context->vcount && menu->current < Context->vcount) ?
+	menu->oldcurrent = (Context->vcount && menu->current >= 0 && menu->current < Context->vcount) ?
 		CURHDR->index : -1;
 	if (mutt_pattern_func (M_LIMIT, _("Limit to messages matching: ")) == 0)
 	{
@@ -760,7 +855,9 @@ int mutt_index_menu (void)
 	  else
 	    menu->current = 0;
 	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	  mutt_linearize_tree (Context, 0);
+	  if ((Sort & SORT_MASK) == SORT_THREADS)
+	    mutt_draw_tree (Context);
+	  menu->redraw = REDRAW_FULL;
 	}
 	break;	  
 
@@ -777,11 +874,6 @@ int mutt_index_menu (void)
 	{
 	  int check;
 	  
-#ifdef USE_IMAP
-          /* logout gracefully from the IMAP server */
-          if (Context && Context->magic == M_IMAP)
-            imap_set_logout (Context);
-#endif
 	  oldcount = Context ? Context->msgcount : 0;
 
 	  if (!Context || (check = mx_close_mailbox (Context, &index_hint)) == 0)
@@ -809,6 +901,7 @@ int mutt_index_menu (void)
       case OP_SEARCH_OPPOSITE:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	if ((menu->current = mutt_search_command (menu->current, op)) == -1)
 	  menu->current = menu->oldcurrent;
 	else
@@ -828,9 +921,78 @@ int mutt_index_menu (void)
 	}
 	break;
 
-      case OP_MAIN_SYNC_FOLDER:
+      case OP_TAG:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
+	if (tag && !option (OPTAUTOTAG))
+	{
+	  for (j = 0; j < Context->vcount; j++)
+	    mutt_set_flag (Context, Context->hdrs[Context->v2r[j]], M_TAG, 0);
+	  menu->redraw = REDRAW_STATUS | REDRAW_INDEX;
+	}
+	else
+	{
+	  mutt_set_flag (Context, CURHDR, M_TAG, !CURHDR->tagged);
+	  menu->redraw = REDRAW_STATUS;
+	  if (option (OPTRESOLVE) && menu->current < Context->vcount - 1)
+	  {
+	    menu->current++;
+	    menu->redraw |= REDRAW_MOTION_RESYNCH;
+	  }
+	  else
+	    menu->redraw |= REDRAW_CURRENT;
+	}
+	break;
+
+      case OP_MAIN_TAG_PATTERN:
+
+	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
+	mutt_pattern_func (M_TAG, _("Tag messages matching: "));
+	menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	break;
+
+      case OP_MAIN_UNDELETE_PATTERN:
+
+	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
+	CHECK_READONLY;
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
+	if (mutt_pattern_func (M_UNDELETE, _("Undelete messages matching: ")) == 0)
+	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	break;
+
+      case OP_MAIN_UNTAG_PATTERN:
+
+	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
+	if (mutt_pattern_func (M_UNTAG, _("Untag messages matching: ")) == 0)
+	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
+	break;
+
+	/* --------------------------------------------------------------------
+	 * The following operations can be performed inside of the pager.
+	 */
+
+#ifdef USE_IMAP
+      case OP_MAIN_IMAP_FETCH:
+	if (Context->magic == M_IMAP)
+	  imap_check_mailbox (Context, &index_hint, 1);
+        break;
+#endif
+      
+      case OP_MAIN_SYNC_FOLDER:
+
+	if (Context && !Context->msgcount)
+	  break;
+
+	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_READONLY;
 	{
 	  int oldvcount = Context->vcount;
@@ -866,59 +1028,18 @@ int mutt_index_menu (void)
 
 	/* check for a fatal error, or all messages deleted */
 	if (!Context->path)
-	  safe_free ((void **) &Context);
-	menu->redraw = REDRAW_FULL;
-	break;
+	  FREE (&Context);
 
-      case OP_TAG:
-
-	CHECK_MSGCOUNT;
-	if (tag && !option (OPTAUTOTAG))
+	/* if we were in the pager, redisplay the message */
+	if (menu->menu == MENU_PAGER)
 	{
-	  for (j = 0; j < Context->vcount; j++)
-	    mutt_set_flag (Context, Context->hdrs[Context->v2r[j]], M_TAG, 0);
-	  menu->redraw = REDRAW_STATUS | REDRAW_INDEX;
-	}
-	else
-	{
-	  mutt_set_flag (Context, CURHDR, M_TAG, !CURHDR->tagged);
-	  menu->redraw = REDRAW_STATUS;
-	  if (option (OPTRESOLVE) && menu->current < Context->vcount - 1)
-	  {
-	    menu->current++;
-	    menu->redraw |= REDRAW_MOTION_RESYNCH;
-	  }
-	  else
-	    menu->redraw |= REDRAW_CURRENT;
+	  op = OP_DISPLAY_MESSAGE;
+	  continue;
 	}
+        else
+	  menu->redraw = REDRAW_FULL;
 	break;
 
-      case OP_MAIN_TAG_PATTERN:
-
-	CHECK_MSGCOUNT;
-	mutt_pattern_func (M_TAG, _("Tag messages matching: "));
-	menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	break;
-
-      case OP_MAIN_UNDELETE_PATTERN:
-
-	CHECK_MSGCOUNT;
-	CHECK_READONLY;
-	if (mutt_pattern_func (M_UNDELETE, _("Undelete messages matching: ")) == 0)
-	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	break;
-
-      case OP_MAIN_UNTAG_PATTERN:
-
-	CHECK_MSGCOUNT;
-	if (mutt_pattern_func (M_UNTAG, _("Untag messages matching: ")) == 0)
-	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	break;
-
-	/* --------------------------------------------------------------------
-	 * The following operations can be performed inside of the pager.
-	 */
-
       case OP_MAIN_CHANGE_FOLDER:
       
 	if (attach_msg)
@@ -934,7 +1055,7 @@ int mutt_index_menu (void)
           cp = _("Open mailbox");
 
 	buf[0] = '\0';
-	mutt_buffy (buf);
+	mutt_buffy (buf, sizeof (buf));
 
 	if (mutt_enter_fname (cp, buf, sizeof (buf), &menu->redraw, 1) == -1)
 	  break;
@@ -967,11 +1088,17 @@ int mutt_index_menu (void)
 	    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 	    break;
 	  }
-	  safe_free ((void **) &Context);
+	  FREE (&Context);
 	}
 
-	sleep (1); /* give a second to read the mailbox status */
+        mutt_sleep (0);
+      
+	/* Set CurrentMenu to MENU_MAIN before executing any folder
+	 * hooks so that all the index menu functions are available to
+	 * the exec command.
+	 */
 
+	CurrentMenu = MENU_MAIN;
 	mutt_folder_hook (buf);
 
 	if ((Context = mx_open_mailbox (buf, 
@@ -994,6 +1121,7 @@ int mutt_index_menu (void)
       case OP_DISPLAY_HEADERS: /* don't weed the headers */
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	/*
 	 * toggle the weeding of headers so that a user can press the key
 	 * again while reading the message.
@@ -1011,6 +1139,8 @@ int mutt_index_menu (void)
 	    menu->current = mutt_thread_next_unread (Context, CURHDR);
 	}
  
+	if (option (OPTPGPAUTODEC) && (tag || !(CURHDR->security & PGP_TRADITIONAL_CHECKED))) 
+	  mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
 	if ((op = mutt_display_message (CURHDR)) == -1)
 	{
 	  unset_option (OPTNEEDRESORT);
@@ -1036,12 +1166,8 @@ int mutt_index_menu (void)
 	{
 	  if (Context)
 	  {
-#ifdef USE_IMAP
-            if (Context->magic == M_IMAP)
-              imap_set_logout (Context);
-#endif
 	    mx_fastclose_mailbox (Context);
-	    safe_free ((void **) &Context);
+	    FREE (&Context);
 	  }
 	  done = 1;
 	}
@@ -1050,8 +1176,9 @@ int mutt_index_menu (void)
       case OP_EDIT_TYPE:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_ATTACH;
-	mutt_edit_content_type (CURHDR, CURHDR->content);
+	mutt_edit_content_type (CURHDR, CURHDR->content, NULL);
 	/* if we were in the pager, redisplay the message */
 	if (menu->menu == MENU_PAGER)
 	{
@@ -1065,6 +1192,7 @@ int mutt_index_menu (void)
       case OP_MAIN_NEXT_UNDELETED:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	if (menu->current >= Context->vcount - 1)
 	{
 	  if (menu->menu == MENU_MAIN)
@@ -1089,6 +1217,7 @@ int mutt_index_menu (void)
       case OP_NEXT_ENTRY:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	if (menu->current >= Context->vcount - 1)
 	{
 	  if (menu->menu == MENU_MAIN)
@@ -1108,6 +1237,7 @@ int mutt_index_menu (void)
       case OP_MAIN_PREV_UNDELETED:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	if (menu->current < 1)
 	{
 	  mutt_error _("You are on the first message.");
@@ -1131,6 +1261,7 @@ int mutt_index_menu (void)
       case OP_PREV_ENTRY:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	if (menu->current < 1)
 	{
 	  if (menu->menu == MENU_MAIN) mutt_error _("You are on the first message.");
@@ -1146,31 +1277,26 @@ int mutt_index_menu (void)
 	  menu->redraw = REDRAW_MOTION;
 	break;
 
+      case OP_DECRYPT_COPY:
+      case OP_DECRYPT_SAVE:
+        if (!WithCrypto)
+          break;   
+        /* fall thru */
       case OP_COPY_MESSAGE:
       case OP_SAVE:
       case OP_DECODE_COPY:
       case OP_DECODE_SAVE:
-#ifdef HAVE_PGP
-      case OP_DECRYPT_COPY:
-      case OP_DECRYPT_SAVE:
-#endif
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
         if (mutt_save_message (tag ? NULL : CURHDR,
-#ifdef HAVE_PGP
 			       (op == OP_DECRYPT_SAVE) ||
-#endif
 			       (op == OP_SAVE) || (op == OP_DECODE_SAVE),
 			       (op == OP_DECODE_SAVE) || (op == OP_DECODE_COPY),
-#ifdef HAVE_PGP
 			       (op == OP_DECRYPT_SAVE) || (op == OP_DECRYPT_COPY) ||
-#endif
 			       0,
 			       &menu->redraw) == 0 &&
-	    (op == OP_SAVE || op == OP_DECODE_SAVE
-#ifdef HAVE_PGP
-	     || op == OP_DECRYPT_SAVE
-#endif
-	     ))
+	     (op == OP_SAVE || op == OP_DECODE_SAVE || op == OP_DECRYPT_SAVE)
+	    )
 	{
 	  if (tag)
 	    menu->redraw |= REDRAW_INDEX;
@@ -1193,14 +1319,22 @@ int mutt_index_menu (void)
       case OP_MAIN_NEXT_UNREAD:
       case OP_MAIN_PREV_NEW:
       case OP_MAIN_PREV_UNREAD:
+      case OP_MAIN_NEXT_NEW_THEN_UNREAD:
+      case OP_MAIN_PREV_NEW_THEN_UNREAD:
 
+      {
+	int first_unread = -1;
+	int first_new    = -1;
+	
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
+
 	i = menu->current;
 	menu->current = -1;
 	for (j = 0; j != Context->vcount; j++)
 	{
-#define CUR Context->hdrs[Context->v2r[i]]
-	  if (op == OP_MAIN_NEXT_NEW || op == OP_MAIN_NEXT_UNREAD)
+#define CURHDRi Context->hdrs[Context->v2r[i]] 
+	  if (op == OP_MAIN_NEXT_NEW || op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_NEXT_NEW_THEN_UNREAD)
 	  {
 	    i++;
 	    if (i > Context->vcount - 1)
@@ -1219,31 +1353,39 @@ int mutt_index_menu (void)
 	    }
 	  }
 
-	  if (CUR->collapsed && (Sort & SORT_MASK) == SORT_THREADS)
+	  if (CURHDRi->collapsed && (Sort & SORT_MASK) == SORT_THREADS)
 	  {
-	    if ((op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD) &&
-		UNREAD (CUR))
-	    {
-	      menu->current = i;
-	      break;
-	    }
-	    if ((op == OP_MAIN_NEXT_NEW || op == OP_MAIN_PREV_NEW) &&
-		UNREAD (CUR) == 1)
-	    {
-	      menu->current = i;
-	      break;
-	    }
+	    if (UNREAD (CURHDRi) && first_unread == -1)
+	      first_unread = i;
+	    if (UNREAD (CURHDRi) == 1 && first_new == -1)
+	      first_new = i;
 	  }
-	  else if ((!CUR->deleted && !CUR->read))
+	  else if ((!CURHDRi->deleted && !CURHDRi->read))
 	  {
-	    if (op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD || !CUR->old)
-	    {
-	      menu->current = i;
-	      break;
-	    }
+	    if (first_unread == -1)
+	      first_unread = i;
+	    if ((!CURHDRi->old) && first_new == -1)
+	      first_new = i;
 	  }
+	  
+	  if ((op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD) &&
+	      first_unread != -1)
+	    break;
+	  if ((op == OP_MAIN_NEXT_NEW || op == OP_MAIN_PREV_NEW ||
+	       op == OP_MAIN_NEXT_NEW_THEN_UNREAD || op == OP_MAIN_PREV_NEW_THEN_UNREAD)
+	      && first_new != -1)
+	    break;
 	}
-#undef CUR
+#undef CURHDRi
+	if ((op == OP_MAIN_NEXT_NEW || op == OP_MAIN_PREV_NEW ||
+	     op == OP_MAIN_NEXT_NEW_THEN_UNREAD || op == OP_MAIN_PREV_NEW_THEN_UNREAD) 
+	    && first_new != -1)
+	  menu->current = first_new;
+	else if ((op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD ||
+		  op == OP_MAIN_NEXT_NEW_THEN_UNREAD || op == OP_MAIN_PREV_NEW_THEN_UNREAD)
+		 && first_unread != -1)
+	  menu->current = first_unread;
+
 	if (menu->current == -1)
 	{
 	  menu->current = menu->oldcurrent;
@@ -1258,12 +1400,26 @@ int mutt_index_menu (void)
 	else
 	  menu->redraw = REDRAW_MOTION;
 	break;
-
+      }
       case OP_FLAG_MESSAGE:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_READONLY;
 
+#ifdef USE_POP
+	if (Context->magic == M_POP)
+	{
+	  mutt_flushinp ();
+	  mutt_error _("Can't change 'important' flag on POP server.");
+	  break;
+	}
+#endif
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_WRITE);
+#endif
+
         if (tag)
         {
 	  for (j = 0; j < Context->vcount; j++)
@@ -1297,7 +1453,13 @@ int mutt_index_menu (void)
       case OP_TOGGLE_NEW:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_READONLY;
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_SEEN);
+#endif
+
 	if (tag)
 	{
 	  for (j = 0; j < Context->vcount; j++)
@@ -1349,6 +1511,7 @@ int mutt_index_menu (void)
       case OP_MAIN_PREV_SUBTHREAD:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	switch (op)
 	{
 	  case OP_MAIN_NEXT_THREAD:
@@ -1388,6 +1551,7 @@ int mutt_index_menu (void)
       case OP_MAIN_PARENT_MESSAGE:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 
 	if ((menu->current = mutt_parent_message (Context, CURHDR)) < 0)
 	{
@@ -1406,7 +1570,13 @@ int mutt_index_menu (void)
       case OP_MAIN_CLEAR_FLAG:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_READONLY;
+	
+/* #ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_WRITE);
+#endif */
+
 	if (mutt_change_flag (tag ? NULL : CURHDR, (op == OP_MAIN_SET_FLAG)) == 0)
 	{
 	  menu->redraw = REDRAW_STATUS;
@@ -1429,7 +1599,14 @@ int mutt_index_menu (void)
 
       case OP_MAIN_COLLAPSE_THREAD:
 	CHECK_MSGCOUNT;
-	
+        CHECK_VISIBLE;
+
+        if ((Sort & SORT_MASK) != SORT_THREADS)
+        {
+	  mutt_error _("Threading is not enabled.");
+	  break;
+	}
+      
 	if (CURHDR->collapsed)
 	{
 	  menu->current = mutt_uncollapse_thread (Context, CURHDR);
@@ -1454,6 +1631,7 @@ int mutt_index_menu (void)
 
       case OP_MAIN_COLLAPSE_ALL:
         CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 
         if ((Sort & SORT_MASK) != SORT_THREADS)
         {
@@ -1463,6 +1641,7 @@ int mutt_index_menu (void)
 
         {
 	  HEADER *h, *base;
+	  THREAD *thread, *top;
 	  int final;
 	  
 	  if (CURHDR->collapsed)
@@ -1474,10 +1653,14 @@ int mutt_index_menu (void)
 	  
 	  base = Context->hdrs[Context->v2r[final]];
 	  
-	  h = Context->tree;
+	  top = Context->tree;
 	  Context->collapsed = !Context->collapsed;
-	  while (h)
+	  while ((thread = top) != NULL)
 	  {
+	    while (!thread->message)
+	      thread = thread->child;
+	    h = thread->message;
+
 	    if (h->collapsed != Context->collapsed)
 	    {
 	      if (h->collapsed)
@@ -1485,7 +1668,7 @@ int mutt_index_menu (void)
 	      else if (option (OPTCOLLAPSEUNREAD) || !UNREAD (h))
 		mutt_collapse_thread (Context, h);
 	    }
-	    h = h->next;
+	    top = top->next;
 	  }
 	  
 	  mutt_set_virtual (Context);
@@ -1499,8 +1682,8 @@ int mutt_index_menu (void)
 	  }
 	  
 	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	  break;
 	}
+	break;
       
       /* --------------------------------------------------------------------
        * These functions are invoked directly from the internal-pager
@@ -1510,13 +1693,15 @@ int mutt_index_menu (void)
 
 	CHECK_ATTACH;
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	ci_bounce_message (tag ? NULL : CURHDR, &menu->redraw);
 	break;
 
       case OP_CREATE_ALIAS:
 
-	mutt_create_alias (Context && Context->vcount ? CURHDR->env : NULL, NULL);
+        mutt_create_alias (Context && Context->vcount ? CURHDR->env : NULL, NULL);
 	MAYBE_REDRAW (menu->redraw);
+        menu->redraw |= REDRAW_CURRENT;
 	break;
 
       case OP_QUERY:
@@ -1528,7 +1713,13 @@ int mutt_index_menu (void)
       case OP_DELETE:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_READONLY;
+	
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	if (tag)
 	{
 	  mutt_tag_set_flag (M_DELETE, 1);
@@ -1566,8 +1757,13 @@ int mutt_index_menu (void)
       case OP_DELETE_SUBTHREAD:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_READONLY;
 
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	rc = mutt_thread_set_flag (CURHDR, M_DELETE, 1,
 				   op == OP_DELETE_THREAD ? 0 : 1);
 
@@ -1586,6 +1782,7 @@ int mutt_index_menu (void)
       case OP_DISPLAY_ADDRESS:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	mutt_display_address (CURHDR->env);
 	break;
 
@@ -1603,9 +1800,25 @@ int mutt_index_menu (void)
       case OP_EDIT_MESSAGE:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_READONLY;
 	CHECK_ATTACH;
 
+#ifdef USE_POP
+	if (Context->magic == M_POP)
+	{
+	  mutt_flushinp ();
+	  mutt_error _("Can't edit message on POP server.");
+	  break;
+	}
+#endif
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_INSERT);
+#endif
+
+	if (option (OPTPGPAUTODEC) && (tag || !(CURHDR->security & PGP_TRADITIONAL_CHECKED))) 
+	  mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
         mutt_edit_message (Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
 
@@ -1614,26 +1827,26 @@ int mutt_index_menu (void)
       case OP_FORWARD_MESSAGE:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_ATTACH;
+	if (option (OPTPGPAUTODEC) && (tag || !(CURHDR->security & PGP_TRADITIONAL_CHECKED))) 
+	  mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
 	ci_send_message (SENDFORWARD, NULL, NULL, Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
 	break;
 
 
-
-#ifdef HAVE_PGP
       case OP_FORGET_PASSPHRASE:
-
-	mutt_forget_passphrase ();
+	crypt_forget_passphrase ();
 	break;
-#endif /* HAVE_PGP */
-
-
 
       case OP_GROUP_REPLY:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_ATTACH;
+	if (option (OPTPGPAUTODEC) && (tag || !(CURHDR->security & PGP_TRADITIONAL_CHECKED))) 
+	  mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
 	ci_send_message (SENDREPLY|SENDGROUPREPLY, NULL, NULL, Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
 	break;
@@ -1642,6 +1855,9 @@ int mutt_index_menu (void)
 
 	CHECK_ATTACH;
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
+	if (option (OPTPGPAUTODEC) && (tag || !(CURHDR->security & PGP_TRADITIONAL_CHECKED))) 
+	  mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
 	ci_send_message (SENDREPLY|SENDLISTREPLY, NULL, NULL, Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
 	break;
@@ -1653,38 +1869,44 @@ int mutt_index_menu (void)
 	menu->redraw = REDRAW_FULL;
 	break;
 
-
-
-
-
-
-
-#ifdef HAVE_PGP
       case OP_MAIL_KEY:
-	
+        if (!(WithCrypto & APPLICATION_PGP))
+          break;
 	CHECK_ATTACH;
 	ci_send_message (SENDKEY, NULL, NULL, NULL, NULL);
 	menu->redraw = REDRAW_FULL;
 	break;
+
       
       case OP_EXTRACT_KEYS:
-      
+        if (!WithCrypto)
+          break;
         CHECK_MSGCOUNT;
-        pgp_extract_keys_from_messages(tag ? NULL : CURHDR);
+        CHECK_VISIBLE;
+        crypt_extract_keys_from_messages(tag ? NULL : CURHDR);
         menu->redraw = REDRAW_FULL;
         break;
+
+
+      case OP_CHECK_TRADITIONAL:
+        if (!(WithCrypto & APPLICATION_PGP))
+          break;
+        CHECK_MSGCOUNT; 
+        CHECK_VISIBLE;
+        if (tag || !(CURHDR->security & PGP_TRADITIONAL_CHECKED)) 
+	  mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
+      
+        if (menu->menu == MENU_PAGER)
+        {
+	  op = OP_DISPLAY_MESSAGE;
+	  continue;
+	}
+        break;
       
-#endif /* HAVE_PGP */
-
-
-
-
-
-
-
       case OP_PIPE:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	mutt_pipe_message (tag ? NULL : CURHDR);
 	MAYBE_REDRAW (menu->redraw);
 	break;
@@ -1692,6 +1914,7 @@ int mutt_index_menu (void)
       case OP_PRINT:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	mutt_print_message (tag ? NULL : CURHDR);
 	break;
 
@@ -1699,8 +1922,13 @@ int mutt_index_menu (void)
       case OP_MAIN_READ_SUBTHREAD:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_READONLY;
 
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_SEEN);
+#endif
+
 	rc = mutt_thread_set_flag (CURHDR, M_READ, 1,
 				   op == OP_MAIN_READ_THREAD ? 0 : 1);
 
@@ -1708,8 +1936,9 @@ int mutt_index_menu (void)
 	{
 	  if (option (OPTRESOLVE))
 	  {
-	    if ((menu->oldcurrent = ci_next_undeleted (menu->current)) == -1)
-	      menu->oldcurrent = menu->current;
+	    if ((menu->current = (op == OP_MAIN_READ_THREAD ? 
+				  mutt_next_thread (CURHDR) : mutt_next_subthread (CURHDR))) == -1)
+	      menu->current = menu->oldcurrent;
 	  }
 	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 	}
@@ -1726,6 +1955,7 @@ int mutt_index_menu (void)
       
         CHECK_ATTACH;
         CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
       
         if (tag)
         {
@@ -1745,6 +1975,9 @@ int mutt_index_menu (void)
 
 	CHECK_ATTACH;
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
+	if (option (OPTPGPAUTODEC) && (tag || !(CURHDR->security & PGP_TRADITIONAL_CHECKED))) 
+	  mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
 	ci_send_message (SENDREPLY, NULL, NULL, Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
 	break;
@@ -1759,6 +1992,7 @@ int mutt_index_menu (void)
       case OP_TAG_SUBTHREAD:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	rc = mutt_thread_set_flag (CURHDR, M_TAG, !CURHDR->tagged,
 				   op == OP_TAG_THREAD ? 0 : 1);
 	
@@ -1778,7 +2012,13 @@ int mutt_index_menu (void)
       case OP_UNDELETE:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_READONLY;
+
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+	
 	if (tag)
 	{
 	  mutt_tag_set_flag (M_DELETE, 0);
@@ -1802,8 +2042,13 @@ int mutt_index_menu (void)
       case OP_UNDELETE_SUBTHREAD:
 
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	CHECK_READONLY;
 
+#ifdef USE_IMAP
+CHECK_IMAP_ACL(IMAP_ACL_DELETE);
+#endif
+
 	rc = mutt_thread_set_flag (CURHDR, M_DELETE, 0,
 				   op == OP_UNDELETE_THREAD ? 0 : 1);
 
@@ -1827,14 +2072,26 @@ int mutt_index_menu (void)
 	mutt_version ();
 	break;
 
+      case OP_BUFFY_LIST:
+	mutt_buffy_list ();
+	break;
+
       case OP_VIEW_ATTACHMENTS:
 	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
 	mutt_view_attachments (CURHDR);
 	if (CURHDR->attach_del)
 	  Context->changed = 1;
 	menu->redraw = REDRAW_FULL;
 	break;
 
+      case OP_END_COND:
+	break;
+
+      case OP_WHAT_KEY:
+	mutt_what_key();
+	break;
+
       default:
 	if (menu->menu == MENU_MAIN)
 	  km_error_key (MENU_MAIN);
@@ -1852,6 +2109,12 @@ int mutt_index_menu (void)
     if (done) break;
   }
 
+#ifdef USE_IMAP
+  /* Close all open IMAP connections */
+  if (!attach_msg)
+    imap_logout_all ();
+#endif
+
   mutt_menuDestroy (&menu);
   return (close);
 }
