@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,10 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include <string.h>
 
@@ -101,33 +105,66 @@ time_t mutt_mktime (struct tm *t, int local)
   return (g);
 }
 
+/* Return 1 if month is February of leap year, else 0 */
+static int isLeapYearFeb (struct tm *tm)
+{
+  if (tm->tm_mon == 1)
+  {
+    int y = tm->tm_year + 1900;
+    return (((y & 3) == 0) && (((y % 100) != 0) || ((y % 400) == 0)));
+  }
+  return (0);
+}
+
 void mutt_normalize_time (struct tm *tm)
 {
   static char DaysPerMonth[12] = {
     31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
   };
+  int nLeap;
 
   while (tm->tm_sec < 0)
   {
     tm->tm_sec += 60;
     tm->tm_min--;
   }
+  while (tm->tm_sec >= 60)
+  {
+    tm->tm_sec -= 60;
+    tm->tm_min++;
+  }
   while (tm->tm_min < 0)
   {
     tm->tm_min += 60;
     tm->tm_hour--;
   }
+  while (tm->tm_min >= 60)
+  {
+    tm->tm_min -= 60;
+    tm->tm_hour++;
+  }
   while (tm->tm_hour < 0)
   {
     tm->tm_hour += 24;
     tm->tm_mday--;
   }
+  while (tm->tm_hour >= 24)
+  {
+    tm->tm_hour -= 24;
+    tm->tm_mday++;
+  }
+  /* use loops on NNNdwmy user input values? */
   while (tm->tm_mon < 0)
   {
     tm->tm_mon += 12;
     tm->tm_year--;
   }
-  while (tm->tm_mday < 0)
+  while (tm->tm_mon >= 12)
+  {
+    tm->tm_mon -= 12;
+    tm->tm_year++;
+  }
+  while (tm->tm_mday <= 0)
   {
     if (tm->tm_mon)
       tm->tm_mon--;
@@ -136,6 +173,18 @@ void mutt_normalize_time (struct tm *tm)
       tm->tm_mon = 11;
       tm->tm_year--;
     }
-    tm->tm_mday += DaysPerMonth[tm->tm_mon];
+    tm->tm_mday += DaysPerMonth[tm->tm_mon] + isLeapYearFeb (tm);
+  }
+  while (tm->tm_mday > (DaysPerMonth[tm->tm_mon] + 
+	(nLeap = isLeapYearFeb (tm))))
+  {
+    tm->tm_mday -= DaysPerMonth[tm->tm_mon] + nLeap;
+    if (tm->tm_mon < 11)
+      tm->tm_mon++;
+    else
+    {
+      tm->tm_mon = 0;
+      tm->tm_year++;
+    }
   }
 }
