@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,13 +13,18 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 /* Close approximation of the mailx(1) builtin editor for sending mail. */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_curses.h"
+#include "mutt_idna.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -33,7 +38,7 @@
  * SLcurses_waddnstr() can't take a "const char *", so this is only
  * declared "static" (sigh)
  */
-static char EditorHelp[] = N_("\
+static char* EditorHelp = N_("\
 ~~		insert a line begining with a single ~\n\
 ~b users	add users to the Bcc: field\n\
 ~c users	add users to the Cc: field\n\
@@ -76,9 +81,12 @@ be_snarf_data (FILE *f, char **buf, int *bufmax, int *buflen, int offset,
     if (fgets (p, tmplen - 1, f) == NULL) break;
     bytes -= mutt_strlen (p);
     if (*bufmax == *buflen)
-      safe_realloc ((void **)&buf, sizeof (char *) * (*bufmax += 25));
+      safe_realloc (&buf, sizeof (char *) * (*bufmax += 25));
     buf[(*buflen)++] = safe_strdup (tmp);
   }
+  if (buf && *bufmax == *buflen) { /* Do not smash memory past buf */
+    safe_realloc (&buf, sizeof (char *) * (++*bufmax));
+  }
   if (buf) buf[*buflen] = NULL;
   return (buf);
 }
@@ -96,7 +104,7 @@ be_snarf_file (const char *path, char **buf, int *max, int *len, int verbose)
     buf = be_snarf_data (f, buf, max, len, 0, sb.st_size, 0);
     if (verbose)
     {
-      snprintf(tmp, sizeof(tmp), "\"%s\" %d bytes\n", path, sb.st_size);
+      snprintf(tmp, sizeof(tmp), "\"%s\" %lu bytes\n", path, (unsigned long) sb.st_size);
       addstr(tmp);
     }
     fclose (f);
@@ -114,7 +122,7 @@ static int be_barf_file (const char *path, char **buf, int buflen)
   FILE *f;
   int i;
   
-  if ((f = fopen (path, "w")) == NULL)
+  if ((f = fopen (path, "w")) == NULL)		/* __FOPEN_CHECKED__ */
   {
     addstr (strerror (errno));
     addch ('\n');
@@ -152,11 +160,11 @@ be_include_messages (char *msg, char **buf, int *bufmax, int *buflen,
       if (Attribution)
       {
 	mutt_make_string (tmp, sizeof (tmp) - 1, Attribution, Context, Context->hdrs[n]);
-	strcat (tmp, "\n");
+	strcat (tmp, "\n");	/* __STRCAT_CHECKED__ */
       }
 
       if (*bufmax == *buflen)
-	safe_realloc ((void **) &buf, sizeof (char *) * (*bufmax += 25));
+	safe_realloc ( &buf, sizeof (char *) * (*bufmax += 25));
       buf[(*buflen)++] = safe_strdup (tmp);
 
       bytes = Context->hdrs[n]->content->length;
@@ -171,7 +179,7 @@ be_include_messages (char *msg, char **buf, int *bufmax, int *buflen,
 			   pfx);
 
       if (*bufmax == *buflen)
-	safe_realloc ((void **)&buf, sizeof (char *) * (*bufmax += 25));
+	safe_realloc (&buf, sizeof (char *) * (*bufmax += 25));
       buf[(*buflen)++] = safe_strdup ("\n");
     }
     else
@@ -189,7 +197,7 @@ static void be_print_header (ENVELOPE *env)
   {
     addstr ("To: ");
     tmp[0] = 0;
-    rfc822_write_address (tmp, sizeof (tmp), env->to);
+    rfc822_write_address (tmp, sizeof (tmp), env->to, 1);
     addstr (tmp);
     addch ('\n');
   }
@@ -197,7 +205,7 @@ static void be_print_header (ENVELOPE *env)
   {
     addstr ("Cc: ");
     tmp[0] = 0;
-    rfc822_write_address (tmp, sizeof (tmp), env->cc);
+    rfc822_write_address (tmp, sizeof (tmp), env->cc, 1);
     addstr (tmp);
     addch ('\n');
   }
@@ -205,7 +213,7 @@ static void be_print_header (ENVELOPE *env)
   {
     addstr ("Bcc: ");
     tmp[0] = 0;
-    rfc822_write_address (tmp, sizeof (tmp), env->bcc);
+    rfc822_write_address (tmp, sizeof (tmp), env->bcc, 1);
     addstr (tmp);
     addch ('\n');
   }
@@ -229,21 +237,24 @@ static void be_edit_header (ENVELOPE *e, int force)
 
   addstr ("To: ");
   tmp[0] = 0;
-  rfc822_write_address (tmp, sizeof (tmp), e->to);
+  mutt_addrlist_to_local (e->to);
+  rfc822_write_address (tmp, sizeof (tmp), e->to, 0);
   if (!e->to || force)
   {
-    if (mutt_enter_string ((unsigned char *) tmp, sizeof (tmp), LINES-1, 4, 0) == 0)
+    if (mutt_enter_string (tmp, sizeof (tmp), LINES-1, 4, 0) == 0)
     {
       rfc822_free_address (&e->to);
       e->to = mutt_parse_adrlist (e->to, tmp);
       e->to = mutt_expand_aliases (e->to);
+      mutt_addrlist_to_idna (e->to, NULL);	/* XXX - IDNA error reporting? */
       tmp[0] = 0;
-      rfc822_write_address (tmp, sizeof (tmp), e->to);
+      rfc822_write_address (tmp, sizeof (tmp), e->to, 1);
       mvaddstr (LINES - 1, 4, tmp);
     }
   }
   else
   {
+    mutt_addrlist_to_idna (e->to, NULL);	/* XXX - IDNA error reporting? */
     addstr (tmp);
   }
   addch ('\n');
@@ -252,11 +263,8 @@ static void be_edit_header (ENVELOPE *e, int force)
   {
     addstr ("Subject: ");
     strfcpy (tmp, e->subject ? e->subject: "", sizeof (tmp));
-    if (mutt_enter_string ((unsigned char *) tmp, sizeof (tmp), LINES-1, 9, 0) == 0)
-    {
-      safe_free ((void **) &e->subject);
-      e->subject = safe_strdup (tmp);
-    }
+    if (mutt_enter_string (tmp, sizeof (tmp), LINES-1, 9, 0) == 0)
+      mutt_str_replace (&e->subject, tmp);
     addch ('\n');
   }
 
@@ -264,16 +272,20 @@ static void be_edit_header (ENVELOPE *e, int force)
   {
     addstr ("Cc: ");
     tmp[0] = 0;
-    rfc822_write_address (tmp, sizeof (tmp), e->cc);
-    if (mutt_enter_string ((unsigned char *) tmp, sizeof (tmp), LINES-1, 4, 0) == 0)
+    mutt_addrlist_to_local (e->cc);
+    rfc822_write_address (tmp, sizeof (tmp), e->cc, 0);
+    if (mutt_enter_string (tmp, sizeof (tmp), LINES-1, 4, 0) == 0)
     {
       rfc822_free_address (&e->cc);
       e->cc = mutt_parse_adrlist (e->cc, tmp);
       e->cc = mutt_expand_aliases (e->cc);
       tmp[0] = 0;
-      rfc822_write_address (tmp, sizeof (tmp), e->cc);
+      mutt_addrlist_to_idna (e->cc, NULL);
+      rfc822_write_address (tmp, sizeof (tmp), e->cc, 1);
       mvaddstr (LINES - 1, 4, tmp);
     }
+    else
+      mutt_addrlist_to_idna (e->cc, NULL);
     addch ('\n');
   }
 
@@ -281,16 +293,20 @@ static void be_edit_header (ENVELOPE *e, int force)
   {
     addstr ("Bcc: ");
     tmp[0] = 0;
-    rfc822_write_address (tmp, sizeof (tmp), e->bcc);
-    if (mutt_enter_string ((unsigned char *) tmp, sizeof (tmp), LINES-1, 5, 0) == 0)
+    mutt_addrlist_to_local (e->bcc);
+    rfc822_write_address (tmp, sizeof (tmp), e->bcc, 0);
+    if (mutt_enter_string (tmp, sizeof (tmp), LINES-1, 5, 0) == 0)
     {
       rfc822_free_address (&e->bcc);
       e->bcc = mutt_parse_adrlist (e->bcc, tmp);
       e->bcc = mutt_expand_aliases (e->bcc);
+      mutt_addrlist_to_idna (e->bcc, NULL);
       tmp[0] = 0;
-      rfc822_write_address (tmp, sizeof (tmp), e->bcc);
+      rfc822_write_address (tmp, sizeof (tmp), e->bcc, 1);
       mvaddstr (LINES - 1, 5, tmp);
     }
+    else
+      mutt_addrlist_to_idna (e->bcc, NULL);
     addch ('\n');
   }
 }
@@ -316,7 +332,7 @@ int mutt_builtin_editor (const char *path, HEADER *msg, HEADER *cur)
   tmp[0] = 0;
   while (!done)
   {
-    if (mutt_enter_string ((unsigned char *) tmp, sizeof (tmp), LINES-1, 0, 0) == -1)
+    if (mutt_enter_string (tmp, sizeof (tmp), LINES-1, 0, 0) == -1)
     {
       tmp[0] = 0;
       continue;
@@ -363,8 +379,8 @@ int mutt_builtin_editor (const char *path, HEADER *msg, HEADER *cur)
 								cur->msgno + 1);
 	    }
 	    buf = be_include_messages (p, buf, &bufmax, &buflen,
-				       (tolower (tmp[1]) == 'm'),
-				       (isupper ((unsigned char) tmp[1])));
+				       (ascii_tolower (tmp[1]) == 'm'),
+				       (ascii_isupper ((unsigned char) tmp[1])));
 	  }
 	  else
 	    addstr (_("No mailbox.\n"));
@@ -391,8 +407,7 @@ int mutt_builtin_editor (const char *path, HEADER *msg, HEADER *cur)
 	    addstr (_("missing filename.\n"));
 	  break;
 	case 's':
-	  safe_free ((void **) &msg->env->subject);
-	  msg->env->subject = safe_strdup (p);
+	  mutt_str_replace (&msg->env->subject, p);
 	  break;
 	case 't':
 	  msg->env->to = rfc822_parse_adrlist (msg->env->to, p);
@@ -416,12 +431,18 @@ int mutt_builtin_editor (const char *path, HEADER *msg, HEADER *cur)
 	case 'v':
 	  if (be_barf_file (path, buf, buflen) == 0)
 	  {
+	    char *tag, *err;
 	    be_free_memory (buf, buflen);
 	    buf = NULL;
 	    bufmax = buflen = 0;
 
 	    if (option (OPTEDITHDRS))
+	    {
+	      mutt_env_to_local (msg->env);
 	      mutt_edit_headers (NONULL(Visual), path, msg, NULL, 0);
+	      if (mutt_env_to_idna (msg->env, &tag, &err))
+		printw (_("Bad IDN in %s: '%s'\n"), tag, err);
+	    }
 	    else
 	      mutt_edit_file (NONULL(Visual), path);
 
@@ -446,9 +467,9 @@ int mutt_builtin_editor (const char *path, HEADER *msg, HEADER *cur)
       done = 1;
     else
     {
-      strncat (tmp, "\n", sizeof(tmp)); tmp[sizeof(tmp) - 1] = '\0';
+      safe_strcat (tmp, sizeof (tmp), "\n");
       if (buflen == bufmax)
-	safe_realloc ((void **)&buf, sizeof (char *) * (bufmax += 25));
+	safe_realloc (&buf, sizeof (char *) * (bufmax += 25));
       buf[buflen++] = safe_strdup (tmp[1] == '~' ? tmp + 1 : tmp);
     }
     
