@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,14 +13,23 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "sort.h"
+#include "mx.h"
 
-void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
+#ifdef USE_IMAP
+#include "imap_private.h"
+#endif
+
+void _mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf, int upd_ctx)
 {
   int changed = h->changed;
   int deleted = ctx->deleted;
@@ -32,95 +41,162 @@ void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
   switch (flag)
   {
     case M_DELETE:
+
+#ifdef USE_IMAP
+	if (ctx && ctx->magic == M_IMAP)
+		if (mutt_bit_isset (((IMAP_DATA *)ctx->data)->capabilities, ACL) \
+		&& !mutt_bit_isset(((IMAP_DATA *)ctx->data)->rights,IMAP_ACL_DELETE))
+			return;
+#endif
+
       if (bf)
       {
-	if (!h->deleted)
+	if (!h->deleted && !ctx->readonly)
 	{
 	  h->deleted = 1;
-	  ctx->deleted++;
+	  if (upd_ctx) ctx->deleted++;
+#ifdef USE_IMAP
+          /* deleted messages aren't treated as changed elsewhere so that the
+           * purge-on-sync option works correctly. This isn't applicable here */
+          if (ctx && ctx->magic == M_IMAP)
+          {
+            h->changed = 1;
+	    if (upd_ctx) ctx->changed = 1;
+          }
+#endif
 	}
       }
       else if (h->deleted)
       {
 	h->deleted = 0;
-	ctx->deleted--;
+	if (upd_ctx) ctx->deleted--;
+#ifdef USE_IMAP
+        /* see my comment above */
+	if (ctx->magic == M_IMAP) 
+	{
+	  h->changed = 1;
+	  if (upd_ctx) ctx->changed = 1;
+	}
+#endif
+	/* 
+	 * If the user undeletes a message which is marked as
+	 * "trash" in the maildir folder on disk, the folder has
+	 * been changed, and is marked accordingly.  However, we do
+	 * _not_ mark the message itself changed, because trashing
+	 * is checked in specific code in the maildir folder
+	 * driver. 
+	 */
+	if (ctx->magic == M_MAILDIR && upd_ctx && h->trash)
+	  ctx->changed = 1;
       }
       break;
 
     case M_NEW:
+
+#ifdef USE_IMAP
+	if (ctx && ctx->magic == M_IMAP)
+		if (mutt_bit_isset (((IMAP_DATA *)ctx->data)->capabilities, ACL) \
+		&& !mutt_bit_isset(((IMAP_DATA *)ctx->data)->rights,IMAP_ACL_SEEN))
+			return;
+#endif
+
       if (bf)
       {
 	if (h->read || h->old)
 	{
 	  h->old = 0;
-	  ctx->new++;
+	  if (upd_ctx) ctx->new++;
 	  if (h->read)
 	  {
 	    h->read = 0;
-	    ctx->unread++;
+	    if (upd_ctx) ctx->unread++;
 	  }
 	  h->changed = 1;
-	  ctx->changed = 1;
+	  if (upd_ctx) ctx->changed = 1;
 	}
       }
       else if (!h->read)
       {
 	if (!h->old)
-	  ctx->new--;
+	  if (upd_ctx) ctx->new--;
 	h->read = 1;
-	ctx->unread--;
+	if (upd_ctx) ctx->unread--;
 	h->changed = 1;
-	ctx->changed = 1;
+	if (upd_ctx) ctx->changed = 1;
       }
       break;
 
     case M_OLD:
+
+#ifdef USE_IMAP
+	if (ctx && ctx->magic == M_IMAP)
+		if (mutt_bit_isset (((IMAP_DATA *)ctx->data)->capabilities, ACL) \
+		&& !mutt_bit_isset(((IMAP_DATA *)ctx->data)->rights,IMAP_ACL_SEEN))
+			return;
+#endif
+
       if (bf)
       {
 	if (!h->old)
 	{
 	  h->old = 1;
 	  if (!h->read)
-	    ctx->new--;
+	    if (upd_ctx) ctx->new--;
 	  h->changed = 1;
-	  ctx->changed = 1;
+	  if (upd_ctx) ctx->changed = 1;
 	}
       }
       else if (h->old)
       {
 	h->old = 0;
 	if (!h->read)
-	  ctx->new++;
+	  if (upd_ctx) ctx->new++;
 	h->changed = 1;
-	ctx->changed = 1;
+	if (upd_ctx) ctx->changed = 1;
       }
       break;
 
     case M_READ:
+
+#ifdef USE_IMAP
+	if (ctx && ctx->magic == M_IMAP)
+		if (mutt_bit_isset (((IMAP_DATA *)ctx->data)->capabilities, ACL) \
+		&& !mutt_bit_isset(((IMAP_DATA *)ctx->data)->rights,IMAP_ACL_SEEN))
+			return;
+#endif
+
       if (bf)
       {
 	if (!h->read)
 	{
 	  h->read = 1;
-	  ctx->unread--;
+	  if (upd_ctx) ctx->unread--;
 	  if (!h->old)
-	    ctx->new--;
+	    if (upd_ctx) ctx->new--;
 	  h->changed = 1;
-	  ctx->changed = 1;
+	  if (upd_ctx) ctx->changed = 1;
 	}
       }
       else if (h->read)
       {
 	h->read = 0;
-	ctx->unread++;
+	if (upd_ctx) ctx->unread++;
 	if (!h->old)
-	  ctx->new++;
+	  if (upd_ctx) ctx->new++;
 	h->changed = 1;
-	ctx->changed = 1;
+	if (upd_ctx) ctx->changed = 1;
       }
       break;
 
     case M_REPLIED:
+
+#ifdef USE_IMAP
+	if (ctx && ctx->magic == M_IMAP)
+		if (mutt_bit_isset (((IMAP_DATA *)ctx->data)->capabilities, ACL) \
+		&& !mutt_bit_isset(((IMAP_DATA *)ctx->data)->rights,IMAP_ACL_WRITE))
+			return;
+#endif
+
       if (bf)
       {
 	if (!h->replied)
@@ -129,39 +205,47 @@ void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
 	  if (!h->read)
 	  {
 	    h->read = 1;
-	    ctx->unread--;
+	    if (upd_ctx) ctx->unread--;
 	    if (!h->old)
-	      ctx->new--;
+	      if (upd_ctx) ctx->new--;
 	  }
 	  h->changed = 1;
-	  ctx->changed = 1;
+	  if (upd_ctx) ctx->changed = 1;
 	}
       }
       else if (h->replied)
       {
 	h->replied = 0;
 	h->changed = 1;
-	ctx->changed = 1;
+	if (upd_ctx) ctx->changed = 1;
       }
       break;
 
     case M_FLAG:
+
+#ifdef USE_IMAP
+	if (ctx && ctx->magic == M_IMAP)
+		if (mutt_bit_isset (((IMAP_DATA *)ctx->data)->capabilities, ACL) \
+		&& !mutt_bit_isset(((IMAP_DATA *)ctx->data)->rights,IMAP_ACL_WRITE))
+			return;
+#endif
+
       if (bf)
       {
 	if (!h->flagged)
 	{
 	  h->flagged = bf;
-	  ctx->flagged++;
+	  if (upd_ctx) ctx->flagged++;
 	  h->changed = 1;
-	  ctx->changed = 1;
+	  if (upd_ctx) ctx->changed = 1;
 	}
       }
       else if (h->flagged)
       {
 	h->flagged = 0;
-	ctx->flagged--;
+	if (upd_ctx) ctx->flagged--;
 	h->changed = 1;
-	ctx->changed = 1;
+	if (upd_ctx) ctx->changed = 1;
       }
       break;
 
@@ -171,17 +255,19 @@ void mutt_set_flag (CONTEXT *ctx, HEADER *h, int flag, int bf)
 	if (!h->tagged)
 	{
 	  h->tagged = 1;
-	  ctx->tagged++;
+	  if (upd_ctx) ctx->tagged++;
 	}
       }
       else if (h->tagged)
       {
 	h->tagged = 0;
-	ctx->tagged--;
+	if (upd_ctx) ctx->tagged--;
       }
       break;
   }
 
+  mutt_set_header_color(ctx, h);
+
   /* if the message status has changed, we need to invalidate the cached
    * search results so that any future search will match the current status
    * of this message and not what it was at the time it was last searched.
@@ -198,14 +284,13 @@ void mutt_tag_set_flag (int flag, int bf)
     if (Context->hdrs[Context->v2r[j]]->tagged)
       mutt_set_flag (Context, Context->hdrs[Context->v2r[j]], flag, bf);
 }
-
-int mutt_thread_set_flag (HEADER *cur, int flag, int bf, int subthread)
+int mutt_thread_set_flag (HEADER *hdr, int flag, int bf, int subthread)
 {
-  HEADER *start;
+  THREAD *start, *cur = hdr->thread;
   
   if ((Sort & SORT_MASK) != SORT_THREADS)
   {
-    mutt_error ("Threading is not enabled.");
+    mutt_error _("Threading is not enabled.");
     return (-1);
   }
 
@@ -214,12 +299,17 @@ int mutt_thread_set_flag (HEADER *cur, int flag, int bf, int subthread)
       cur = cur->parent;
   start = cur;
   
-  mutt_set_flag (Context, cur, flag, bf);
+  if (cur->message)
+    mutt_set_flag (Context, cur->message, flag, bf);
+
   if ((cur = cur->child) == NULL)
     return (0);
+
   FOREVER
   {
-    mutt_set_flag (Context, cur, flag, bf);
+    if (cur->message)
+      mutt_set_flag (Context, cur->message, flag, bf);
+
     if (cur->child)
       cur = cur->child;
     else if (cur->next)
@@ -241,11 +331,14 @@ int mutt_thread_set_flag (HEADER *cur, int flag, int bf, int subthread)
 int mutt_change_flag (HEADER *h, int bf)
 {
   int i, flag;
+  event_t event;
 
-  mvprintw (LINES - 1, 0, "%s flag? (D/N/O/r/*/!): ", bf ? "Set" : "Clear");
+  mvprintw (LINES - 1, 0, "%s? (D/N/O/r/*/!): ", bf ? _("Set flag") : _("Clear flag"));
   clrtoeol ();
 
-  if ((i = mutt_getch ()) == ERR)
+  event = mutt_getch();
+  i = event.ch;
+  if (i == -1)
   {
     CLEARLINE (LINES-1);
     return (-1);
