@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,9 +13,13 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 
 #include <ctype.h>
@@ -57,7 +61,7 @@ static int is_day_name (const char *s)
  * From [ <return-path> ] <weekday> <month> <day> <time> [ <timezone> ] <year>
  */
 
-time_t is_from (const char *s, char *path, size_t pathlen)
+int is_from (const char *s, char *path, size_t pathlen, time_t *tp)
 {
   struct tm tm;
   int yr;
@@ -78,31 +82,23 @@ time_t is_from (const char *s, char *path, size_t pathlen)
   {
     const char *p;
     size_t len;
+    short q = 0;
 
-    /* looks like we got the return-path, so extract it  */
-    if (*s == '"')
+    for (p = s; *p && (q || !ISSPACE (*p)); p++)
     {
-      /* sometimes we see bogus addresses like
-       *	From "/foo/bar baz/"@dumbdar.com Sat Nov 22 15:29:32 PST 1997
-       */
-      p = s;
-      p++; /* skip over the quote */
-      do
+      if (*p == '\\')
       {
-	if (!(p = strpbrk (p, "\\\"")))
+	if (*++p == '\0') 
 	  return 0;
-	if (*p == '\\')
-	  p += 2;
       }
-      while (*p != '"');
-      while (*p && !ISSPACE (*p))
-	p++;
-    }
-    else
-    {
-      if ((p = strchr (s, ' ')) == NULL)
-	return 0;
+      else if (*p == '"')
+      {
+	q = !q;
+      }
     }
+    
+    if (q || !*p) return 0;
+    
     if (path)
     {
       len = (size_t) (p - s);
@@ -110,8 +106,9 @@ time_t is_from (const char *s, char *path, size_t pathlen)
 	len = pathlen - 1;
       memcpy (path, s, len);
       path[len] = 0;
+      dprint (3, (debugfile, "is_from(): got return path: %s\n", path));
     }
-
+    
     s = p + 1;
     SKIPWS (s);
     if (!*s)
@@ -178,12 +175,13 @@ time_t is_from (const char *s, char *path, size_t pathlen)
 
   /* year */
   if (sscanf (s, "%d", &yr) != 1) return 0;
-  tm.tm_year = yr < 70 ? yr + 100 : (yr > 1900 ? yr - 1900 : yr);
-
+  tm.tm_year = yr > 1900 ? yr - 1900 : (yr < 70 ? yr + 100 : yr);
+  
   dprint (3,(debugfile, "is_from(): month=%d, day=%d, hr=%d, min=%d, sec=%d, yr=%d.\n",
 	     tm.tm_mon, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec, tm.tm_year));
 
   tm.tm_isdst = -1;
 
-  return (mutt_mktime (&tm, 0));
+  if (tp) *tp = mutt_mktime (&tm, 0);
+  return 1;
 }
