@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -51,6 +51,8 @@ struct binding_t OpGeneric[] = {
   { "half-down", 	OP_HALF_DOWN,		"]" },
   { "help",		OP_HELP,		"?" },
   { "tag-prefix",	OP_TAG_PREFIX,		";" },
+  { "tag-prefix-cond",	OP_TAG_PREFIX_COND,	NULL },
+  { "end-cond",		OP_END_COND,		NULL },
   { "shell-escape",	OP_SHELL_ESCAPE,	"!" },
   { "select-entry",	OP_GENERIC_SELECT_ENTRY,M_ENTER_S },
   { "search",		OP_SEARCH,		"/" },
@@ -60,6 +62,7 @@ struct binding_t OpGeneric[] = {
   { "current-top",      OP_CURRENT_TOP,		NULL },
   { "current-middle",   OP_CURRENT_MIDDLE,	NULL },
   { "current-bottom",   OP_CURRENT_BOTTOM,	NULL },
+  { "what-key",		OP_WHAT_KEY,		NULL },
   { NULL,		0,			NULL }
 };
 
@@ -84,6 +87,9 @@ struct binding_t OpMain[] = {
   { "group-reply",		OP_GROUP_REPLY,			"g" },
 #ifdef USE_POP
   { "fetch-mail",		OP_MAIN_FETCH_MAIL,		"G" },
+#endif
+#ifdef USE_IMAP
+  { "imap-fetch-mail",		OP_MAIN_IMAP_FETCH,		NULL },
 #endif
   { "display-toggle-weed",		OP_DISPLAY_HEADERS,		"h" },
   { "next-undeleted",		OP_MAIN_NEXT_UNDELETED,		"j" },
@@ -122,24 +128,25 @@ struct binding_t OpMain[] = {
   { "set-flag",			OP_MAIN_SET_FLAG,		"w" },
   { "clear-flag",		OP_MAIN_CLEAR_FLAG,		"W" },
   { "display-message",		OP_DISPLAY_MESSAGE,		M_ENTER_S },
+  { "buffy-list",		OP_BUFFY_LIST,			"." },
   { "sync-mailbox",		OP_MAIN_SYNC_FOLDER,		"$" },
   { "display-address",		OP_DISPLAY_ADDRESS,		"@" },
   { "pipe-message",		OP_PIPE,			"|" },
-  { "next-new",			OP_MAIN_NEXT_NEW,		"\t" },
-  { "previous-new",		OP_MAIN_PREV_NEW,		"\033\t" },
+  { "next-new",			OP_MAIN_NEXT_NEW,		NULL },
+  { "next-new-then-unread",	OP_MAIN_NEXT_NEW_THEN_UNREAD,	"\t" },
+  { "previous-new",		OP_MAIN_PREV_NEW,		NULL },
+  { "previous-new-then-unread",	OP_MAIN_PREV_NEW_THEN_UNREAD,	"\033\t" },
   { "next-unread",		OP_MAIN_NEXT_UNREAD,		NULL },
   { "previous-unread",		OP_MAIN_PREV_UNREAD,		NULL },
   { "parent-message",		OP_MAIN_PARENT_MESSAGE,		"P" },
 
 
-#ifdef HAVE_PGP
   { "extract-keys",		OP_EXTRACT_KEYS,		"\013" },
   { "forget-passphrase",	OP_FORGET_PASSPHRASE,		"\006" },
+  { "check-traditional-pgp",	OP_CHECK_TRADITIONAL,		"\033P" },
   { "mail-key",			OP_MAIL_KEY,			"\033k" },
   { "decrypt-copy",		OP_DECRYPT_COPY,		NULL },
   { "decrypt-save",		OP_DECRYPT_SAVE,		NULL },
-#endif
-
 
 
   { NULL,			0,				NULL }
@@ -160,6 +167,9 @@ struct binding_t OpPager[] = {
   { "forward-message",	OP_FORWARD_MESSAGE,		"f" },
   { "flag-message",	OP_FLAG_MESSAGE,		"F" },
   { "group-reply",	OP_GROUP_REPLY,			"g" },
+#ifdef USE_IMAP
+  { "imap-fetch-mail",  OP_MAIN_IMAP_FETCH,		NULL },
+#endif
   { "display-toggle-weed",	OP_DISPLAY_HEADERS,		"h" },
   { "next-undeleted",	OP_MAIN_NEXT_UNDELETED,		"j" },
   { "next-entry",	OP_NEXT_ENTRY,			"J" },
@@ -194,15 +204,18 @@ struct binding_t OpPager[] = {
   { "show-version",	OP_VERSION,			"V" },
   { "search-toggle",	OP_SEARCH_TOGGLE,		"\\" },
   { "display-address",	OP_DISPLAY_ADDRESS,		"@" },
-  { "next-new",		OP_MAIN_NEXT_NEW,		"\t" },
+  { "next-new",		OP_MAIN_NEXT_NEW,		NULL },
+  { "next-new-then-unread", 
+                        OP_MAIN_NEXT_NEW_THEN_UNREAD,   "\t" },
   { "pipe-message",	OP_PIPE,			"|" },
   { "help",		OP_HELP,			"?" },
   { "next-page",	OP_NEXT_PAGE,			" " },
   { "previous-page",	OP_PREV_PAGE,			"-" },
   { "top",		OP_PAGER_TOP,			"^" },
-  { "bottom",		OP_PAGER_BOTTOM,		"$" },
+  { "sync-mailbox",	OP_MAIN_SYNC_FOLDER,            "$" },
   { "shell-escape",	OP_SHELL_ESCAPE,		"!" },
   { "enter-command",	OP_ENTER_COMMAND,		":" },
+  { "buffy-list",	OP_BUFFY_LIST,			"." },
   { "search",		OP_SEARCH,			"/" },
   { "search-reverse",	OP_SEARCH_REVERSE,		"\033/" },
   { "search-opposite",	OP_SEARCH_OPPOSITE,		NULL },
@@ -210,24 +223,24 @@ struct binding_t OpPager[] = {
   { "jump",		OP_JUMP,			NULL },
   { "next-unread",	OP_MAIN_NEXT_UNREAD,		NULL },
   { "previous-new",	OP_MAIN_PREV_NEW,		NULL },
+  { "previous-new-then-unread",
+      			OP_MAIN_PREV_NEW_THEN_UNREAD,   NULL },
   { "previous-unread",	OP_MAIN_PREV_UNREAD,		NULL },
   { "half-up",		OP_HALF_UP,			NULL },
   { "half-down",	OP_HALF_DOWN,			NULL },
   { "previous-line",	OP_PREV_LINE,			NULL },
+  { "bottom",		OP_PAGER_BOTTOM,		NULL },
   { "parent-message",	OP_MAIN_PARENT_MESSAGE,		"P" },
 
 
 
 
-#ifdef HAVE_PGP
+  { "check-traditional-pgp",	OP_CHECK_TRADITIONAL,	"\033P"   },
+  { "mail-key",		OP_MAIL_KEY,			"\033k" },
   { "extract-keys",	OP_EXTRACT_KEYS,		"\013" },
   { "forget-passphrase",OP_FORGET_PASSPHRASE,		"\006" },
-  { "mail-key",		OP_MAIL_KEY,			"\033k" },
-  { "decrypt-copy",		OP_DECRYPT_COPY,		NULL },
-  { "decrypt-save",		OP_DECRYPT_SAVE,		NULL },
-#endif
-
-
+  { "decrypt-copy",	OP_DECRYPT_COPY,		NULL },
+  { "decrypt-save",    	OP_DECRYPT_SAVE,		NULL },
 
 
   { NULL,		0,				NULL }
@@ -235,7 +248,7 @@ struct binding_t OpPager[] = {
 
 struct binding_t OpAttach[] = {
   { "bounce-message",	OP_BOUNCE_MESSAGE,		"b" },
-  { "display-toggle-weed",	OP_DISPLAY_HEADERS,		"h" },
+  { "display-toggle-weed",	OP_DISPLAY_HEADERS,	"h" },
   { "edit-type",	OP_EDIT_TYPE,			"\005" },
   { "print-entry",	OP_PRINT,			"p" },
   { "save-entry",	OP_SAVE,			"s" },
@@ -250,14 +263,11 @@ struct binding_t OpAttach[] = {
   { "view-attach",	OP_VIEW_ATTACH,			M_ENTER_S },
   { "delete-entry",	OP_DELETE,			"d" },
   { "undelete-entry",	OP_UNDELETE,			"u" },
+  { "collapse-parts",	OP_ATTACH_COLLAPSE,		"v" },
 
-
-
-#ifdef HAVE_PGP
-  { "extract-keys",	OP_EXTRACT_KEYS,		"\013" },
-#endif
-
-
+  { "check-traditional-pgp",	OP_CHECK_TRADITIONAL,		"\033P"   },
+  { "extract-keys",		OP_EXTRACT_KEYS,		"\013" },
+  { "forget-passphrase",	OP_FORGET_PASSPHRASE,		"\006" },
 
   { NULL,		0,				NULL }
 };
@@ -265,7 +275,6 @@ struct binding_t OpAttach[] = {
 struct binding_t OpCompose[] = {
   { "attach-file",	OP_COMPOSE_ATTACH_FILE,		"a" },
   { "attach-message",	OP_COMPOSE_ATTACH_MESSAGE,	"A" },
-  { "change-charset",	OP_COMPOSE_CHANGE_CHARSET,	NULL },
   { "edit-bcc",		OP_COMPOSE_EDIT_BCC,		"b" },
   { "edit-cc",		OP_COMPOSE_EDIT_CC,		"c" },
   { "copy-file",	OP_SAVE,			"C" },
@@ -292,7 +301,6 @@ struct binding_t OpCompose[] = {
   { "edit-to",		OP_COMPOSE_EDIT_TO,		"t" },
   { "edit-type",	OP_EDIT_TYPE,			"\024" },
   { "write-fcc",	OP_COMPOSE_WRITE_MESSAGE,	"w" },
-  { "recode-attachment",OP_COMPOSE_RECODE,		NULL },
   { "toggle-unlink",	OP_COMPOSE_TOGGLE_UNLINK,	"u" },
   { "toggle-recode",    OP_COMPOSE_TOGGLE_RECODE,	NULL },
   { "update-encoding",	OP_COMPOSE_UPDATE_ENCODING,	"U" },
@@ -300,11 +308,12 @@ struct binding_t OpCompose[] = {
   { "send-message",	OP_COMPOSE_SEND_MESSAGE,	"y" },
   { "pipe-entry",	OP_PIPE,			"|" },
 
-#ifdef HAVE_PGP
   { "attach-key",	OP_COMPOSE_ATTACH_KEY,		"\033k" },
-  { "forget-passphrase",OP_FORGET_PASSPHRASE,		"\006"  },
   { "pgp-menu",		OP_COMPOSE_PGP_MENU,		"p" 	},
-#endif
+
+  { "forget-passphrase",OP_FORGET_PASSPHRASE,		"\006"  },
+
+  { "smime-menu",	OP_COMPOSE_SMIME_MENU,		"S" 	},
 
 #ifdef MIXMASTER
   { "mix",		OP_COMPOSE_MIX,			"M" },
@@ -319,6 +328,13 @@ struct binding_t OpPost[] = {
   { NULL,		0,		NULL }
 };
 
+struct binding_t OpAlias[] = {
+  { "delete-entry",	OP_DELETE,	"d" },
+  { "undelete-entry",	OP_UNDELETE,	"u" },
+  { NULL,		0,		NULL }
+};
+  
+
 /* The file browser */
 struct binding_t OpBrowser[] = {
   { "change-dir",	OP_CHANGE_DIRECTORY,	"c" },
@@ -330,9 +346,11 @@ struct binding_t OpBrowser[] = {
   { "check-new",	OP_CHECK_NEW,		NULL },
   { "toggle-mailboxes", OP_TOGGLE_MAILBOXES, 	"\t" },
   { "view-file",	OP_BROWSER_VIEW_FILE,	" " },
+  { "buffy-list",	OP_BUFFY_LIST,		"." },
 #ifdef USE_IMAP
-  { "new-mailbox",      OP_NEW_MAILBOX,         "n" },
+  { "create-mailbox",   OP_CREATE_MAILBOX,      "C" },
   { "delete-mailbox",   OP_DELETE_MAILBOX,      "d" },
+  { "rename-mailbox",   OP_RENAME_MAILBOX,      "r" },
   { "subscribe",	OP_BROWSER_SUBSCRIBE,	"s" },
   { "unsubscribe",	OP_BROWSER_UNSUBSCRIBE,	"u" },
   { "toggle-subscribed", OP_BROWSER_TOGGLE_LSUB, "T" },
@@ -352,31 +370,49 @@ struct binding_t OpQuery[] = {
 struct binding_t OpEditor[] = {
   { "bol",		OP_EDITOR_BOL,			"\001" },
   { "backward-char",	OP_EDITOR_BACKWARD_CHAR,	"\002" },
+  { "backward-word",	OP_EDITOR_BACKWARD_WORD,	"\033b"},
+  { "capitalize-word",	OP_EDITOR_CAPITALIZE_WORD,	"\033c"},
+  { "downcase-word",	OP_EDITOR_DOWNCASE_WORD,	"\033l"},
+  { "upcase-word",	OP_EDITOR_UPCASE_WORD,		"\033u"},
   { "delete-char",	OP_EDITOR_DELETE_CHAR,		"\004" },
   { "eol",		OP_EDITOR_EOL,			"\005" },
   { "forward-char",	OP_EDITOR_FORWARD_CHAR,		"\006" },
+  { "forward-word",	OP_EDITOR_FORWARD_WORD,		"\033f"},
   { "backspace",	OP_EDITOR_BACKSPACE,		"\010" },
   { "kill-eol",		OP_EDITOR_KILL_EOL,		"\013" },
+  { "kill-eow",		OP_EDITOR_KILL_EOW,		"\033d"},
   { "kill-line",	OP_EDITOR_KILL_LINE,		"\025" },
   { "quote-char",	OP_EDITOR_QUOTE_CHAR,		"\026" },
   { "kill-word",	OP_EDITOR_KILL_WORD,		"\027" },
-  { "complete",		OP_EDITOR_COMPLETE,		"\t" },
+  { "complete",		OP_EDITOR_COMPLETE,		"\t"   },
   { "complete-query",	OP_EDITOR_COMPLETE_QUERY,	"\024" },
-  { "buffy-cycle",	OP_EDITOR_BUFFY_CYCLE,		" " },
-  { "history-up",	OP_EDITOR_HISTORY_UP,		NULL },
-  { "history-down",	OP_EDITOR_HISTORY_DOWN,		NULL },
-  { NULL,		0,				NULL }
+  { "buffy-cycle",	OP_EDITOR_BUFFY_CYCLE,		" "    },
+  { "history-up",	OP_EDITOR_HISTORY_UP,		NULL   },
+  { "history-down",	OP_EDITOR_HISTORY_DOWN,		NULL   },
+  { "transpose-chars",	OP_EDITOR_TRANSPOSE_CHARS,	NULL   },
+  { NULL,		0,				NULL   }
 };
 
 
 
-#ifdef HAVE_PGP
 struct binding_t OpPgp[] = {
   { "verify-key",	OP_VERIFY_KEY,		"c" },
   { "view-name",	OP_VIEW_ID,		"%" },
   { NULL,		0,				NULL }
 };
-#endif /* HAVE_PGP */
+
+
+
+/* When using the GPGME based backend we have some useful functions
+   for the SMIME menu.  */
+struct binding_t OpSmime[] = {
+#ifdef CRYPT_BACKEND_GPGME
+  { "verify-key",    OP_VERIFY_KEY,             "c" },
+  { "view-name",     OP_VIEW_ID,	        "%" },
+#endif
+  { NULL,	0,	NULL }
+};
+
 
 
 #ifdef MIXMASTER
