@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,40 +13,60 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "hash.h"
+#include "mutt.h"
 
-int hash_string (unsigned char *s, int n)
+#define SOMEPRIME 149711
+
+int hash_string (const unsigned char *s, int n)
 {
   int h = 0;
+
+#if 0
   while (*s)
     h += *s++;
+#else
+  while (*s)
+    h += (h << 7) + *s++;
+  h = (h * SOMEPRIME) % n;
+  h = (h >= 0) ? h : h + n;
+#endif
+
   return (h % n);
 }
 
 HASH *hash_create (int nelem)
 {
-  HASH *table = malloc (sizeof (HASH));
+  HASH *table = safe_malloc (sizeof (HASH));
+  if (nelem == 0)
+    nelem = 2;
   table->nelem = nelem;
-  table->table = calloc (nelem, sizeof (struct hash_elem *));
+  table->table = safe_calloc (nelem, sizeof (struct hash_elem *));
   return table;
 }
 
-/* table	hash table to update
-   key		key to hash on
-   data		data to associate with `key'
-   allow_dup	if nonzero, duplicate keys are allowed in the table */
-int hash_insert (HASH *table, char *key, void *data, int allow_dup)
+/* table        hash table to update
+ * key          key to hash on
+ * data         data to associate with `key'
+ * allow_dup    if nonzero, duplicate keys are allowed in the table 
+ */
+int hash_insert (HASH * table, const char *key, void *data, int allow_dup)
 {
-  struct hash_elem *ptr = malloc (sizeof (struct hash_elem));
-  int h = hash_string ((unsigned char *) key, table->nelem);
+  struct hash_elem *ptr;
+  int h;
 
+  ptr = (struct hash_elem *) safe_malloc (sizeof (struct hash_elem));
+  h = hash_string ((unsigned char *) key, table->nelem);
   ptr->key = key;
   ptr->data = data;
 
@@ -62,10 +82,10 @@ int hash_insert (HASH *table, char *key, void *data, int allow_dup)
 
     for (tmp = table->table[h], last = NULL; tmp; last = tmp, tmp = tmp->next)
     {
-      r = strcmp (tmp->key, key);
+      r = mutt_strcmp (tmp->key, key);
       if (r == 0)
       {
-	free (ptr);
+	FREE (&ptr);
 	return (-1);
       }
       if (r > 0)
@@ -80,45 +100,42 @@ int hash_insert (HASH *table, char *key, void *data, int allow_dup)
   return h;
 }
 
-void *hash_find_hash (HASH *table, int hash, char *key)
+void *hash_find_hash (const HASH * table, int hash, const char *key)
 {
   struct hash_elem *ptr = table->table[hash];
   for (; ptr; ptr = ptr->next)
   {
-    if (strcmp (key, ptr->key) == 0)
+    if (mutt_strcmp (key, ptr->key) == 0)
       return (ptr->data);
   }
   return NULL;
 }
 
-void hash_delete_hash (HASH *table, int hash, char *key, void *data,
+void hash_delete_hash (HASH * table, int hash, const char *key, const void *data,
 		       void (*destroy) (void *))
 {
   struct hash_elem *ptr = table->table[hash];
-  struct hash_elem *last = NULL;
-  for (; ptr; last = ptr, ptr = ptr->next)
+  struct hash_elem **last = &table->table[hash];
+
+  for (; ptr; last = &ptr->next, ptr = ptr->next)
   {
     /* if `data' is given, look for a matching ->data member.  this is
-       required for the case where we have multiple entries with the same
-       key */
-    if (data == ptr->data || (!data && strcmp (ptr->key, key) == 0))
+     * required for the case where we have multiple entries with the same
+     * key
+     */
+    if ((data == ptr->data) || (!data && mutt_strcmp (ptr->key, key) == 0))
     {
-      if (last)
-	last->next = ptr->next;
-      else
-	table->table[hash] = ptr->next;
-      if (destroy)
-	destroy (ptr->data);
-      memset (ptr, 0, sizeof (struct hash_elem));
-      free (ptr);
-      ptr = 0;
+      *last = ptr->next;
+      if (destroy) destroy (ptr->data);
+      FREE (&ptr);
       return;
     }
   }
 }
 
 /* ptr		pointer to the hash table to be freed
-   destroy()	function to call to free the ->data member (optional) */
+ * destroy()	function to call to free the ->data member (optional) 
+ */
 void hash_destroy (HASH **ptr, void (*destroy) (void *))
 {
   int i;
@@ -133,10 +150,9 @@ void hash_destroy (HASH **ptr, void (*destroy) (void *))
       elem = elem->next;
       if (destroy)
 	destroy (tmp->data);
-      memset (tmp, 0, sizeof (struct hash_elem));
-      free (tmp);
+      FREE (&tmp);
     }
   }
-  free (pptr);
-  *ptr = NULL;
+  FREE (&pptr->table);
+  FREE (ptr);
 }
