@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000,2002 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,16 +13,19 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_curses.h"
-
-
-
-
-
+#include "sort.h"
+#include "charset.h"
+#include "mutt_crypt.h"
+#include "mutt_idna.h"
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -31,14 +34,16 @@
 
 int mutt_is_mail_list (ADDRESS *addr)
 {
-  LIST *p;
+  if (!mutt_match_rx_list (addr->mailbox, UnMailLists))
+    return mutt_match_rx_list (addr->mailbox, MailLists);
+  return 0;
+}
 
-  if (addr->mailbox)
-  {
-    for (p = MailLists; p; p = p->next)
-      if (strncasecmp (addr->mailbox, p->data, strlen (p->data)) == 0)
-	return 1;
-  }
+int mutt_is_subscribed_list (ADDRESS *addr)
+{
+  if (!mutt_match_rx_list (addr->mailbox, UnMailLists)
+      && !mutt_match_rx_list (addr->mailbox, UnSubscribedLists))
+    return mutt_match_rx_list (addr->mailbox, SubscribedLists);
   return 0;
 }
 
@@ -51,7 +56,7 @@ check_for_mailing_list (ADDRESS *adr, char *pfx, char *buf, int buflen)
 {
   for (; adr; adr = adr->next)
   {
-    if (mutt_is_mail_list (adr))
+    if (mutt_is_subscribed_list (adr))
     {
       if (pfx && buf && buflen)
 	snprintf (buf, buflen, "%s%s", pfx, mutt_get_name (adr));
@@ -70,7 +75,7 @@ check_for_mailing_list_addr (ADDRESS *adr, char *buf, int buflen)
 {
   for (; adr; adr = adr->next)
   {
-    if (mutt_is_mail_list (adr))
+    if (mutt_is_subscribed_list (adr))
     {
       if (buf && buflen)
 	snprintf (buf, buflen, "%s", adr->mailbox);
@@ -85,7 +90,7 @@ static int first_mailing_list (char *buf, size_t buflen, ADDRESS *a)
 {
   for (; a; a = a->next)
   {
-    if (mutt_is_mail_list (a))
+    if (mutt_is_subscribed_list (a))
     {
       mutt_save_path (buf, buflen, a);
       return 1;
@@ -142,7 +147,7 @@ static void make_from_addr (ENVELOPE *hdr, char *buf, size_t len, int do_lists)
     *buf = 0;
 }
 
-int mutt_user_is_recipient (ADDRESS *a)
+static int user_in_addr (ADDRESS *a)
 {
   for (; a; a = a->next)
     if (mutt_addr_is_user (a))
@@ -156,32 +161,48 @@ int mutt_user_is_recipient (ADDRESS *a)
  * 2: user is in the TO list
  * 3: user is in the CC list
  * 4: user is originator
+ * 5: sent to a subscribed mailinglist
  */
-static int user_is_recipient (ENVELOPE *hdr)
+int mutt_user_is_recipient (HEADER *h)
 {
-  if (mutt_addr_is_user (hdr->from))
-    return 4;
+  ENVELOPE *env = h->env;
 
-  if (mutt_user_is_recipient (hdr->to))
+  if(!h->recip_valid)
   {
-    if (hdr->to->next || hdr->cc)
-      return 2; /* non-unique recipient */
+    h->recip_valid = 1;
+    
+    if (mutt_addr_is_user (env->from))
+      h->recipient = 4;
+    else if (user_in_addr (env->to))
+    {
+      if (env->to->next || env->cc)
+	h->recipient = 2; /* non-unique recipient */
+      else
+	h->recipient = 1; /* unique recipient */
+    }
+    else if (user_in_addr (env->cc))
+      h->recipient = 3;
+    else if (check_for_mailing_list (env->to, NULL, NULL, 0))
+      h->recipient = 5;
+    else if (check_for_mailing_list (env->cc, NULL, NULL, 0))
+      h->recipient = 5;
     else
-      return 1; /* unique recipient */
+      h->recipient = 0;
   }
-
-  if (mutt_user_is_recipient (hdr->cc))
-    return 3;
-
-  return (0);
+  
+  return h->recipient;
 }
 
 /* %a = address of author
+ * %A = reply-to address (if present; otherwise: address of author
  * %b = filename of the originating folder
  * %B = the list to which the letter was sent
  * %c = size of message in bytes
  * %C = current message number
- * %d = date and time of message (using strftime)
+ * %d = date and time of message using $date_format and sender's timezone
+ * %D = date and time of message using $date_format and local timezone
+ * %e = current message number in thread
+ * %E = number of messages in current thread
  * %f = entire from line
  * %F = like %n, unless from self
  * %i = message-id
@@ -196,6 +217,9 @@ static int user_is_recipient (ENVELOPE *hdr)
  * %t = `to:' field (recipients)
  * %T = $to_chars
  * %u = user (login) name of author
+ * %v = first name of author, unless from self
+ * %y = `x-label:' field (if present)
+ * %Y = `x-label:' field (if present, tree unfolded, and != parent's x-label)
  * %Z = status flags	*/
 
 struct hdr_format_info
@@ -216,11 +240,15 @@ hdr_format_str (char *dest,
 		format_flag flags)
 {
   struct hdr_format_info *hfi = (struct hdr_format_info *) data;
-  HEADER *hdr;
+  HEADER *hdr, *htmp;
   CONTEXT *ctx;
   char fmt[SHORT_STRING], buf2[SHORT_STRING], ch, *p;
   int do_locales, i;
   int optional = (flags & M_FORMAT_OPTIONAL);
+  int threads = ((Sort & SORT_MASK) == SORT_THREADS);
+  int is_index = (flags & M_FORMAT_INDEX);
+#define THREAD_NEW (threads && hdr->collapsed && hdr->num_hidden > 1 && mutt_thread_contains_unread (ctx, hdr) == 1)
+#define THREAD_OLD (threads && hdr->collapsed && hdr->num_hidden > 1 && mutt_thread_contains_unread (ctx, hdr) == 2)
   size_t len;
 
   hdr = hfi->hdr;
@@ -229,11 +257,18 @@ hdr_format_str (char *dest,
   dest[0] = 0;
   switch (op)
   {
+    case 'A':
+      if(hdr->env->reply_to && hdr->env->reply_to->mailbox)
+      {
+	mutt_format_s (dest, destlen, prefix, mutt_addr_for_display (hdr->env->reply_to));
+	break;
+      }
+      /* fall through if 'A' returns nothing */
+
     case 'a':
       if(hdr->env->from && hdr->env->from->mailbox)
       {
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, hdr->env->from->mailbox);
+	mutt_format_s (dest, destlen, prefix, mutt_addr_for_display (hdr->env->from));
       }
       else
         dest[0] = '\0';
@@ -243,7 +278,13 @@ hdr_format_str (char *dest,
       if (!first_mailing_list (dest, destlen, hdr->env->to) &&
 	  !first_mailing_list (dest, destlen, hdr->env->cc))
 	dest[0] = 0;
-      break;
+      if (dest[0])
+      {
+	strfcpy (buf2, dest, sizeof(buf2));
+	mutt_format_s (dest, destlen, prefix, buf2);
+	break;
+      }
+      /* fall through if 'B' returns nothing */
 
     case 'b':
       if(ctx)
@@ -255,12 +296,13 @@ hdr_format_str (char *dest,
       }
       else 
 	strfcpy(dest, "(null)", destlen);
+      strfcpy (buf2, dest, sizeof(buf2));
+      mutt_format_s (dest, destlen, prefix, buf2);
       break;
     
     case 'c':
       mutt_pretty_size (buf2, sizeof (buf2), (long) hdr->content->length);
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, buf2);
+      mutt_format_s (dest, destlen, prefix, buf2);
       break;
 
     case 'C':
@@ -269,6 +311,7 @@ hdr_format_str (char *dest,
       break;
 
     case 'd':
+    case 'D':
     case '{':
     case '[':
     case '(':
@@ -282,7 +325,7 @@ hdr_format_str (char *dest,
 
 	p = dest;
 
-	cp = (op == 'd') ? (NONULL (DateFmt)) : src;
+	cp = (op == 'd' || op == 'D') ? (NONULL (DateFmt)) : src;
 	if (*cp == '!')
 	{
 	  do_locales = 0;
@@ -292,7 +335,7 @@ hdr_format_str (char *dest,
 	  do_locales = 1;
 
 	len = destlen - 1;
-	while (len > 0 && ((op == 'd' && *cp) ||
+	while (len > 0 && (((op == 'd' || op == 'D') && *cp) ||
 			   (op == '{' && *cp != '}') || 
 			   (op == '[' && *cp != ']') ||
 			   (op == '(' && *cp != ')') ||
@@ -301,11 +344,11 @@ hdr_format_str (char *dest,
 	  if (*cp == '%')
 	  {
 	    cp++;
-	    if (*cp == 'Z' && (op == 'd' || op == '{'))
+	    if ((*cp == 'Z' || *cp == 'z') && (op == 'd' || op == '{'))
 	    {
 	      if (len >= 5)
 	      {
-		sprintf (p, "%c%02d%02d", hdr->zoccident ? '-' : '+',
+		sprintf (p, "%c%02u%02u", hdr->zoccident ? '-' : '+',
 			 hdr->zhours, hdr->zminutes);
 		p += 5;
 		len -= 5;
@@ -337,7 +380,7 @@ hdr_format_str (char *dest,
 	if (do_locales && Locale)
 	  setlocale (LC_TIME, Locale);
 
-	if (op == '[')
+	if (op == '[' || op == 'D')
 	  tm = localtime (&hdr->date_sent);
 	else if (op == '(')
 	  tm = localtime (&hdr->received);
@@ -362,42 +405,74 @@ hdr_format_str (char *dest,
 	if (do_locales)
 	  setlocale (LC_TIME, "C");
 
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, buf2);
-	if (len > 0 && op != 'd')
+	mutt_format_s (dest, destlen, prefix, buf2);
+	if (len > 0 && op != 'd' && op != 'D') /* Skip ending op */
 	  src = cp + 1;
       }
       break;
 
+    case 'e':
+      snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
+      snprintf (dest, destlen, fmt, mutt_messages_in_thread(ctx, hdr, 1));
+      break;
+
+    case 'E':
+      if (!optional)
+      {
+	snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
+	snprintf (dest, destlen, fmt, mutt_messages_in_thread(ctx, hdr, 0));
+      }
+      else if (mutt_messages_in_thread(ctx, hdr, 0) <= 1)
+	optional = 0;
+      break;
+
     case 'f':
       buf2[0] = 0;
-      rfc822_write_address (buf2, sizeof (buf2), hdr->env->from);
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, buf2);
+      rfc822_write_address (buf2, sizeof (buf2), hdr->env->from, 1);
+      mutt_format_s (dest, destlen, prefix, buf2);
       break;
 
     case 'F':
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      make_from (hdr->env, buf2, sizeof (buf2), 0);
-      snprintf (dest, destlen, fmt, buf2);
+      if (!optional)
+      {
+        make_from (hdr->env, buf2, sizeof (buf2), 0);
+	mutt_format_s (dest, destlen, prefix, buf2);
+      }
+      else if (mutt_addr_is_user (hdr->env->from))
+        optional = 0;
+      break;
+
+    case 'H':
+      /* (Hormel) spam score */
+      if (optional)
+	optional = hdr->env->spam ? 1 : 0;
+
+       if (hdr->env->spam)
+         mutt_format_s (dest, destlen, prefix, NONULL (hdr->env->spam->data));
+       else
+         mutt_format_s (dest, destlen, prefix, "");
+
       break;
 
     case 'i':
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, hdr->env->message_id ? hdr->env->message_id : "<no.id>");
+      mutt_format_s (dest, destlen, prefix, hdr->env->message_id ? hdr->env->message_id : "<no.id>");
       break;
 
     case 'l':
-      snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
-      snprintf (dest, destlen, fmt, (int) hdr->lines);
+      if (!optional)
+      {
+	snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
+	snprintf (dest, destlen, fmt, (int) hdr->lines);
+      }
+      else if (hdr->lines <= 0)
+        optional = 0;
       break;
 
     case 'L':
       if (!optional)
       {
 	make_from (hdr->env, buf2, sizeof (buf2), 1);
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, buf2);
+	mutt_format_s (dest, destlen, prefix, buf2);
       }
       else if (!check_for_mailing_list (hdr->env->to, NULL, NULL, 0) &&
 	       !check_for_mailing_list (hdr->env->cc, NULL, NULL, 0))
@@ -417,13 +492,20 @@ hdr_format_str (char *dest,
       break;
 
     case 'n':
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, mutt_get_name (hdr->env->from));
+      mutt_format_s (dest, destlen, prefix, mutt_get_name (hdr->env->from));
       break;
 
     case 'N':
-      snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
-      snprintf (dest, destlen, fmt, hdr->score);
+      if (!optional)
+      {
+	snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
+	snprintf (dest, destlen, fmt, hdr->score);
+      }
+      else
+      {
+	if (hdr->score == 0)
+	  optional = 0;
+      }
       break;
 
     case 'O':
@@ -432,8 +514,7 @@ hdr_format_str (char *dest,
 	make_from_addr (hdr->env, buf2, sizeof (buf2), 1);
 	if (!option (OPTSAVEADDRESS) && (p = strpbrk (buf2, "%@")))
 	  *p = 0;
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, buf2);
+	mutt_format_s (dest, destlen, prefix, buf2);
       }
       else if (!check_for_mailing_list_addr (hdr->env->to, NULL, 0) &&
 	       !check_for_mailing_list_addr (hdr->env->cc, NULL, 0))
@@ -442,25 +523,41 @@ hdr_format_str (char *dest,
       }
       break;
 
-    case 's':
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      if (flags & M_FORMAT_TREE)
+    case 'M':
+      snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
+      if (!optional)
       {
-	if (flags & M_FORMAT_FORCESUBJ)
-	{
-	  snprintf (buf2, sizeof (buf2), "%s%s", hdr->tree,
-		    hdr->env->subject ? hdr->env->subject : "");
-	  snprintf (dest, destlen, fmt, buf2);
-	}
+	if (threads && is_index && hdr->collapsed && hdr->num_hidden > 1)
+	  snprintf (dest, destlen, fmt, hdr->num_hidden);
+	else if (is_index && threads)
+	  mutt_format_s (dest, destlen, prefix, " ");
 	else
-	  snprintf (dest, destlen, fmt, hdr->tree);
+	  *dest = '\0';
       }
       else
       {
-	snprintf (dest, destlen, fmt, hdr->env->subject ? hdr->env->subject : "");
+	if (!(threads && is_index && hdr->collapsed && hdr->num_hidden > 1))
+	  optional = 0;
       }
       break;
 
+    case 's':
+      
+      if (flags & M_FORMAT_TREE && !hdr->collapsed)
+      {
+	if (flags & M_FORMAT_FORCESUBJ)
+	{
+	  mutt_format_s (dest, destlen, "", NONULL (hdr->env->subject));
+	  snprintf (buf2, sizeof (buf2), "%s%s", hdr->tree, dest);
+	  mutt_format_s_tree (dest, destlen, prefix, buf2);
+	}
+	else
+	  mutt_format_s_tree (dest, destlen, prefix, hdr->tree);
+      }
+      else
+	mutt_format_s (dest, destlen, prefix, NONULL (hdr->env->subject));
+      break;
+
     case 'S':
       if (hdr->deleted)
 	ch = 'D';
@@ -495,50 +592,103 @@ hdr_format_str (char *dest,
 	else if (hdr->env->cc)
 	  snprintf (buf2, sizeof (buf2), "Cc %s", mutt_get_name (hdr->env->cc));
       }
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, buf2);
+      mutt_format_s (dest, destlen, prefix, buf2);
       break;
 
     case 'T':
       snprintf (fmt, sizeof (fmt), "%%%sc", prefix);
       snprintf (dest, destlen, fmt,
-		(Tochars && ((i = user_is_recipient (hdr->env))) < strlen (Tochars)) ? Tochars[i] : ' ');
+		(Tochars && ((i = mutt_user_is_recipient (hdr))) < mutt_strlen (Tochars)) ? Tochars[i] : ' ');
       break;
 
     case 'u':
       if (hdr->env->from && hdr->env->from->mailbox)
       {
-	strfcpy (buf2, hdr->env->from->mailbox, sizeof (buf2));
+	strfcpy (buf2, mutt_addr_for_display (hdr->env->from), sizeof (buf2));
 	if ((p = strpbrk (buf2, "%@")))
 	  *p = 0;
       }
       else
 	buf2[0] = 0;
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, buf2);
+      mutt_format_s (dest, destlen, prefix, buf2);
+      break;
+
+    case 'v':
+      if (mutt_addr_is_user (hdr->env->from)) 
+      {
+	if (hdr->env->to)
+	  mutt_format_s (buf2, sizeof (buf2), prefix, mutt_get_name (hdr->env->to));
+	else if (hdr->env->cc)
+	  mutt_format_s (buf2, sizeof (buf2), prefix, mutt_get_name (hdr->env->cc));
+	else
+	  *buf2 = 0;
+      }
+      else
+	mutt_format_s (buf2, sizeof (buf2), prefix, mutt_get_name (hdr->env->from));
+      if ((p = strpbrk (buf2, " %@")))
+	*p = 0;
+      mutt_format_s (dest, destlen, prefix, buf2);
       break;
 
     case 'Z':
-      if (hdr->mailcap)
-	ch = 'M';
+    
+      ch = ' ';
 
+      if (WithCrypto && hdr->security & GOODSIGN)
+        ch = 'S';
+      else if (WithCrypto && hdr->security & ENCRYPT)
+      	ch = 'P';
+      else if (WithCrypto && hdr->security & SIGN)
+        ch = 's';
+      else if ((WithCrypto & APPLICATION_PGP) && hdr->security & PGPKEY)
+        ch = 'K';
 
-
-
-
-
-      else
-	ch = ' ';
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
       snprintf (buf2, sizeof (buf2),
-		"%c%c%c",
-		(hdr->read && (ctx && ctx->msgnotreadyet != hdr->msgno))
-		? (hdr->replied ? 'r' : ' ') : (hdr->old ? 'O' : 'N'),
+		"%c%c%c", (THREAD_NEW ? 'n' : (THREAD_OLD ? 'o' : 
+		((hdr->read && (ctx && ctx->msgnotreadyet != hdr->msgno))
+		? (hdr->replied ? 'r' : ' ') : (hdr->old ? 'O' : 'N')))),
 		hdr->deleted ? 'D' : (hdr->attach_del ? 'd' : ch),
 		hdr->tagged ? '*' :
 		(hdr->flagged ? '!' :
-		 (Tochars && ((i = user_is_recipient (hdr->env)) < strlen (Tochars)) ? Tochars[user_is_recipient (hdr->env)] : ' ')));
-      snprintf (dest, destlen, fmt, buf2);
+		 (Tochars && ((i = mutt_user_is_recipient (hdr)) < mutt_strlen (Tochars)) ? Tochars[i] : ' ')));
+      mutt_format_s (dest, destlen, prefix, buf2);
+      break;
+
+     case 'y':
+       if (optional)
+	 optional = hdr->env->x_label ? 1 : 0;
+
+       mutt_format_s (dest, destlen, prefix, NONULL (hdr->env->x_label));
+       break;
+ 
+    case 'Y':
+      if (hdr->env->x_label)
+      {
+	i = 1;	/* reduce reuse recycle */
+	htmp = NULL;
+	if (flags & M_FORMAT_TREE
+	    && (hdr->thread->prev && hdr->thread->prev->message
+		&& hdr->thread->prev->message->env->x_label))
+	  htmp = hdr->thread->prev->message;
+	else if (flags & M_FORMAT_TREE
+		 && (hdr->thread->parent && hdr->thread->parent->message
+		     && hdr->thread->parent->message->env->x_label))
+	  htmp = hdr->thread->parent->message;
+	if (htmp && mutt_strcasecmp (hdr->env->x_label,
+				     htmp->env->x_label) == 0)
+	  i = 0;
+      }
+      else
+	i = 0;
+
+      if (optional)
+	optional = i;
+
+      if (i)
+        mutt_format_s (dest, destlen, prefix, NONULL (hdr->env->x_label));
+      else
+        mutt_format_s (dest, destlen, prefix, "");
+
       break;
 
     default:
@@ -552,6 +702,8 @@ hdr_format_str (char *dest,
     mutt_FormatString (dest, destlen, elsestring, hdr_format_str, (unsigned long) hfi, flags);
 
   return (src);
+#undef THREAD_NEW
+#undef THREAD_OLD
 }
 
 void
