@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -16,12 +16,14 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */
 
-#include "mutt.h"
-
-#ifdef HAVE_PGP
-#include "pgp.h"
+#if HAVE_CONFIG_H
+# include "config.h"
 #endif
 
+#include "mutt.h"
+#include "mutt_crypt.h"
+#include "mutt_idna.h"
+
 #include <sys/stat.h>
 #include <string.h>
 #include <ctype.h>
@@ -37,11 +39,10 @@ void mutt_edit_headers (const char *editor,
   char *p;
   FILE *ifp, *ofp;
   int i, keep;
-  int in_reply_to = 0;	/* did we see the in-reply-to field ? */
   ENVELOPE *n;
   time_t mtime;
   struct stat st;
-  LIST *cur, *last = NULL, *tmp;
+  LIST *cur, **last = NULL, *tmp;
 
   mutt_mktemp (path);
   if ((ofp = safe_fopen (path, "w")) == NULL)
@@ -49,7 +50,8 @@ void mutt_edit_headers (const char *editor,
     mutt_perror (path);
     return;
   }
-
+  
+  mutt_env_to_local (msg->env);
   mutt_write_rfc822_header (ofp, msg->env, NULL, 1, 0);
   fputc ('\n', ofp);	/* tie off the header. */
 
@@ -71,7 +73,8 @@ void mutt_edit_headers (const char *editor,
     return;
   }
 
-  mtime = st.st_mtime;
+  mtime = mutt_decrease_mtime (path, &st);
+
   mutt_edit_file (editor, path);
   stat (path, &st);
   if (mtime == st.st_mtime)
@@ -110,16 +113,21 @@ void mutt_edit_headers (const char *editor,
   /* restore old info. */
   n->references = msg->env->references;
   msg->env->references = NULL;
+
   mutt_free_envelope (&msg->env);
-  msg->env = n;
+  msg->env = n; n = NULL;
+
+  if (!msg->env->in_reply_to)
+    mutt_free_list (&msg->env->references);
 
   mutt_expand_aliases_env (msg->env);
-  
+
   /* search through the user defined headers added to see if either a 
    * fcc: or attach-file: field was specified.  
    */
 
   cur = msg->env->userhdrs;
+  last = &msg->env->userhdrs;
   while (cur)
   {
     keep = 1;
@@ -128,9 +136,7 @@ void mutt_edit_headers (const char *editor,
      * not, remove the references: field later so that we can generate a new
      * message based upon this one.
      */
-    if (mutt_strncasecmp ("in-reply-to:", cur->data, 12) == 0)
-      in_reply_to = 1;
-    else if (fcc && mutt_strncasecmp ("fcc:", cur->data, 4) == 0)
+    if (fcc && ascii_strncasecmp ("fcc:", cur->data, 4) == 0)
     {
       p = cur->data + 4;
       SKIPWS (p);
@@ -141,7 +147,7 @@ void mutt_edit_headers (const char *editor,
       }
       keep = 0;
     }
-    else if (mutt_strncasecmp ("attach:", cur->data, 7) == 0)
+    else if (ascii_strncasecmp ("attach:", cur->data, 7) == 0)
     {
       BODY *body;
       BODY *parts;
@@ -175,34 +181,27 @@ void mutt_edit_headers (const char *editor,
     }
 
 
-
-#ifdef HAVE_PGP
-    else if (mutt_strncasecmp ("pgp:", cur->data, 4) == 0)
+    else if ((WithCrypto & APPLICATION_PGP)
+             &&ascii_strncasecmp ("pgp:", cur->data, 4) == 0)
     {
-      msg->pgp = mutt_parse_pgp_hdr (cur->data + 4, 0);
+      msg->security = mutt_parse_crypt_hdr (cur->data + 4, 0);
+      if (msg->security)
+	msg->security |= APPLICATION_PGP;
       keep = 0;
     }
-#endif
-
 
     if (keep)
     {
-      last = cur;
-      cur = cur->next;
+      last = &cur->next;
+      cur  = cur->next;
     }
     else
     {
-      if (last)
-	last->next = cur->next;
-      else
-	msg->env->userhdrs = cur->next;
-      tmp = cur;
-      cur = cur->next;
+      tmp       = cur;
+      *last     = cur->next;
+      cur       = cur->next;
       tmp->next = NULL;
       mutt_free_list (&tmp);
     }
   }
-
-  if (!in_reply_to)
-    mutt_free_list (&msg->env->references);
 }
