@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,11 +13,15 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #define HELP_C
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "keymap.h"
@@ -84,59 +88,63 @@ mutt_compile_help (char *buf, size_t buflen, int menu, struct mapping_t *items)
   return buf;
 }
 
-static int print_macro (FILE *f, int maxchar, const char **macro)
+static int print_macro (FILE *f, int maxwidth, const char **macro)
 {
-  int c = (unsigned char) **macro;
-  int n = maxchar;
+  int n = maxwidth;
+  wchar_t wc;
+  int w;
+  size_t k;
+  size_t len = mutt_strlen (*macro);
+  mbstate_t mbstate1, mbstate2;
 
-  while (c)
+  memset (&mbstate1, 0, sizeof (mbstate1));
+  memset (&mbstate2, 0, sizeof (mbstate2));
+  for (; len && (k = mbrtowc (&wc, *macro, len, &mbstate1)); *macro += k, len -= k)
   {
-    if (!IsPrint(c))
+    if (k == (size_t)(-1) || k == (size_t)(-2))
     {
-      if (c >= ' ' && c != 127)
-	c = '?';
-      else if (n < 2)
-	c = 0;
-      else
+      k = (k == (size_t)(-1)) ? 1 : len;
+      wc = replacement_char ();
+    }
+    /* glibc-2.1.3's wcwidth() returns 1 for unprintable chars! */
+    if (IsWPrint (wc) && (w = wcwidth (wc)) >= 0)
+    {
+      if (w > n)
+	break;
+      n -= w;
       {
-	--n;
-	switch (c)
-	{
-	  case '\033':
-	    fputc ('\\', f);
-	    c = 'e';
-	    break;
-	  case '\n':
-	    fputc ('\\', f);
-	    c = 'n';
-	    break;
-	  case '\r':
-	    fputc ('\\', f);
-	    c = 'r';
-	    break;
-	  case '\t':
-	    fputc ('\\', f);
-	    c = 't';
-	    break;
-	  default:
-	    fputc ('^', f);
-	    c = (c + '@') & 127;
-	    break;
-	}
+	char buf[MB_LEN_MAX*2];
+	size_t n1, n2;
+	if ((n1 = wcrtomb (buf, wc, &mbstate2)) != (size_t)(-1) &&
+	    (n2 = wcrtomb (buf + n1, 0, &mbstate2)) != (size_t)(-1))
+	  fputs (buf, f);
       }
     }
-
-    if (c && n > 0)
+    else if (wc < 0x20 || wc == 0x7f)
     {
-      --n;
-      fputc(c, f);
-      c = (unsigned char) *++*macro;
+      if (2 > n)
+	break;
+      n -= 2;
+      if (wc == '\033')
+	fprintf (f, "\\e");
+      else if (wc == '\n')
+	fprintf (f, "\\n");
+      else if (wc == '\r')
+	fprintf (f, "\\r");
+      else if (wc == '\t')
+	fprintf (f, "\\t");
+      else
+	fprintf (f, "^%c", (char)((wc + '@') & 0x7f));
     }
     else
-      c = 0;
+    {
+      if (1 > n)
+	break;
+      n -= 1;
+      fprintf (f, "?");
+    }
   }
-
-  return (maxchar - n);
+  return (maxwidth - n);
 }
 
 static int pad (FILE *f, int col, int i)
@@ -214,6 +222,7 @@ static void format_line (FILE *f, int ismacro,
       {
 	SKIPWS(t3);
 
+	/* FIXME: this is completely wrong */
 	if ((n = mutt_strlen (t3)) > COLS - col)
 	{
 	  n = COLS - col;
