@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 1996-9 Brandon Long <blong@fiction.net>
  * Copyright (C) 1999-2002 Brendan Cully <brendan@kublai.com>
  * 
@@ -21,6 +21,10 @@
 /* command.c: routines for sending commands to an IMAP server and parsing
  *  responses */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "imap_private.h"
 #include "message.h"
@@ -105,7 +109,7 @@ int imap_cmd_step (IMAP_DATA* idata)
   {
     if (len == cmd->blen)
     {
-      safe_realloc ((void**) &cmd->buf, cmd->blen + IMAP_CMD_BUFSIZE);
+      safe_realloc (&cmd->buf, cmd->blen + IMAP_CMD_BUFSIZE);
       cmd->blen = cmd->blen + IMAP_CMD_BUFSIZE;
       dprint (3, (debugfile, "imap_cmd_step: grew buffer to %u bytes\n",
 		  cmd->blen));
@@ -129,11 +133,13 @@ int imap_cmd_step (IMAP_DATA* idata)
   /* don't let one large string make cmd->buf hog memory forever */
   if ((cmd->blen > IMAP_CMD_BUFSIZE) && (len <= IMAP_CMD_BUFSIZE))
   {
-    safe_realloc ((void**) &cmd->buf, IMAP_CMD_BUFSIZE);
+    safe_realloc (&cmd->buf, IMAP_CMD_BUFSIZE);
     cmd->blen = IMAP_CMD_BUFSIZE;
     dprint (3, (debugfile, "imap_cmd_step: shrank buffer to %u bytes\n", cmd->blen));
   }
-  
+
+  idata->lastread = time(NULL);
+
   /* handle untagged messages. The caller still gets its shot afterwards. */
   if (!ascii_strncmp (cmd->buf, "* ", 2) &&
       cmd_handle_untagged (idata))
@@ -192,7 +198,7 @@ int imap_exec (IMAP_DATA* idata, const char* cmd, int flags)
 
   rc = mutt_socket_write_d (idata->conn, out,
     flags & IMAP_CMD_PASS ? IMAP_LOG_PASS : IMAP_LOG_CMD);
-  safe_free ((void**) &out);
+  FREE (&out);
 
   if (rc < 0)
   {
@@ -257,8 +263,7 @@ void imap_cmd_finish (IMAP_DATA* idata)
       /* check_status: curs_main uses imap_check_mailbox to detect
        *   whether the index needs updating */
       idata->check_status = IMAP_NEWMAIL_PENDING;
-      idata->reopen &= ~IMAP_NEWMAIL_PENDING;
-      count = imap_read_headers (idata, idata->ctx->msgcount, count-1)+1;
+      imap_read_headers (idata, idata->ctx->msgcount, count-1);
     }
     else if (idata->reopen & IMAP_EXPUNGE_PENDING)
     {
@@ -289,6 +294,11 @@ static void cmd_handle_fatal (IMAP_DATA* idata)
     mutt_error (_("Mailbox closed"));
     mutt_sleep (1);
     idata->state = IMAP_DISCONNECTED;
+  }
+
+  if (idata->state != IMAP_SELECTED)
+  {
+    idata->state = IMAP_DISCONNECTED;
     idata->status = 0;
   }
 }
@@ -302,7 +312,7 @@ static int cmd_handle_untagged (IMAP_DATA* idata)
 
   s = imap_next_word (idata->cmd.buf);
 
-  if ((idata->state == IMAP_SELECTED) && isdigit (*s))
+  if ((idata->state == IMAP_SELECTED) && isdigit ((unsigned char) *s))
   {
     pn = s;
     s = imap_next_word (s);
@@ -389,7 +399,7 @@ static int cmd_handle_untagged (IMAP_DATA* idata)
 /* cmd_make_sequence: make a tag suitable for starting an IMAP command */
 static void cmd_make_sequence (IMAP_DATA* idata)
 {
-  snprintf (idata->cmd.seq, sizeof (idata->cmd.seq), "a%04d", idata->seqno++);
+  snprintf (idata->cmd.seq, sizeof (idata->cmd.seq), "a%04u", idata->seqno++);
 
   if (idata->seqno > 9999)
     idata->seqno = 0;
@@ -462,19 +472,20 @@ static void cmd_parse_fetch (IMAP_DATA* idata, char* s)
 
   msgno = atoi (s);
   
+  if (msgno <= idata->ctx->msgcount)
   /* see cmd_parse_expunge */
-  for (cur = 0; cur < idata->ctx->msgcount; cur++)
-  {
-    h = idata->ctx->hdrs[cur];
-    
-    if (h->active && h->index+1 == msgno)
+    for (cur = 0; cur < idata->ctx->msgcount; cur++)
     {
-      dprint (2, (debugfile, "Message UID %d updated\n", HEADER_DATA(h)->uid));
-      break;
+      h = idata->ctx->hdrs[cur];
+      
+      if (h->active && h->index+1 == msgno)
+      {
+	dprint (2, (debugfile, "Message UID %d updated\n", HEADER_DATA(h)->uid));
+	break;
+      }
+      
+      h = NULL;
     }
-
-    h = NULL;
-  }
   
   if (!h)
   {
@@ -519,7 +530,7 @@ static void cmd_parse_myrights (IMAP_DATA* idata, char* s)
   /* zero out current rights set */
   memset (idata->rights, 0, sizeof (idata->rights));
 
-  while (*s && !isspace(*s))
+  while (*s && !isspace((unsigned char) *s))
   {
     switch (*s) 
     {
