@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 1996-9 Brandon Long <blong@fiction.net>
- * Copyright (C) 1999-2002 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 1999-2003 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,10 @@
 
 /* Support for IMAP4rev1, with the occasional nod to IMAP 4. */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mx.h"
@@ -29,7 +33,7 @@
 #include "browser.h"
 #include "message.h"
 #include "imap_private.h"
-#ifdef USE_SSL
+#if defined(USE_SSL) || defined(USE_GNUTLS)
 # include "mutt_ssl.h"
 #endif
 
@@ -104,20 +108,37 @@ int imap_create_mailbox (IMAP_DATA* idata, char* mailbox)
   return 0;
 }
 
+int imap_rename_mailbox (IMAP_DATA* idata, IMAP_MBOX* mx, const char* newname)
+{
+  char oldmbox[LONG_STRING];
+  char newmbox[LONG_STRING];
+  char buf[LONG_STRING];
+
+  imap_munge_mbox_name (oldmbox, sizeof (oldmbox), mx->mbox);
+  imap_munge_mbox_name (newmbox, sizeof (newmbox), newname);
+
+  snprintf (buf, sizeof (buf), "RENAME %s %s", oldmbox, newmbox);
+
+  if (imap_exec (idata, buf, 0) != 0)
+    return -1;
+
+  return 0;
+}
+
 int imap_delete_mailbox (CONTEXT* ctx, IMAP_MBOX mx)
 {
   char buf[LONG_STRING], mbox[LONG_STRING];
   IMAP_DATA *idata;
 
   if (!ctx || !ctx->data) {
-  	if (!(idata = imap_conn_find (&mx.account,
-    		option (OPTIMAPPASSIVE) ? M_IMAP_CONN_NONEW : 0)))
-  	{
-    		FREE (&mx.mbox);
-    		return -1;
-	}
+    if (!(idata = imap_conn_find (&mx.account,
+          option (OPTIMAPPASSIVE) ? M_IMAP_CONN_NONEW : 0)))
+    {
+      FREE (&mx.mbox);
+      return -1;
+    }
   } else {
-	  idata = ctx->data;
+    idata = ctx->data;
   }
 
   imap_munge_mbox_name (mbox, sizeof (mbox), mx.mbox);
@@ -340,30 +361,49 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
 
   /* don't open a new connection if one isn't wanted */
   if (flags & M_IMAP_CONN_NONEW)
-    if (!idata || idata->state == IMAP_DISCONNECTED)
-      goto err_conn;
+  {
+    if (!idata)
+    {
+      mutt_socket_free (conn);
+      return NULL;
+    }
+    if (idata->state < IMAP_AUTHENTICATED)
+      return NULL;
+  }
   
   if (!idata)
   {
     /* The current connection is a new connection */
     if (! (idata = imap_new_idata ()))
-      goto err_conn;
+    {
+      mutt_socket_free (conn);
+      return NULL;
+    }
 
     conn->data = idata;
     idata->conn = conn;
   }
+
   if (idata->state == IMAP_DISCONNECTED)
-    if (imap_open_connection (idata) != 0)
-      goto err_idata;
+    imap_open_connection (idata);
+  if (idata->state == IMAP_CONNECTED)
+  {
+    if (!imap_authenticate (idata))
+    {
+      idata->state = IMAP_AUTHENTICATED;
+      if (idata->conn->ssf)
+	dprint (2, (debugfile, "Communication encrypted at %d bits\n",
+		    idata->conn->ssf));
+    }
+    else
+      mutt_account_unsetpass (&idata->conn->account);
+    
+    FREE (&idata->capstr);
+  }
+  if (idata->state == IMAP_AUTHENTICATED)
+    imap_get_delim (idata);
   
   return idata;
-
- err_idata:
-  imap_free_idata (&idata);
- err_conn:
-  mutt_socket_free (conn);
-
-  return NULL;
 }
 
 int imap_open_connection (IMAP_DATA* idata)
@@ -375,15 +415,21 @@ int imap_open_connection (IMAP_DATA* idata)
 
   idata->state = IMAP_CONNECTED;
 
-  if (imap_cmd_step (idata) != IMAP_CMD_CONTINUE)
-    goto bail;
+  if (imap_cmd_step (idata) != IMAP_CMD_CONTINUE) {
+    mutt_error (_("Unexpected response received from server: %s"), idata->cmd.buf);
+    mutt_sleep (1);
+
+    mutt_socket_close (idata->conn);
+    idata->state = IMAP_DISCONNECTED;
+    return -1;
+  }
 
   if (ascii_strncasecmp ("* OK", idata->cmd.buf, 4) == 0)
   {
     /* TODO: Parse new tagged CAPABILITY data (* OK [CAPABILITY...]) */
     if (imap_check_capabilities (idata))
       goto bail;
-#if defined(USE_SSL) && !defined(USE_NSS)
+#if defined(USE_SSL) || defined(USE_GNUTLS)
     /* Attempt STARTTLS if available and desired. */
     if (mutt_bit_isset (idata->capabilities, STARTTLS) && !idata->conn->ssf)
     {
@@ -397,11 +443,15 @@ int imap_open_connection (IMAP_DATA* idata)
 	  goto bail;
 	if (rc != -2)
 	{
+#ifdef USE_SSL
 	  if (mutt_ssl_starttls (idata->conn))
+#elif USE_GNUTLS
+	  if (mutt_gnutls_starttls (idata->conn))
+#endif
 	  {
 	    mutt_error (_("Could not negotiate TLS connection"));
 	    mutt_sleep (1);
-	    goto bail;
+	    goto err_close_conn;
 	  }
 	  else
 	  {
@@ -413,16 +463,13 @@ int imap_open_connection (IMAP_DATA* idata)
       }
     }
 #endif    
-    if (imap_authenticate (idata))
-      goto bail;
-    if (idata->conn->ssf)
-      dprint (2, (debugfile, "Communication encrypted at %d bits\n",
-	idata->conn->ssf));
   }
   else if (ascii_strncasecmp ("* PREAUTH", idata->cmd.buf, 9) == 0)
   {
+    idata->state = IMAP_AUTHENTICATED;
     if (imap_check_capabilities (idata) != 0)
       goto bail;
+    FREE (&idata->capstr);
   } 
   else
   {
@@ -430,17 +477,13 @@ int imap_open_connection (IMAP_DATA* idata)
     goto bail;
   }
 
-  FREE (&idata->capstr);
-  idata->state = IMAP_AUTHENTICATED;
-
-  imap_get_delim (idata);
   return 0;
 
  err_close_conn:
   mutt_socket_close (idata->conn);
+  idata->state = IMAP_DISCONNECTED;
  bail:
   FREE (&idata->capstr);
-  idata->state = IMAP_DISCONNECTED;
   return -1;
 }
 
@@ -520,14 +563,14 @@ int imap_open_mailbox (CONTEXT* ctx)
   /* we require a connection which isn't currently in IMAP_SELECTED state */
   if (!(idata = imap_conn_find (&(mx.account), M_IMAP_CONN_NOSELECT)))
     goto fail_noidata;
+  if (idata->state < IMAP_AUTHENTICATED)
+    goto fail;
+
   conn = idata->conn;
 
   /* once again the context is new */
   ctx->data = idata;
 
-  if (idata->status == IMAP_FATAL)
-    goto fail;
-
   /* Clean up path and replace the one in the ctx */
   imap_fix_path (idata, mx.mbox, buf, sizeof (buf));
   FREE(&(idata->mailbox));
@@ -560,17 +603,6 @@ int imap_open_mailbox (CONTEXT* ctx)
     if ((rc = imap_cmd_step (idata)) != IMAP_CMD_CONTINUE)
       break;
 
-    pc = idata->cmd.buf + 2;
-    pc = imap_next_word (pc);
-    if (!ascii_strncasecmp ("EXISTS", pc, 6))
-    {
-      /* imap_handle_untagged will have picked up the EXISTS message and
-       * set the NEW_MAIL flag. We clear it here. */
-      idata->reopen &= ~IMAP_NEWMAIL_PENDING;
-      count = idata->newMailCount;
-      idata->newMailCount = 0;
-    }
-
     pc = idata->cmd.buf + 2;
 
     /* Obtain list of available flags here, may be overridden by a
@@ -596,6 +628,25 @@ int imap_open_mailbox (CONTEXT* ctx)
       if ((pc = imap_get_flags (&(idata->flags), pc)) == NULL)
 	goto fail;
     }
+#ifdef USE_HCACHE
+    /* save UIDVALIDITY for the header cache */
+    else if (ascii_strncasecmp("OK [UIDVALIDITY", pc, 14) == 0)
+    {
+      dprint(2, (debugfile, "Getting mailbox UIDVALIDITY\n"));
+      pc += 3;
+      pc = imap_next_word(pc);
+      sscanf(pc, "%u", &(idata->uid_validity));
+    }
+#endif
+    else
+    {
+      pc = imap_next_word (pc);
+      if (!ascii_strncasecmp ("EXISTS", pc, 6))
+      {
+	count = idata->newMailCount;
+	idata->newMailCount = 0;
+      }
+    }
   }
   while (rc == IMAP_CMD_CONTINUE);
 
@@ -613,7 +664,8 @@ int imap_open_mailbox (CONTEXT* ctx)
     goto fail;
 
   /* check for READ-ONLY notification */
-  if (!ascii_strncasecmp (imap_get_qualifier (idata->cmd.buf), "[READ-ONLY]", 11))
+  if (!ascii_strncasecmp (imap_get_qualifier (idata->cmd.buf), "[READ-ONLY]", 11)  \
+  && !mutt_bit_isset (idata->capabilities, ACL))
   {
     dprint (2, (debugfile, "Mailbox is read-only.\n"));
     ctx->readonly = 1;
@@ -646,6 +698,11 @@ int imap_open_mailbox (CONTEXT* ctx)
   {
     if (imap_check_acl (idata))
       goto fail;
+    if (!(mutt_bit_isset(idata->rights, IMAP_ACL_DELETE) ||
+          mutt_bit_isset(idata->rights, IMAP_ACL_SEEN) ||
+          mutt_bit_isset(idata->rights, IMAP_ACL_WRITE) ||
+          mutt_bit_isset(idata->rights, IMAP_ACL_INSERT)))
+       ctx->readonly = 1;
   }
   /* assume we have all rights if ACL is unavailable */
   else
@@ -661,8 +718,8 @@ int imap_open_mailbox (CONTEXT* ctx)
   }
 
   ctx->hdrmax = count;
-  ctx->hdrs = safe_malloc (count * sizeof (HEADER *));
-  ctx->v2r = safe_malloc (count * sizeof (int));
+  ctx->hdrs = safe_calloc (count, sizeof (HEADER *));
+  ctx->v2r = safe_calloc (count, sizeof (int));
   ctx->msgcount = 0;
   if (count && (imap_read_headers (idata, 0, count-1) < 0))
   {
@@ -676,7 +733,8 @@ int imap_open_mailbox (CONTEXT* ctx)
   return 0;
 
  fail:
-  idata->state = IMAP_AUTHENTICATED;
+  if (idata->state == IMAP_SELECTED)
+    idata->state = IMAP_AUTHENTICATED;
  fail_noidata:
   FREE (&mx.mbox);
   return -1;
@@ -702,7 +760,7 @@ int imap_open_mailbox_append (CONTEXT *ctx)
   conn = idata->conn;
 
   ctx->magic = M_IMAP;
-  ctx->data = (void *) idata;
+  ctx->data = idata;
 
   /* check mailbox existance */
 
@@ -756,22 +814,9 @@ void imap_logout (IMAP_DATA* idata)
   imap_cmd_start (idata, "LOGOUT");
   while (imap_cmd_step (idata) == IMAP_CMD_CONTINUE)
     ;
-}
 
-int imap_close_connection (CONTEXT *ctx)
-{
-  dprint (1, (debugfile, "imap_close_connection(): closing connection\n"));
-  /* if the server didn't shut down on us, close the connection gracefully */
-  if (CTX_DATA->status != IMAP_BYE)
-  {
-    mutt_message _("Closing connection to IMAP server...");
-    imap_logout (CTX_DATA);
-    mutt_clear_error ();
-  }
-  mutt_socket_close (CTX_DATA->conn);
-  CTX_DATA->state = IMAP_DISCONNECTED;
-  CTX_DATA->conn->data = NULL;
-  return 0;
+  FREE(& idata->cmd.buf);
+  FREE(& idata);
 }
 
 /* imap_set_flag: append str to flags if we currently have permission
@@ -781,7 +826,7 @@ static void imap_set_flag (IMAP_DATA* idata, int aclbit, int flag,
 {
   if (mutt_bit_isset (idata->rights, aclbit))
     if (flag)
-      strncat (flags, str, flsize);
+      safe_strcat (flags, flsize, str);
 }
 
 /* imap_make_msg_set: make an IMAP4rev1 UID message set out of a set of
@@ -871,7 +916,7 @@ int imap_make_msg_set (IMAP_DATA* idata, BUFFER* buf, int flag, int changed)
     }
   }
 
-  safe_free ((void**) &hdrs);
+  FREE (&hdrs);
 
   return count;
 }
@@ -913,7 +958,7 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
    * to be changed. */
   imap_allow_reopen (ctx);
 
-  if ((rc = imap_check_mailbox (ctx, index_hint)) != 0)
+  if ((rc = imap_check_mailbox (ctx, index_hint, 0)) != 0)
     return rc;
 
   memset (&cmd, 0, sizeof (cmd));
@@ -1079,6 +1124,8 @@ void imap_close_mailbox (CONTEXT* ctx)
     idata->reopen &= IMAP_REOPEN_ALLOW;
     idata->state = IMAP_AUTHENTICATED;
     FREE (&(idata->mailbox));
+    mutt_free_list (&idata->flags);
+    idata->ctx = NULL;
   }
 
   /* free IMAP part of headers */
@@ -1090,7 +1137,7 @@ void imap_close_mailbox (CONTEXT* ctx)
     if (idata->cache[i].path)
     {
       unlink (idata->cache[i].path);
-      safe_free ((void **) &idata->cache[i].path);
+      FREE (&idata->cache[i].path);
     }
   }
 }
@@ -1103,25 +1150,18 @@ void imap_close_mailbox (CONTEXT* ctx)
  *	0		no change
  *	-1		error
  */
-int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
+int imap_check_mailbox (CONTEXT *ctx, int *index_hint, int force)
 {
   /* overload keyboard timeout to avoid many mailbox checks in a row.
    * Most users don't like having to wait exactly when they press a key. */
-
   IMAP_DATA* idata;
-  time_t now;
   int result = 0;
 
   idata = (IMAP_DATA*) ctx->data;
 
-  now = time(NULL);
-  if (now > ImapLastCheck + Timeout)
-  {
-    ImapLastCheck = now;
-
-    if (imap_exec (idata, "NOOP", 0) != 0)
-      return -1;
-  }
+  if ((force || time(NULL) > idata->lastread + Timeout)
+      && imap_exec (idata, "NOOP", 0) != 0)
+    return -1;
 
   /* We call this even when we haven't run NOOP in case we have pending
    * changes to process, since we can reopen here. */
@@ -1218,7 +1258,7 @@ int imap_mailbox_check (char* path, int new)
       {
 	s = imap_next_word (s);
 	s = imap_next_word (s);
-	if (isdigit (*s))
+	if (isdigit ((unsigned char) *s))
 	{
 	  if (*s != '0')
 	  {
