@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000,2002 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,14 +13,19 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_menu.h"
 #include "mutt_curses.h"
 #include "keymap.h"
 #include "mapping.h"
+#include "mutt_crypt.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -37,13 +42,16 @@ struct mapping_t Menus[] = {
  { "index",	MENU_MAIN },
  { "pager",	MENU_PAGER },
  { "postpone",	MENU_POST },
-
-  
-
-#ifdef _PGPPATH
  { "pgp",	MENU_PGP },
-#endif  
-  
+ { "smime",	MENU_SMIME },
+#ifdef HAVE_GPGME
+ { "key_select_pgp",	MENU_KEY_SELECT_PGP },
+ { "key_select_smime",	MENU_KEY_SELECT_SMIME },
+#endif
+
+#ifdef MIXMASTER
+  { "mix", 	MENU_MIX },
+#endif
   
 
  { "query",	MENU_QUERY },
@@ -72,6 +80,9 @@ static struct mapping_t KeyNames[] = {
   { "<Esc>",	'\033' },
   { "<Tab>",	'\t' },
   { "<Space>",	' ' },
+#ifdef KEY_BTAB
+  { "<BackTab>", KEY_BTAB },
+#endif
   { NULL,	0 }
 };
 
@@ -96,7 +107,7 @@ static int parse_fkey(char *s)
   char *t;
   int n = 0;
 
-  if(s[0] != '<' || tolower(s[1]) != 'f')
+  if(s[0] != '<' || ascii_tolower(s[1]) != 'f')
     return -1;
 
   for(t = s + 2; *t && isdigit((unsigned char) *t); t++)
@@ -111,6 +122,22 @@ static int parse_fkey(char *s)
     return n;
 }
 
+/*
+ * This function parses the string <NNN> and uses the octal value as the key
+ * to bind.
+ */
+static int parse_keycode (const char *s)
+{
+  if (isdigit ((unsigned char) s[1]) &&
+      isdigit ((unsigned char) s[2]) &&
+      isdigit ((unsigned char) s[3]) &&
+      s[4] == '>')
+  {
+    return (s[3] - '0') + (s[2] - '0') * 8 + (s[1] - '0') * 64;
+  }
+  return -1;
+}
+
 static int parsekeys (char *str, keycode_t *d, int max)
 {
   int n, len = max;
@@ -138,13 +165,18 @@ static int parsekeys (char *str, keycode_t *d, int max)
 	s = t;
 	*d = KEY_F (n);
       }
+      else if ((n = parse_keycode(s)) > 0)
+      {
+	s = t;
+	*d = n;
+      }
       
       *t = c;
     }
 
     if(!*d)
     {
-      *d = *s;
+      *d = (unsigned char)*s;
       s++;
     }
     d++;
@@ -230,7 +262,7 @@ static int get_op (struct binding_t *bindings, const char *start, size_t len)
 
   for (i = 0; bindings[i].name; i++)
   {
-    if (!mutt_strncasecmp (start, bindings[i].name, len) &&   
+    if (!ascii_strncasecmp (start, bindings[i].name, len) &&   
 	mutt_strlen (bindings[i].name) == len)
       return bindings[i].op;
   }
@@ -267,10 +299,17 @@ static void push_string (char *s)
 	;
       if (pp >= s)
       {
+	if ((i = parse_fkey (pp)) > 0)
+	{
+	  mutt_ungetch (KEY_F (i), 0);
+	  p = pp - 1;
+	  continue;
+	}
+
 	l = p - pp + 1;
 	for (i = 0; KeyNames[i].name; i++)
 	{
-	  if (!mutt_strncasecmp (pp, KeyNames[i].name, l))
+	  if (!ascii_strncasecmp (pp, KeyNames[i].name, l))
 	    break;
 	}
 	if (KeyNames[i].name)
@@ -342,22 +381,13 @@ int km_dokey (int menu)
 
   FOREVER
   {
-/* ncurses doesn't return on resized screen when timeout is set to zero */
-#if !defined (USE_SLANG_CURSES) && defined (HAVE_RESIZETERM)
-    if (menu == MENU_MAIN || menu == MENU_PAGER)
-      timeout (menu == MENU_MAIN && Timeout > 0 ? Timeout * 1000 : INT_MAX);
-#else
-    if (menu == MENU_MAIN && Timeout > 0)
-      timeout (Timeout * 1000); /* milliseconds */
-#endif
+    /* ncurses doesn't return on resized screen when timeout is set to zero */
+    if (menu != MENU_EDITOR)
+      timeout ((Timeout > 0 ? Timeout : 60) * 1000);
 
     tmp = mutt_getch();
 
-#if !defined (USE_SLANG_CURSES) && defined (HAVE_RESIZETERM)
-    if (menu == MENU_MAIN || menu == MENU_PAGER)
-#else
-    if (menu == MENU_MAIN && Timeout > 0)
-#endif
+    if (menu != MENU_EDITOR)
       timeout (-1); /* restore blocking operation */
 
     LastKey = tmp.ch;
@@ -371,8 +401,8 @@ int km_dokey (int menu)
       struct binding_t *bindings;
 
       /* is this a valid op for this menu? */
-      bindings = km_get_table (menu);
-      if ((func = get_func (bindings, tmp.op)))
+      if ((bindings = km_get_table (menu)) &&
+	  (func = get_func (bindings, tmp.op)))
 	return tmp.op;
 
       if (menu == MENU_EDITOR && get_func (OpEditor, tmp.op))
@@ -526,14 +556,27 @@ void km_init (void)
   create_bindings (OpPager, MENU_PAGER);
   create_bindings (OpPost, MENU_POST);
   create_bindings (OpQuery, MENU_QUERY);
+  create_bindings (OpAlias, MENU_ALIAS);
 
 
+  if ((WithCrypto & APPLICATION_PGP))
+    create_bindings (OpPgp, MENU_PGP);
 
-#ifdef _PGPPATH
-  create_bindings (OpPgp, MENU_PGP);
+  if ((WithCrypto & APPLICATION_SMIME))
+    create_bindings (OpSmime, MENU_SMIME);
+
+#ifdef CRYPT_BACKEND_GPGME
+  create_bindings (OpPgp, MENU_KEY_SELECT_PGP);
+  create_bindings (OpSmime, MENU_KEY_SELECT_SMIME);
 #endif
 
-
+#ifdef MIXMASTER
+  create_bindings (OpMix, MENU_MIX);
+  
+  km_bindkey ("<space>", MENU_MIX, OP_GENERIC_SELECT_ENTRY);
+  km_bindkey ("h", MENU_MIX, OP_MIX_CHAIN_PREV);
+  km_bindkey ("l", MENU_MIX, OP_MIX_CHAIN_NEXT);
+#endif
   
   /* bindings for the line editor */
   create_bindings (OpEditor, MENU_EDITOR);
@@ -569,6 +612,8 @@ void km_init (void)
   km_bindkey ("8", MENU_GENERIC, OP_JUMP);
   km_bindkey ("9", MENU_GENERIC, OP_JUMP);
 
+  km_bindkey ("<enter>", MENU_GENERIC, OP_GENERIC_SELECT_ENTRY);
+
   /* Miscellaneous extra bindings */
   
   km_bindkey (" ", MENU_MAIN, OP_DISPLAY_MESSAGE);
@@ -578,8 +623,10 @@ void km_init (void)
   km_bindkey ("K", MENU_MAIN, OP_PREV_ENTRY);
   km_bindkey ("x", MENU_MAIN, OP_EXIT);
 
-  km_bindkey ("x", MENU_PAGER, OP_PAGER_EXIT);
-  km_bindkey ("q", MENU_PAGER, OP_PAGER_EXIT);
+  km_bindkey ("<enter>", MENU_MAIN, OP_DISPLAY_MESSAGE);
+
+  km_bindkey ("x", MENU_PAGER, OP_EXIT);
+  km_bindkey ("i", MENU_PAGER, OP_EXIT);
   km_bindkey ("<backspace>", MENU_PAGER, OP_PREV_LINE);
   km_bindkey ("<pagedown>", MENU_PAGER, OP_NEXT_PAGE);
   km_bindkey ("<pageup>", MENU_PAGER, OP_PREV_PAGE);
@@ -599,7 +646,14 @@ void km_init (void)
   km_bindkey ("8", MENU_PAGER, OP_JUMP);
   km_bindkey ("9", MENU_PAGER, OP_JUMP);
 
-  km_bindkey ("<return>", MENU_ALIAS, OP_TAG);
+  km_bindkey ("<enter>", MENU_PAGER, OP_NEXT_LINE);
+  
+  km_bindkey ("<return>", MENU_ALIAS, OP_GENERIC_SELECT_ENTRY);
+  km_bindkey ("<enter>",  MENU_ALIAS, OP_GENERIC_SELECT_ENTRY);
+  km_bindkey ("<space>", MENU_ALIAS, OP_TAG);
+
+  km_bindkey ("<enter>", MENU_ATTACH, OP_VIEW_ATTACH);
+  km_bindkey ("<enter>", MENU_COMPOSE, OP_VIEW_ATTACH);
 
   /* edit-to (default "t") hides generic tag-entry in Compose menu
      This will bind tag-entry to  "T" in the Compose menu */
@@ -647,38 +701,53 @@ int mutt_parse_push (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
   return (r);
 }
 
-/* expects to see: <menu-string> <key-string> */
-char *parse_keymap (int *menu, BUFFER *s, BUFFER *err)
+/* expects to see: <menu-string>,<menu-string>,... <key-string> */
+static char *parse_keymap (int *menu, BUFFER *s, int maxmenus, int *nummenus, BUFFER *err)
 {
   BUFFER buf;
+  int i=0;
+  char *p, *q;
 
   memset (&buf, 0, sizeof (buf));
 
   /* menu name */
   mutt_extract_token (&buf, s, 0);
+  p = buf.data;
   if (MoreArgs (s))
   {
-    if ((*menu = mutt_check_menu (buf.data)) == -1)
+    while (i < maxmenus)
     {
-      snprintf (err->data, err->dsize, _("%s: no such menu"), buf.data);
-    }
-    else
-    {
-      /* key sequence */
-      mutt_extract_token (&buf, s, 0);
+      q = strchr(p,',');
+      if (q)
+        *q = '\0';
 
-      if (!*buf.data)
+      if ((menu[i] = mutt_check_menu (p)) == -1)
       {
-	strfcpy (err->data, _("null key sequence"), err->dsize);
+         snprintf (err->data, err->dsize, _("%s: no such menu"), p);
+         goto error;
       }
-      else if (MoreArgs (s))
-	return (buf.data);
+      ++i;
+      if (q)
+        p = q+1;
+      else
+        break;
     }
+    *nummenus=i;
+    /* key sequence */
+    mutt_extract_token (&buf, s, 0);
+
+    if (!*buf.data)
+    {
+      strfcpy (err->data, _("null key sequence"), err->dsize);
+    }
+    else if (MoreArgs (s))
+      return (buf.data);
   }
   else
   {
     strfcpy (err->data, _("too few arguments"), err->dsize);
   }
+error:
   FREE (&buf.data);
   return (NULL);
 }
@@ -713,6 +782,8 @@ struct binding_t *km_get_table (int menu)
       return OpPost;
     case MENU_FOLDER:
       return OpBrowser;
+    case MENU_ALIAS:
+      return OpAlias;
     case MENU_ATTACH:
       return OpAttach;
     case MENU_EDITOR:
@@ -720,14 +791,20 @@ struct binding_t *km_get_table (int menu)
     case MENU_QUERY:
       return OpQuery;
 
-
-
-#ifdef _PGPPATH
     case MENU_PGP:
+      return (WithCrypto & APPLICATION_PGP)? OpPgp:NULL;
+
+#ifdef CRYPT_BACKEND_GPGME
+    case MENU_KEY_SELECT_PGP:
       return OpPgp;
+    case MENU_KEY_SELECT_SMIME:
+      return OpSmime;
 #endif
 
-
+#ifdef MIXMASTER
+    case MENU_MIX:
+      return OpMix;
+#endif
 
   }
   return NULL;
@@ -738,9 +815,10 @@ int mutt_parse_bind (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
   struct binding_t *bindings = NULL;
   char *key;
-  int menu, r = 0;
+  int menu[sizeof(Menus)/sizeof(struct mapping_t)-1], r = 0, nummenus, i;
 
-  if ((key = parse_keymap (&menu, s, err)) == NULL)
+  if ((key = parse_keymap (menu, s, sizeof (menu)/sizeof (menu[0]),
+			   &nummenus, err)) == NULL)
     return (-1);
 
   /* function to execute */
@@ -750,20 +828,29 @@ int mutt_parse_bind (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
     strfcpy (err->data, _("bind: too many arguments"), err->dsize);
     r = -1;
   }
-  else if (mutt_strcasecmp ("noop", buf->data) == 0)
-    km_bindkey (key, menu, OP_NULL); /* the `unbind' command */
+  else if (ascii_strcasecmp ("noop", buf->data) == 0)
+  {
+    for (i = 0; i < nummenus; ++i)
+    {
+      km_bindkey (key, menu[i], OP_NULL); /* the `unbind' command */
+    }
+  }
   else
   {
-    /* First check the "generic" list of commands */
-    if (menu == MENU_PAGER || menu == MENU_EDITOR || menu == MENU_GENERIC ||
-	try_bind (key, menu, buf->data, OpGeneric) != 0)
+    for (i = 0; i < nummenus; ++i)
     {
-      /* Now check the menu-specific list of commands (if they exist) */
-      bindings = km_get_table (menu);
-      if (bindings && try_bind (key, menu, buf->data, bindings) != 0)
+      /* First check the "generic" list of commands */
+      if (menu[i] == MENU_PAGER || menu[i] == MENU_EDITOR ||
+      menu[i] == MENU_GENERIC ||
+	  try_bind (key, menu[i], buf->data, OpGeneric) != 0)
       {
-	snprintf (err->data, err->dsize, _("%s: no such function in map"), buf->data);
-	r = -1;
+        /* Now check the menu-specific list of commands (if they exist) */
+        bindings = km_get_table (menu[i]);
+        if (bindings && try_bind (key, menu[i], buf->data, bindings) != 0)
+        {
+          snprintf (err->data, err->dsize, _("%s: no such function in map"), buf->data);
+          r = -1;
+        }
       }
     }
   }
@@ -774,11 +861,11 @@ int mutt_parse_bind (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 /* macro <menu> <key> <macro> <description> */
 int mutt_parse_macro (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  int menu, r = -1;
+  int menu[sizeof(Menus)/sizeof(struct mapping_t)-1], r = -1, nummenus, i;
   char *seq = NULL;
   char *key;
 
-  if ((key = parse_keymap (&menu, s, err)) == NULL)
+  if ((key = parse_keymap (menu, s, sizeof (menu) / sizeof (menu[0]), &nummenus, err)) == NULL)
     return (-1);
 
   mutt_extract_token (buf, s, M_TOKEN_CONDENSE);
@@ -791,7 +878,7 @@ int mutt_parse_macro (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
   {
     if (MoreArgs (s))
     {
-      seq = strdup (buf->data);
+      seq = safe_strdup (buf->data);
       mutt_extract_token (buf, s, M_TOKEN_CONDENSE);
 
       if (MoreArgs (s))
@@ -800,61 +887,89 @@ int mutt_parse_macro (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
       }
       else
       {
-	km_bind (key, menu, OP_MACRO, seq, buf->data);
-	r = 0;
+        for (i = 0; i < nummenus; ++i)
+        {
+          km_bind (key, menu[i], OP_MACRO, seq, buf->data);
+          r = 0;
+        }
       }
 
       FREE (&seq);
     }
     else
     {
-      km_bind (key, menu, OP_MACRO, buf->data, NULL);
-      r = 0;
+      for (i = 0; i < nummenus; ++i)
+      {
+        km_bind (key, menu[i], OP_MACRO, buf->data, NULL);
+        r = 0;
+      }
     }
   }
   FREE (&key);
   return (r);
 }
 
-/* exec command-name */
+/* exec function-name */
 int mutt_parse_exec (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
   int ops[128]; 
   int nops = 0;
   struct binding_t *bindings = NULL;
-  char *command;
-  
+  char *function;
+
   if (!MoreArgs (s))
   {
-    strfcpy (err->data, _("exec: too few arguments"), err->dsize);
+    strfcpy (err->data, _("exec: no arguments"), err->dsize);
     return (-1);
   }
 
   do
   {
     mutt_extract_token (buf, s, 0);
-    command = buf->data;
+    function = buf->data;
 
     if ((bindings = km_get_table (CurrentMenu)) == NULL 
 	&& CurrentMenu != MENU_PAGER)
       bindings = OpGeneric;
-    
-    ops[nops] = get_op (bindings, command, mutt_strlen(command));
+
+    ops[nops] = get_op (bindings, function, mutt_strlen(function));
     if (ops[nops] == OP_NULL && CurrentMenu != MENU_PAGER)
-      ops[nops] = get_op (OpGeneric, command, mutt_strlen(command));
-    
+      ops[nops] = get_op (OpGeneric, function, mutt_strlen(function));
+
     if (ops[nops] == OP_NULL)
     {
       mutt_flushinp ();
-      mutt_error (_("%s: no such command"), command);
+      mutt_error (_("%s: no such function"), function);
       return (-1);
     }
     nops++;
   }
   while(MoreArgs(s) && nops < sizeof(ops)/sizeof(ops[0]));
-  
+
   while(nops)
     mutt_ungetch(0, ops[--nops]);
-  
+
   return 0;
 }
+
+/*
+ * prompts the user to enter a keystroke, and displays the octal value back
+ * to the user.
+ */
+void mutt_what_key (void)
+{
+  int ch;
+
+  mvprintw(LINES-1,0, _("Enter keys (^G to abort): "));
+  do {
+    ch = getch();
+    if (ch != ERR && ch != ctrl ('G'))
+    {
+      mutt_message(_("Char = %s, Octal = %o, Decimal = %d"),
+	       km_keyname(ch), ch, ch);
+    }
+  }
+  while (ch != ERR && ch != ctrl ('G'));
+
+  mutt_flushinp();
+}
