@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000,2 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 #ifndef KEYMAP_H
 #define KEYMAP_H
 
+#include "mapping.h"
+
 /* maximal length of a key binding sequence used for buffer in km_bindkey */
 #define MAX_SEQ 8
 
@@ -46,6 +48,7 @@ int km_expand_key (char *, size_t, struct keymap_t *);
 struct keymap_t *km_find_func (int, int);
 void km_init (void);
 void km_error_key (int);
+void mutt_what_key (void);
 
 enum
 {
@@ -61,10 +64,14 @@ enum
   MENU_QUERY,
 
   
-#ifdef HAVE_PGP
   MENU_PGP,
-#endif
+  MENU_SMIME,
 
+#ifdef CRYPT_BACKEND_GPGME
+  MENU_KEY_SELECT_PGP,
+  MENU_KEY_SELECT_SMIME,
+#endif
+  
 #ifdef MIXMASTER
   MENU_MIX,
 #endif
@@ -102,9 +109,9 @@ extern struct binding_t OpEditor[];
 extern struct binding_t OpQuery[];
 extern struct binding_t OpAlias[];
 
-#ifdef HAVE_PGP
 extern struct binding_t OpPgp[];
-#endif /* HAVE_PGP */
+
+extern struct binding_t OpSmime[];
 
 #ifdef MIXMASTER
 extern struct binding_t OpMix[];
