@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,9 @@
 #define M_APPEND	(1<<1) /* open mailbox for appending messages */
 #define M_READONLY	(1<<2) /* open in read-only mode */
 #define M_QUIET		(1<<3) /* do not print any messages */
+#define M_NEWFOLDER	(1<<4) /* create a new folder - same as M_APPEND, but uses
+				* safe_fopen() for mbox-style folders.
+				*/
 
 /* mx_open_new_message() */
 #define M_ADD_FROM	1	/* add a From_ line */
@@ -33,7 +36,8 @@ enum
 {
   M_NEW_MAIL = 1,	/* new mail received in mailbox */
   M_LOCKED,		/* couldn't lock the mailbox */
-  M_REOPENED		/* mailbox was reopened */
+  M_REOPENED,		/* mailbox was reopened */
+  M_FLAGS               /* nondestructive flags change (IMAP) */
 };
 
 typedef struct
@@ -42,6 +46,12 @@ typedef struct
   char *path;	/* path to temp file */
   short magic;	/* type of mailbox this message belongs to */
   short write;	/* nonzero if message is open for writing */
+  struct {
+    unsigned read : 1;
+    unsigned flagged : 1;
+    unsigned replied : 1;
+  } flags;
+  time_t received;	/* the time at which this message was received */
 } MESSAGE;
 
 CONTEXT *mx_open_mailbox (const char *, int, CONTEXT *);
@@ -61,6 +71,11 @@ int mx_check_mailbox (CONTEXT *, int *, int);
 #ifdef USE_IMAP
 int mx_is_imap (const char *);
 #endif
+#ifdef USE_POP
+int mx_is_pop (const char *);
+#endif
 
+int mx_access (const char*, int);
+int mx_check_empty (const char *);
 
 #endif
