@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
- * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
+ * Copyright (C) 1999-2002 Thomas Roessler <roessler@does-not-exist.org>
+ * Copyright (C) 2004 g10 Code GmbH
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -19,11 +20,17 @@
 
 #define MAIN_C 1
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "keymap.h"
 #include "mailbox.h"
-#include "reldate.h"
+#include "url.h"
+#include "mutt_crypt.h"
+#include "mutt_idna.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -37,23 +44,28 @@
 #include <getopt.h>
 #endif
 
+#ifdef HAVE_LIBIDN
+#include <stringprep.h>
+#endif
+
 static const char *ReachingUs = N_("\
 To contact the developers, please mail to <mutt-dev@mutt.org>.\n\
-To report a bug, please use the muttbug utility.\n");
+To report a bug, please use the flea(1) utility.\n");
 
 static const char *Notice = N_("\
-Copyright (C) 1996-2000 Michael R. Elkins and others.\n\
+Copyright (C) 1996-2002 Michael R. Elkins and others.\n\
 Mutt comes with ABSOLUTELY NO WARRANTY; for details type `mutt -vv'.\n\
 Mutt is free software, and you are welcome to redistribute it\n\
 under certain conditions; type `mutt -vv' for details.\n");
 
 static const char *Copyright = N_("\
-Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>\n\
-Copyright (C) 1996-2000 Brandon Long <blong@fiction.net>\n\
-Copyright (C) 1997-2000 Thomas Roessler <roessler@guug.de>\n\
-Copyright (C) 1998-2000 Werner Koch <wk@isil.d.shuttle.de>\n\
-Copyright (C) 1999-2000 Brendan Cully <brendan@kublai.com>\n\
-Copyright (C) 1999-2000 Tommi Komulainen <Tommi.Komulainen@iki.fi>\n\
+Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>\n\
+Copyright (C) 1996-2002 Brandon Long <blong@fiction.net>\n\
+Copyright (C) 1997-2002 Thomas Roessler <roessler@does-not-exist.org>\n\
+Copyright (C) 1998-2002 Werner Koch <wk@isil.d.shuttle.de>\n\
+Copyright (C) 1999-2002 Brendan Cully <brendan@kublai.com>\n\
+Copyright (C) 1999-2002 Tommi Komulainen <Tommi.Komulainen@iki.fi>\n\
+Copyright (C) 2000-2002 Edmund Grimley Evans <edmundo@rano.org>\n\
 \n\
 Lots of others not mentioned here contributed lots of code,\n\
 fixes, and suggestions.\n\
@@ -72,23 +84,6 @@ fixes, and suggestions.\n\
     along with this program; if not, write to the Free Software\n\
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.\n\
 ");
-#ifdef HAVE_PGP
-
-const char* ShaCopyright = N_("\n\
-SHA1 implementation Copyright (C) 1995-1997 Eric A. Young <eay@cryptsoft.com>\n\
-\n\
-    Redistribution and use in source and binary forms, with or without\n\
-    modification, are permitted under certain conditions.\n\
-\n\
-    The SHA1 implementation comes AS IS, and ANY EXPRESS OR IMPLIED\n\
-    WARRANTIES, including, but not limited to, the implied warranties of\n\
-    merchantability and fitness for a particular purpose ARE DISCLAIMED.\n\
-\n\
-    You should have received a copy of the full distribution terms\n\
-    along with this program; if not, write to the program's developers.\n\
-");
-#endif
-
 
 void mutt_exit (int code)
 {
@@ -98,25 +93,30 @@ void mutt_exit (int code)
 
 static void mutt_usage (void)
 {
-  printf ("Mutt %s (%s)\n", MUTT_VERSION, ReleaseDate);
+  puts (mutt_make_version ());
+
   puts _(
-"usage: mutt [ -nRzZ ] [ -e <cmd> ] [ -F <file> ] [ -m <type> ] [ -f <file> ]\n\
+"usage: mutt [ -nRyzZ ] [ -e <cmd> ] [ -F <file> ] [ -m <type> ] [ -f <file> ]\n\
+       mutt [ -nR ] [ -e <cmd> ] [ -F <file> ] -Q <query> [ -Q <query> ] [...]\n\
+       mutt [ -nR ] [ -e <cmd> ] [ -F <file> ] -A <alias> [ -A <alias> ] [...]\n\
        mutt [ -nx ] [ -e <cmd> ] [ -a <file> ] [ -F <file> ] [ -H <file> ] [ -i <file> ] [ -s <subj> ] [ -b <addr> ] [ -c <addr> ] <addr> [ ... ]\n\
        mutt [ -n ] [ -e <cmd> ] [ -F <file> ] -p\n\
        mutt -v[v]\n\
 \n\
 options:\n\
+  -A <alias>\texpand the given alias\n\
   -a <file>\tattach a file to the message\n\
   -b <address>\tspecify a blind carbon-copy (BCC) address\n\
   -c <address>\tspecify a carbon-copy (CC) address\n\
   -e <command>\tspecify a command to be executed after initialization\n\
   -f <file>\tspecify which mailbox to read\n\
   -F <file>\tspecify an alternate muttrc file\n\
-  -H <file>\tspecify a draft file to read header from\n\
-  -i <file>\tspecify a file which Mutt should include in the reply\n\
+  -H <file>\tspecify a draft file to read header and body from\n\
+  -i <file>\tspecify a file which Mutt should include in the body\n\
   -m <type>\tspecify a default mailbox type\n\
   -n\t\tcauses Mutt not to read the system Muttrc\n\
   -p\t\trecall a postponed message\n\
+  -Q <variable>\tquery a configuration variable\n\
   -R\t\topen mailbox in read-only mode\n\
   -s <subj>\tspecify a subject (must be in quotes if it has spaces)\n\
   -v\t\tshow version and compile-time definitions\n\
@@ -133,7 +133,7 @@ static void show_version (void)
 {
   struct utsname uts;
 
-  printf ("Mutt %s (%s)\n", MUTT_VERSION, ReleaseDate);
+  puts (mutt_make_version());
   puts (_(Notice));
 
   uname (&uts);
@@ -146,12 +146,24 @@ static void show_version (void)
   printf ("System: %s %s", uts.sysname, uts.release);
 #endif
 
+  printf (" (%s)", uts.machine);
+
 #ifdef NCURSES_VERSION
   printf (" [using ncurses %s]", NCURSES_VERSION);
 #elif defined(USE_SLANG_CURSES)
   printf (" [using slang %d]", SLANG_VERSION);
 #endif
 
+#ifdef _LIBICONV_VERSION
+  printf (" [using libiconv %d.%d]", _LIBICONV_VERSION >> 8,
+	  _LIBICONV_VERSION & 0xff);
+#endif
+
+#ifdef HAVE_LIBIDN
+  printf (" [using libidn %s (compiled with %s)]", stringprep_check_version (NULL), 
+	  STRINGPREP_VERSION);
+#endif
+  
   puts (_("\nCompile options:"));
 
 #ifdef DOMAIN
@@ -188,6 +200,14 @@ static void show_version (void)
 	"-USE_DOTLOCK  "
 #endif
 
+#ifdef DL_STANDALONE
+	"+DL_STANDALONE  "
+#else
+	"-DL_STANDALONE  "
+#endif
+
+	"\n"
+	
 #ifdef USE_FCNTL
 	"+USE_FCNTL  "
 #else
@@ -195,12 +215,24 @@ static void show_version (void)
 #endif
 
 #ifdef USE_FLOCK
-	"+USE_FLOCK"
+	"+USE_FLOCK   "
 #else
-	"-USE_FLOCK"
+	"-USE_FLOCK   "
+#endif
+	
+#ifdef USE_INODESORT
+	"+USE_INODESORT   "
+#else
+	"-USE_INODESORT   "
 #endif
 	);
   puts (
+#ifdef USE_POP
+	"+USE_POP  "
+#else
+	"-USE_POP  "
+#endif
+
 #ifdef USE_IMAP
         "+USE_IMAP  "
 #else
@@ -213,18 +245,31 @@ static void show_version (void)
 	"-USE_GSS  "
 #endif
 
+	
 #ifdef USE_SSL
 	"+USE_SSL  "
 #else
 	"-USE_SSL  "
 #endif
 
-#ifdef USE_POP
-	"+USE_POP  "
+#ifdef USE_GNUTLS
+	"+USE_GNUTLS  "
 #else
-	"-USE_POP  "
+	"-USE_GNUTLS  "
 #endif
 
+#ifdef USE_SASL
+	"+USE_SASL  "
+#else
+	"-USE_SASL  "
+#endif
+#ifdef USE_SASL2
+	"+USE_SASL2  "
+#else
+	"-USE_SASL2  "
+#endif
+	"\n"
+	
 #ifdef HAVE_REGCOMP
 	"+HAVE_REGCOMP  "
 #else
@@ -244,11 +289,61 @@ static void show_version (void)
 #else
 	"-HAVE_COLOR  "
 #endif
-
-#ifdef HAVE_PGP
-	"+HAVE_PGP  "
+	
+#ifdef HAVE_START_COLOR
+	"+HAVE_START_COLOR  "
 #else
-	"-HAVE_PGP  "
+	"-HAVE_START_COLOR  "
+#endif
+	
+#ifdef HAVE_TYPEAHEAD
+	"+HAVE_TYPEAHEAD  "
+#else
+	"-HAVE_TYPEAHEAD  "
+#endif
+	
+#ifdef HAVE_BKGDSET
+	"+HAVE_BKGDSET  "
+#else
+	"-HAVE_BKGDSET  "
+#endif
+
+	"\n"
+	
+#ifdef HAVE_CURS_SET
+	"+HAVE_CURS_SET  "
+#else
+	"-HAVE_CURS_SET  "
+#endif
+	
+#ifdef HAVE_META
+	"+HAVE_META  "
+#else
+	"-HAVE_META  "
+#endif
+	
+#ifdef HAVE_RESIZETERM
+	"+HAVE_RESIZETERM  "
+#else
+	"-HAVE_RESIZETERM  "
+#endif
+        );	
+  
+  puts (
+#ifdef CRYPT_BACKEND_CLASSIC_PGP
+        "+CRYPT_BACKEND_CLASSIC_PGP  "
+#else
+        "-CRYPT_BACKEND_CLASSIC_PGP  "
+#endif
+#ifdef CRYPT_BACKEND_CLASSIC_SMIME
+        "+CRYPT_BACKEND_CLASSIC_SMIME  "
+#else
+        "-CRYPT_BACKEND_CLASSIC_SMIME  "
+#endif
+#ifdef CRYPT_BACKEND_GPGME
+        "+CRYPT_BACKEND_GPGME  "
+#else
+        "-CRYPT_BACKEND_GPGME  "
 #endif
 
 #ifdef BUFFY_SIZE
@@ -257,23 +352,89 @@ static void show_version (void)
 	"-BUFFY_SIZE "
 #endif
 #ifdef EXACT_ADDRESS
-	"+"
+	"+EXACT_ADDRESS  "
 #else
-	"-"
+	"-EXACT_ADDRESS  "
 #endif
-	"EXACT_ADDRESS  "
-#ifdef ENABLE_NLS
-	"+"
-#else
-	"-"
-#endif
-	"ENABLE_NLS"
-	);
 
-  printf ("SENDMAIL=\"%s\"\n", SENDMAIL);
-  printf ("MAILPATH=\"%s\"\n", MAILPATH);
-  printf ("SHAREDIR=\"%s\"\n", SHAREDIR);
-  printf ("SYSCONFDIR=\"%s\"\n", SYSCONFDIR);
+#ifdef SUN_ATTACHMENT
+	"+SUN_ATTACHMENT  "
+#else
+	"-SUN_ATTACHMENT  "
+#endif
+
+	"\n"
+	
+#ifdef ENABLE_NLS
+	"+ENABLE_NLS  "
+#else
+	"-ENABLE_NLS  "
+#endif
+
+#ifdef LOCALES_HACK
+	"+LOCALES_HACK  "
+#else
+	"-LOCALES_HACK  "
+#endif
+	      
+#ifdef HAVE_WC_FUNCS
+	"+HAVE_WC_FUNCS  "
+#else
+	"-HAVE_WC_FUNCS  "
+#endif
+	
+#ifdef HAVE_LANGINFO_CODESET
+	"+HAVE_LANGINFO_CODESET  "
+#else
+	"-HAVE_LANGINFO_CODESET  "
+#endif
+
+	
+#ifdef HAVE_LANGINFO_YESEXPR
+ 	"+HAVE_LANGINFO_YESEXPR  "
+#else
+ 	"-HAVE_LANGINFO_YESEXPR  "
+#endif
+	
+	"\n"
+
+#if HAVE_ICONV
+	"+HAVE_ICONV  "
+#else
+	"-HAVE_ICONV  "
+#endif
+
+#if ICONV_NONTRANS
+	"+ICONV_NONTRANS  "
+#else
+	"-ICONV_NONTRANS  "
+#endif
+
+#if HAVE_LIBIDN
+	"+HAVE_LIBIDN  "
+#else
+	"-HAVE_LIBIDN  "
+#endif
+	
+#if HAVE_GETSID
+	"+HAVE_GETSID  "
+#else
+	"-HAVE_GETSID  "
+#endif
+
+#if HAVE_GETADDRINFO
+	"+HAVE_GETADDRINFO  "
+#else
+	"-HAVE_GETADDRINFO  "
+#endif
+
+#if USE_HCACHE
+	"+USE_HCACHE  "
+#else
+	"-USE_HCACHE  "
+#endif
+
+	);
 
 #ifdef ISPELL
   printf ("ISPELL=\"%s\"\n", ISPELL);
@@ -281,8 +442,21 @@ static void show_version (void)
   puts ("-ISPELL");
 #endif
 
+  printf ("SENDMAIL=\"%s\"\n", SENDMAIL);
+  printf ("MAILPATH=\"%s\"\n", MAILPATH);
+  printf ("PKGDATADIR=\"%s\"\n", PKGDATADIR);
+  printf ("SYSCONFDIR=\"%s\"\n", SYSCONFDIR);
+  printf ("EXECSHELL=\"%s\"\n", EXECSHELL);
+#ifdef MIXMASTER
+  printf ("MIXMASTER=\"%s\"\n", MIXMASTER);
+#else
+  puts ("-MIXMASTER");
+#endif
+
   puts(_(ReachingUs));
 
+  mutt_print_patchlist();
+  
   exit (0);
 }
 
@@ -293,6 +467,7 @@ static void start_curses (void)
 #ifdef USE_SLANG_CURSES
   SLtt_Ignore_Beep = 1; /* don't do that #*$@^! annoying visual beep! */
   SLsmg_Display_Eight_Bit = 128; /* characters above this are printable */
+  SLtt_set_color(0, NULL, "default", "default");
 #else
   /* should come before initscr() so that ncurses 4.2 doesn't try to install
      its own SIGWINCH handler */
@@ -303,7 +478,7 @@ static void start_curses (void)
     puts _("Error initializing terminal.");
     exit (1);
   }
-#ifdef USE_SLANG_CURSES
+#if 1 /* USE_SLANG_CURSES  - commenting out suggested in #455. */
   /* slang requires the signal handlers to be set after initializing */
   mutt_signal_init ();
 #endif
@@ -335,6 +510,8 @@ int main (int argc, char **argv)
   HEADER *msg = NULL;
   LIST *attach = NULL;
   LIST *commands = NULL;
+  LIST *queries = NULL;
+  LIST *alias_queries = NULL;
   int sendflags = 0;
   int flags = 0;
   int version = 0;
@@ -362,15 +539,19 @@ int main (int argc, char **argv)
   setlocale (LC_CTYPE, "");
 
   mutt_error = mutt_nocurses_error;
+  mutt_message = mutt_nocurses_error;
   SRAND (time (NULL));
   umask (077);
 
   memset (Options, 0, sizeof (Options));
   memset (QuadOptions, 0, sizeof (QuadOptions));
   
-  while ((i = getopt (argc, argv, "a:b:F:f:c:d:e:H:s:i:hm:npRvxyzZ")) != EOF)
+  while ((i = getopt (argc, argv, "A:a:b:F:f:c:d:e:H:s:i:hm:npQ:RvxyzZ")) != EOF)
     switch (i)
     {
+      case 'A':
+        alias_queries = mutt_add_list (alias_queries, optarg);
+        break;
       case 'a':
 	attach = mutt_add_list (attach, optarg);
 	break;
@@ -430,6 +611,10 @@ int main (int argc, char **argv)
 	sendflags |= SENDPOSTPONED;
 	break;
 
+      case 'Q':
+        queries = mutt_add_list (queries, optarg);
+        break;
+      
       case 'R':
 	flags |= M_RO; /* read-only mode */
 	break;
@@ -470,17 +655,14 @@ int main (int argc, char **argv)
       show_version ();
       break;
     default:
-      printf ("Mutt %s (%s)\n", MUTT_VERSION, ReleaseDate);
+      puts (mutt_make_version ());
       puts (_(Copyright));
-#ifdef HAVE_PGP
-      puts(_(ShaCopyright));
-#endif
       puts (_(ReachingUs));
       exit (0);
   }
 
   /* Check for a batch send. */
-  if (!isatty (0))
+  if (!isatty (0) || queries || alias_queries)
   {
     set_option (OPTNOCURSES);
     sendflags = SENDBATCH;
@@ -495,6 +677,33 @@ int main (int argc, char **argv)
   mutt_init (flags & M_NOSYSRC, commands);
   mutt_free_list (&commands);
 
+  /* Initialize crypto backends.  */
+  crypt_init ();
+
+  if (queries)
+    return mutt_query_variables (queries);
+
+  if (alias_queries)
+  {
+    int rv = 0;
+    ADDRESS *a;
+    for (; alias_queries; alias_queries = alias_queries->next)
+    {
+      if ((a = mutt_lookup_alias (alias_queries->data)))
+      {	
+	/* output in machine-readable form */
+	mutt_addrlist_to_idna (a, NULL);
+	mutt_write_address_list (a, stdout, 0, 0);
+      }
+      else
+      {
+	rv = 1;
+	printf ("%s\n", alias_queries->data);
+      }
+    }
+    return rv;
+  }
+  
   if (newMagic)
     mx_set_magic (newMagic);
 
@@ -503,6 +712,7 @@ int main (int argc, char **argv)
     SETCOLOR (MT_COLOR_NORMAL);
     clear ();
     mutt_error = mutt_curses_error;
+    mutt_message = mutt_curses_message;
   }
 
   /* Create the Maildir directory if it doesn't exist. */
@@ -521,7 +731,7 @@ int main (int argc, char **argv)
     if (stat (fpath, &sb) == -1 && errno == ENOENT)
     {
       snprintf (msg, sizeof (msg), _("%s does not exist. Create it?"), Maildir);
-      if (mutt_yesorno (msg, 1) == 1)
+      if (mutt_yesorno (msg, M_YES) == M_YES)
       {
 	if (mkdir (fpath, 0700) == -1 && errno != EEXIST)
 	  mutt_error ( _("Can't create %s: %s."), Maildir, strerror (errno));
@@ -542,7 +752,8 @@ int main (int argc, char **argv)
     FILE *fin = NULL;
     char buf[LONG_STRING];
     char *tempfile = NULL, *infile = NULL;
-
+    char *bodytext = NULL;
+    
     if (!option (OPTNOCURSES))
       mutt_flushinp ();
 
@@ -557,7 +768,12 @@ int main (int argc, char **argv)
 	msg->env = mutt_new_envelope ();
 
       for (i = optind; i < argc; i++)
-	msg->env->to = rfc822_parse_adrlist (msg->env->to, argv[i]);
+      {
+	if (url_check_scheme (argv[i]) == U_MAILTO)
+	  url_parse_mailto (msg->env, &bodytext, argv[i]);
+	else
+	  msg->env->to = rfc822_parse_adrlist (msg->env->to, argv[i]);
+      }
 
       if (option (OPTAUTOEDIT) && !msg->env->to && !msg->env->cc)
       {
@@ -567,36 +783,45 @@ int main (int argc, char **argv)
 	exit (1);
       }
 
-      msg->env->subject = safe_strdup (subject);
-      
+      if (subject)
+	msg->env->subject = safe_strdup (subject);
+
       if (includeFile)
 	infile = includeFile;
     }
 
-    if (infile)
+    if (infile || bodytext)
     {
-      if (mutt_strcmp ("-", infile) == 0)
-	fin = stdin;
-      else
+      if (infile)
       {
-	char path[_POSIX_PATH_MAX];
-
-	strfcpy (path, infile, sizeof (path));
-	mutt_expand_path (path, sizeof (path));
-	if ((fin = fopen (path, "r")) == NULL)
+	if (mutt_strcmp ("-", infile) == 0)
+	  fin = stdin;
+	else 
 	{
-	  if (!option (OPTNOCURSES))
-	    mutt_endwin (NULL);
-	  perror (path);
-	  exit (1);
+	  char path[_POSIX_PATH_MAX];
+	  
+	  strfcpy (path, infile, sizeof (path));
+	  mutt_expand_path (path, sizeof (path));
+	  if ((fin = fopen (path, "r")) == NULL)
+	  {
+	    if (!option (OPTNOCURSES))
+	      mutt_endwin (NULL);
+	    perror (path);
+	    exit (1);
+	  }
 	}
       }
+      else
+	fin = NULL;
+
       mutt_mktemp (buf);
       tempfile = safe_strdup (buf);
 
       if (draftFile)
 	msg->env = mutt_read_rfc822_header (fin, NULL, 1, 0);
 
+      /* is the following if still needed? */
+      
       if (tempfile)
       {
 	FILE *fout;
@@ -610,14 +835,18 @@ int main (int argc, char **argv)
 	  FREE (&tempfile);
 	  exit (1);
 	}
-
-	mutt_copy_stream (fin, fout);
+	if (fin)
+	  mutt_copy_stream (fin, fout);
+	else if (bodytext)
+	  fputs (bodytext, fout);
 	fclose (fout);
-	if (fin != stdin)
+	if (fin && fin != stdin)
 	  fclose (fin);
       }
     }
 
+    FREE (&bodytext);
+    
     if (attach)
     {
       LIST *t = attach;
@@ -660,7 +889,7 @@ int main (int argc, char **argv)
 	exit (1);
       }
       folder[0] = 0;
-      mutt_buffy (folder);
+      mutt_buffy (folder, sizeof (folder));
     }
     else if (flags & M_SELECT)
     {
@@ -669,7 +898,7 @@ int main (int argc, char **argv)
 	exit (1);
       }
       folder[0] = 0;
-      mutt_select_file (folder, sizeof (folder), 1);
+      mutt_select_file (folder, sizeof (folder), M_SEL_FOLDER | M_SEL_BUFFY);
       if (!folder[0])
       {
 	mutt_endwin (NULL);
@@ -685,19 +914,15 @@ int main (int argc, char **argv)
 
     if (flags & M_IGNORE)
     {
-      struct stat st;
-
       /* check to see if there are any messages in the folder */
-      if (stat (folder, &st) != 0)
+      switch (mx_check_empty (folder))
       {
-	mutt_endwin (strerror (errno));
-	exit (1);
-      }
-
-      if (st.st_size == 0)
-      {
-	mutt_endwin _("Mailbox is empty.");
-	exit (1);
+	case -1:
+	  mutt_endwin (strerror (errno));
+	  exit (1);
+	case 1:
+	  mutt_endwin _("Mailbox is empty.");
+	  exit (1);
       }
     }
 
@@ -708,7 +933,7 @@ int main (int argc, char **argv)
     {
       mutt_index_menu ();
       if (Context)
-	safe_free ((void **)&Context);
+	FREE (&Context);
     }
     mutt_endwin (Errorbuf);
   }
