@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@
 
 /* This file contains code to parse ``mbox'' and ``mmdf'' style mailboxes */
 
-/* OS/2's "kendra" mail folder format is also supported.  It's a slightly
- * modified version of MMDF.
- */
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include "mutt.h"
 #include "mailbox.h"
@@ -82,7 +82,7 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
 {
   char buf[HUGE_STRING];
   char return_path[LONG_STRING];
-  int count = 0;
+  int count = 0, oldmsgcount = ctx->msgcount;
   int lines;
   time_t t, tz;
   long loc, tmploc;
@@ -120,14 +120,14 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
     if (fgets (buf, sizeof (buf) - 1, ctx->fp) == NULL)
       break;
 
-    if (mutt_strcmp (buf, ctx->magic == M_MMDF ? MMDF_SEP : KENDRA_SEP) == 0)
+    if (mutt_strcmp (buf, MMDF_SEP) == 0)
     {
       loc = ftell (ctx->fp);
       
       count++;
       if (!ctx->quiet && ReadInc && ((count % ReadInc == 0) || count == 1))
 	mutt_message (_("Reading %s... %d (%d%%)"), ctx->path, count,
-		      loc / (ctx->size / 100 + 1));
+		      (int)(loc / (ctx->size / 100 + 1)));
 
 
       if (ctx->msgcount == ctx->hdrmax)
@@ -138,14 +138,14 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
 
       if (fgets (buf, sizeof (buf) - 1, ctx->fp) == NULL)
       {
+	/* TODO: memory leak??? */
 	dprint (1, (debugfile, "mmdf_parse_mailbox: unexpected EOF\n"));
 	break;
       }
 
       return_path[0] = 0;
-      t = is_from (buf, return_path, sizeof (return_path));
 
-      if (!t)
+      if (!is_from (buf, return_path, sizeof (return_path), &t))
       {
 	if (fseek (ctx->fp, loc, SEEK_SET) != 0)
 	{
@@ -169,7 +169,7 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
 	{
 	  if (fseek (ctx->fp, tmploc, SEEK_SET) != 0 ||
 	      fgets (buf, sizeof (buf) - 1, ctx->fp) == NULL ||
-	      mutt_strcmp (ctx->magic == M_MMDF ? MMDF_SEP : KENDRA_SEP, buf) != 0)
+	      mutt_strcmp (MMDF_SEP, buf) != 0)
 	  {
 	    if (fseek (ctx->fp, loc, SEEK_SET) != 0)
 	      dprint (1, (debugfile, "mmdf_parse_mailbox: fseek() failed\n"));
@@ -190,7 +190,7 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
 	  if (fgets (buf, sizeof (buf) - 1, ctx->fp) == NULL)
 	    break;
 	  lines++;
-	} while (mutt_strcmp (buf, ctx->magic == M_MMDF ? MMDF_SEP : KENDRA_SEP) != 0);
+	} while (mutt_strcmp (buf, MMDF_SEP) != 0);
 
 	hdr->lines = lines;
 	hdr->content->length = loc - hdr->content->offset;
@@ -202,9 +202,7 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
       if (!hdr->env->from)
 	hdr->env->from = rfc822_cpy_adr (hdr->env->return_path);
 
-      mx_update_context (ctx);
-      if(ctx->magic == M_KENDRA && feof(ctx->fp))
-	break;
+      ctx->msgcount++;
     }
     else
     {
@@ -214,7 +212,10 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
     }
   }
 
-  return 0;
+  if (ctx->msgcount > oldmsgcount)
+    mx_update_context (ctx, ctx->msgcount - oldmsgcount);
+
+  return (0);
 }
 
 /* Note that this function is also called when new mail is appended to the
@@ -264,7 +265,7 @@ int mbox_parse_mailbox (CONTEXT *ctx)
   loc = ftell (ctx->fp);
   while (fgets (buf, sizeof (buf), ctx->fp) != NULL)
   {
-    if ((t = is_from (buf, return_path, sizeof (return_path))))
+    if (is_from (buf, return_path, sizeof (return_path), &t))
     {
       /* Save the Content-Length of the previous message */
       if (count > 0)
@@ -285,7 +286,7 @@ int mbox_parse_mailbox (CONTEXT *ctx)
 
       if (!ctx->quiet && ReadInc && ((count % ReadInc == 0) || count == 1))
 	mutt_message (_("Reading %s... %d (%d%%)"), ctx->path, count,
-		      ftell (ctx->fp) / (ctx->size / 100 + 1));
+		      (int)(ftell (ctx->fp) / (ctx->size / 100 + 1)));
 
       if (ctx->msgcount == ctx->hdrmax)
 	mx_alloc_memory (ctx);
@@ -360,7 +361,7 @@ int mbox_parse_mailbox (CONTEXT *ctx)
 	}
       }
 
-      mx_update_context (ctx);
+      ctx->msgcount++;
 
       if (!curhdr->env->return_path && return_path[0])
 	curhdr->env->return_path = rfc822_parse_adrlist (curhdr->env->return_path, return_path);
@@ -393,6 +394,8 @@ int mbox_parse_mailbox (CONTEXT *ctx)
 
     if (!PREV->lines)
       PREV->lines = lines ? lines - 1 : 0;
+
+    mx_update_context (ctx, count);
   }
 
   return (0);
@@ -419,7 +422,7 @@ int mbox_open_mailbox (CONTEXT *ctx)
 
   if (ctx->magic == M_MBOX)
     rc = mbox_parse_mailbox (ctx);
-  else if (ctx->magic == M_MMDF || ctx->magic == M_KENDRA)
+  else if (ctx->magic == M_MMDF)
     rc = mmdf_parse_mailbox (ctx);
   else
     rc = -1;
@@ -603,8 +606,7 @@ int mbox_check_mailbox (CONTEXT *ctx, int *index_hint)
       if (fgets (buffer, sizeof (buffer), ctx->fp) != NULL)
       {
 	if ((ctx->magic == M_MBOX && mutt_strncmp ("From ", buffer, 5) == 0) ||
-	    (ctx->magic == M_MMDF && mutt_strcmp (MMDF_SEP, buffer) == 0) ||
-	    (ctx->magic == M_KENDRA && mutt_strcmp(KENDRA_SEP, buffer) == 0))
+	    (ctx->magic == M_MMDF && mutt_strcmp (MMDF_SEP, buffer) == 0))
 	{
 	  if (fseek (ctx->fp, ctx->size, SEEK_SET) != 0)
 	    dprint (1, (debugfile, "mbox_check_mailbox: fseek() failed\n"));
@@ -733,7 +735,7 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
       unlink (tempfile);
     }
     mutt_error _("Could not create temporary file!");
-    sleep (5);
+    mutt_sleep (5);
     goto bail;
   }
 
@@ -750,7 +752,7 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
      * never happen, is we presume it is a bug in mutt.
      */
     mutt_error _("sync: mbox modified, but no modified messages! (report this bug)");
-    sleep(5); /* the mutt_error /will/ get cleared! */
+    mutt_sleep(5); /* the mutt_error /will/ get cleared! */
     dprint(1, (debugfile, "mbox_sync_mailbox(): no modified messages.\n"));
     unlink (tempfile);
     goto bail;
@@ -766,8 +768,6 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
    */
   if (ctx->magic == M_MMDF)
     offset -= (sizeof MMDF_SEP - 1);
-  else if (ctx->magic == M_KENDRA)
-    offset -= (sizeof KENDRA_SEP - 1);
   
   /* allocate space for the new offsets */
   newOffset = safe_calloc (ctx->msgcount - first, sizeof (struct m_update_t));
@@ -791,29 +791,19 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
       j++;
       if (!ctx->quiet && WriteInc && ((i % WriteInc) == 0 || j == 1))
 	mutt_message (_("Writing messages... %d (%d%%)"), i,
-		      ftell (ctx->fp) / (ctx->size / 100 + 1));
+		      (int)(ftell (ctx->fp) / (ctx->size / 100 + 1)));
 
       if (ctx->magic == M_MMDF)
       {
 	if (fputs (MMDF_SEP, fp) == EOF)
 	{
 	  mutt_perror (tempfile);
-	  sleep (5);
+	  mutt_sleep (5);
 	  unlink (tempfile);
 	  goto bail;
 	}
 	  
       }
-      else if (ctx->magic == M_KENDRA)
-      {
-	if (fputs (KENDRA_SEP, fp) == EOF)
-	{
-	  mutt_perror (tempfile);
-	  sleep (5);
-	  unlink (tempfile);
-	  goto bail;
-	}
-      }
 
       /* save the new offset for this message.  we add `offset' because the
        * temporary file only contains saved message which are located after
@@ -824,7 +814,7 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
       if (mutt_copy_message (fp, ctx, ctx->hdrs[i], M_CM_UPDATE, CH_FROM | CH_UPDATE | CH_UPDATE_LEN) == -1)
       {
 	mutt_perror (tempfile);
-	sleep (5);
+	mutt_sleep (5);
 	unlink (tempfile);
 	goto bail;
       }
@@ -844,25 +834,16 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
 	  if(fputs(MMDF_SEP, fp) == EOF) 
 	  {
 	    mutt_perror (tempfile);
-	    sleep (5);
+	    mutt_sleep (5);
 	    unlink (tempfile);
 	    goto bail; 
 	  }
 	  break;
-	case M_KENDRA:
-	  if(fputs(KENDRA_SEP, fp) == EOF)
-	  {
-	    mutt_perror (tempfile);
-	    sleep (5);
-	    unlink (tempfile);
-	    goto bail;
-	  }
-	  break;
 	default:
 	  if(fputs("\n", fp) == EOF) 
 	  {
 	    mutt_perror (tempfile);
-	    sleep (5);
+	    mutt_sleep (5);
 	    unlink (tempfile);
 	    goto bail;
 	  }
@@ -876,7 +857,7 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
     dprint(1, (debugfile, "mbox_sync_mailbox: fclose() returned non-zero.\n"));
     unlink (tempfile);
     mutt_perror (tempfile);
-    sleep (5);
+    mutt_sleep (5);
     goto bail;
   }
   fp = NULL;
@@ -885,7 +866,7 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
   if (stat (ctx->path, &statbuf) == -1)
   {
     mutt_perror (ctx->path);
-    sleep (5);
+    mutt_sleep (5);
     unlink (tempfile);
     goto bail;
   }
@@ -896,7 +877,7 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
     mx_fastclose_mailbox (ctx);
     dprint (1, (debugfile, "mbox_sync_mailbox: unable to reopen temp copy of mailbox!\n"));
     mutt_perror (tempfile);
-    sleep (5);
+    mutt_sleep (5);
     return (-1);
   }
 
@@ -904,8 +885,7 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
       /* do a sanity check to make sure the mailbox looks ok */
       fgets (buf, sizeof (buf), ctx->fp) == NULL ||
       (ctx->magic == M_MBOX && mutt_strncmp ("From ", buf, 5) != 0) ||
-      (ctx->magic == M_MMDF && mutt_strcmp (MMDF_SEP, buf) != 0) ||
-      (ctx->magic == M_KENDRA && mutt_strcmp (KENDRA_SEP, buf) != 0))
+      (ctx->magic == M_MMDF && mutt_strcmp (MMDF_SEP, buf) != 0))
   {
     dprint (1, (debugfile, "mbox_sync_mailbox: message not in expected position."));
     dprint (1, (debugfile, "\tLINE: %s\n", buf));
@@ -923,6 +903,7 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
       /* copy the temp mailbox back into place starting at the first
        * change/deleted message
        */
+      mutt_message _("Committing changes...");
       i = mutt_copy_stream (fp, ctx->fp);
 
       if (ferror (ctx->fp))
@@ -947,14 +928,14 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
     
     char savefile[_POSIX_PATH_MAX];
     
-    snprintf (savefile, sizeof (savefile), "%s/mutt.%s-%s-%d",
-	      NONULL (Tempdir), NONULL(Username), NONULL(Hostname), getpid ());
+    snprintf (savefile, sizeof (savefile), "%s/mutt.%s-%s-%u",
+	      NONULL (Tempdir), NONULL(Username), NONULL(Hostname), (unsigned int)getpid ());
     rename (tempfile, savefile);
     mutt_unblock_signals ();
     mx_fastclose_mailbox (ctx);
     mutt_pretty_mailbox (savefile);
     mutt_error (_("Write failed!  Saved partial mailbox to %s"), savefile);
-    sleep (5);
+    mutt_sleep (5);
     return (-1);
   }
 
@@ -985,8 +966,8 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
       ctx->hdrs[i]->index = j++;
     }
   }
-  safe_free ((void **) &newOffset);
-  safe_free ((void **) &oldOffset);
+  FREE (&newOffset);
+  FREE (&oldOffset);
   unlink (tempfile); /* remove partial copy of the mailbox */
   mutt_unblock_signals ();
   Sort = save_sort; /* Restore the default value. */
@@ -1014,8 +995,8 @@ bail:  /* Come here in case of disaster */
   mbox_unlock_mailbox (ctx);
 
   mutt_unblock_signals ();
-  safe_free ((void **) &newOffset);
-  safe_free ((void **) &oldOffset);
+  FREE (&newOffset);
+  FREE (&oldOffset);
 
   if ((ctx->fp = freopen (ctx->path, "r", ctx->fp)) == NULL)
   {
@@ -1074,14 +1055,17 @@ int mutt_reopen_mailbox (CONTEXT *ctx, int *index_hint)
   old_msgcount = 0;
   
   /* simulate a close */
-  hash_destroy (&ctx->id_hash, NULL);
-  hash_destroy (&ctx->subj_hash, NULL);
-  safe_free ((void **) &ctx->v2r);
+  if (ctx->id_hash)
+    hash_destroy (&ctx->id_hash, NULL);
+  if (ctx->subj_hash)
+    hash_destroy (&ctx->subj_hash, NULL);
+  mutt_clear_threads (ctx);
+  FREE (&ctx->v2r);
   if (ctx->readonly)
   {
     for (i = 0; i < ctx->msgcount; i++)
       mutt_free_header (&(ctx->hdrs[i])); /* nothing to do! */
-    safe_free ((void **) &ctx->hdrs);
+    FREE (&ctx->hdrs);
   }
   else
   {
@@ -1100,14 +1084,13 @@ int mutt_reopen_mailbox (CONTEXT *ctx, int *index_hint)
   ctx->unread = 0;
   ctx->flagged = 0;
   ctx->changed = 0;
-  ctx->id_hash = hash_create (1031);
-  ctx->subj_hash = hash_create (1031);
+  ctx->id_hash = NULL;
+  ctx->subj_hash = NULL;
 
   switch (ctx->magic)
   {
     case M_MBOX:
     case M_MMDF:
-    case M_KENDRA:
       if (fseek (ctx->fp, 0, SEEK_SET) != 0)
       {
         dprint (1, (debugfile, "mutt_reopen_mailbox: fseek() failed\n"));
@@ -1131,7 +1114,7 @@ int mutt_reopen_mailbox (CONTEXT *ctx, int *index_hint)
     /* free the old headers */
     for (j = 0; j < old_msgcount; j++)
       mutt_free_header (&(old_hdrs[j]));
-    safe_free ((void **) &old_hdrs);
+    FREE (&old_hdrs);
 
     ctx->quiet = 0;
     return (-1);
@@ -1210,7 +1193,7 @@ int mutt_reopen_mailbox (CONTEXT *ctx, int *index_hint)
 	msg_mod = 1;
       }
     }
-    safe_free ((void **) &old_hdrs);
+    FREE (&old_hdrs);
   }
 
   ctx->quiet = 0;
@@ -1218,3 +1201,18 @@ int mutt_reopen_mailbox (CONTEXT *ctx, int *index_hint)
   return ((ctx->changed || msg_mod) ? M_REOPENED : M_NEW_MAIL);
 }
 
+/*
+ * Returns:
+ * 1 if the mailbox is not empty
+ * 0 if the mailbox is empty
+ * -1 on error
+ */
+int mbox_check_empty (const char *path)
+{
+  struct stat st;
+
+  if (stat (path, &st) == -1)
+    return -1;
+
+  return ((st.st_size == 0));
+}
