@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,10 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mutt_menu.h"
@@ -30,6 +34,8 @@
 
 extern int Charset_is_utf8; /* FIXME: bad modularisation */
 
+extern size_t UngetCount;
+
 static void print_enriched_string (int attr, unsigned char *s, int do_color)
 {
   wchar_t wc;
@@ -215,7 +221,8 @@ void menu_redraw_index (MUTTMENU *menu)
 
 	if (i == menu->current)
 	{
-	  SETCOLOR (MT_COLOR_INDICATOR);
+          attrset (menu->color (i));
+	  ADDCOLOR (MT_COLOR_INDICATOR);
 	  addstr ("->");
           attrset (menu->color (i));
 	  addch (' ');
@@ -228,14 +235,14 @@ void menu_redraw_index (MUTTMENU *menu)
       }
       else
       {
+        attrset (menu->color (i));
+            
 	if (i == menu->current)
 	{
-	  SETCOLOR (MT_COLOR_INDICATOR);
+	  ADDCOLOR (MT_COLOR_INDICATOR);
 	  BKGDSET (MT_COLOR_INDICATOR);
 	}
-        else
-          attrset (menu->color (i));
-            
+
 	CLEARLINE (i - menu->top + menu->offset);
 	print_enriched_string (menu->color(i), (unsigned char *) buf, i != menu->current);
         SETCOLOR (MT_COLOR_NORMAL);
@@ -280,7 +287,8 @@ void menu_redraw_motion (MUTTMENU *menu)
 
     /* now draw it in the new location */
     move (menu->current + menu->offset - menu->top, 0);
-    SETCOLOR (MT_COLOR_INDICATOR);
+    attrset (menu->color (menu->current));
+    ADDCOLOR (MT_COLOR_INDICATOR);
     addstr ("->");
     SETCOLOR (MT_COLOR_NORMAL);
   }
@@ -296,7 +304,8 @@ void menu_redraw_motion (MUTTMENU *menu)
     /* now draw the new one to reflect the change */
     menu_make_entry (buf, sizeof (buf), menu, menu->current);
     menu_pad_string (buf, sizeof (buf));
-    SETCOLOR (MT_COLOR_INDICATOR);
+    attrset (menu->color (menu->current));
+    ADDCOLOR (MT_COLOR_INDICATOR);
     BKGDSET (MT_COLOR_INDICATOR);
     CLEARLINE (menu->current - menu->top + menu->offset);
     print_enriched_string (menu->color(menu->current), (unsigned char *) buf, 0);
@@ -319,7 +328,8 @@ void menu_redraw_current (MUTTMENU *menu)
     int attr = menu->color (menu->current);
     attrset (attr);
     clrtoeol ();
-    SETCOLOR (MT_COLOR_INDICATOR);
+    attrset (menu->color (menu->current));
+    ADDCOLOR (MT_COLOR_INDICATOR);
     addstr ("->");
     attrset (attr);
     addch (' ');
@@ -329,7 +339,8 @@ void menu_redraw_current (MUTTMENU *menu)
   }
   else
   {
-    SETCOLOR (MT_COLOR_INDICATOR);
+    attrset (menu->color (menu->current));
+    ADDCOLOR (MT_COLOR_INDICATOR);
     BKGDSET (MT_COLOR_INDICATOR);
     clrtoeol ();
     print_enriched_string (menu->color(menu->current), (unsigned char *) buf, 0);
@@ -360,32 +371,37 @@ void menu_redraw_prompt (MUTTMENU *menu)
 
 void menu_check_recenter (MUTTMENU *menu)
 {
-  if (menu->max <= menu->pagelen && menu->top != 0)
+  int c = MIN (MenuContext, menu->pagelen / 2);
+  int old_top = menu->top;
+
+  if (!option (OPTMENUMOVEOFF) && menu->max <= menu->pagelen) /* less entries than lines */
   {
-    menu->top = 0;
-    set_option (OPTNEEDREDRAW);
-    menu->redraw |= REDRAW_INDEX;
-  }
-  else if (menu->current >= menu->top + menu->pagelen)
-  {
-    if (option (OPTMENUSCROLL) || (menu->pagelen <= 0))
-      menu->top = menu->current - menu->pagelen + 1;
-    else
-      menu->top += menu->pagelen * ((menu->current - menu->top) / menu->pagelen);
-    menu->redraw |= REDRAW_INDEX;
-  }
-  else if (menu->current < menu->top)
-  {
-    if (option (OPTMENUSCROLL) || (menu->pagelen <= 0))
-      menu->top = menu->current;
-    else
-    {
-      menu->top -= menu->pagelen * ((menu->top + menu->pagelen - 1 - menu->current) / menu->pagelen);
-      if (menu->top < 0)
-	menu->top = 0;
+    if (menu->top != 0) {
+      menu->top = 0;
+      set_option (OPTNEEDREDRAW);
     }
-    menu->redraw |= REDRAW_INDEX;
   }
+  else if (menu->current >= menu->top + menu->pagelen - c) /* indicator below bottom threshold */
+  {
+    if (option (OPTMENUSCROLL) || (menu->pagelen <= 0))
+      menu->top = menu->current - menu->pagelen + c + 1;
+    else
+      menu->top += (menu->pagelen - c) * ((menu->current - menu->top) / (menu->pagelen - c)) - c;
+  }
+  else if (menu->current < menu->top + c) /* indicator above top threshold */
+  {
+    if (option (OPTMENUSCROLL) || (menu->pagelen <= 0))
+      menu->top = menu->current - c;
+    else
+      menu->top -= (menu->pagelen - c) * ((menu->top + menu->pagelen - 1 - menu->current) / (menu->pagelen - c)) - c;
+  }
+
+  if (!option (OPTMENUMOVEOFF)) /* make entries stick to bottom */
+    menu->top = MIN (menu->top, menu->max - menu->pagelen);
+  menu->top = MAX (menu->top, 0);
+
+  if (menu->top != old_top)
+    menu->redraw |= REDRAW_INDEX;
 }
 
 void menu_jump (MUTTMENU *menu)
@@ -417,10 +433,13 @@ void menu_next_line (MUTTMENU *menu)
 {
   if (menu->max)
   {
-    if (menu->top + 1 < menu->max)
+    int c = MIN (MenuContext, menu->pagelen / 2);
+
+    if (menu->top + 1 < menu->max - c
+      && (option(OPTMENUMOVEOFF) || (menu->max > menu->pagelen && menu->top < menu->max - menu->pagelen)))
     {
       menu->top++;
-      if (menu->current < menu->top)
+      if (menu->current < menu->top + c && menu->current < menu->max - 1)
 	menu->current++;
       menu->redraw = REDRAW_INDEX;
     }
@@ -435,8 +454,10 @@ void menu_prev_line (MUTTMENU *menu)
 {
   if (menu->top > 0)
   {
+    int c = MIN (MenuContext, menu->pagelen / 2);
+
     menu->top--;
-    if (menu->current >= menu->top + menu->pagelen)
+    if (menu->current >= menu->top + menu->pagelen - c && menu->current > 1)
       menu->current--;
     menu->redraw = REDRAW_INDEX;
   }
@@ -469,7 +490,9 @@ void menu_next_page (MUTTMENU *menu)
 
 void menu_prev_page (MUTTMENU *menu)
 {
-  if (menu->top > 0)
+  int c = MIN (MenuContext, menu->pagelen / 2);
+
+  if (menu->top > c)
   {
     if ((menu->top -= menu->pagelen) < 0)
       menu->top = 0;
@@ -678,17 +701,17 @@ void mutt_menuDestroy (MUTTMENU **p)
 {
   int i;
 
-  safe_free ((void **) &(*p)->searchBuf);
+  FREE (&(*p)->searchBuf);
 
   if ((*p)->dialog) 
   {
     for (i=0; i < (*p)->max; i++)
-      safe_free ((void **) &(*p)->dialog[i]);
+      FREE (&(*p)->dialog[i]);
 
-    safe_free ((void **) (*p)->dialog);
+    FREE (& (*p)->dialog);
   }
 
-  safe_free ((void **) p);
+  FREE (p);
 }
 
 #define M_SEARCH_UP   1
@@ -793,6 +816,34 @@ static int menu_dialog_dokey (MUTTMENU *menu, int *ip)
   }
 }
 
+int menu_redraw (MUTTMENU *menu)
+{
+  /* See if all or part of the screen needs to be updated.  */
+  if (menu->redraw & REDRAW_FULL)
+  {
+    menu_redraw_full (menu);
+    /* allow the caller to do any local configuration */
+    return (OP_REDRAW);
+  }
+  
+  if (!menu->dialog)
+    menu_check_recenter (menu);
+  
+  if (menu->redraw & REDRAW_STATUS)
+    menu_redraw_status (menu);
+  if (menu->redraw & REDRAW_INDEX)
+    menu_redraw_index (menu);
+  else if (menu->redraw & (REDRAW_MOTION | REDRAW_MOTION_RESYNCH))
+    menu_redraw_motion (menu);
+  else if (menu->redraw == REDRAW_CURRENT)
+    menu_redraw_current (menu);
+  
+  if (menu->dialog)
+    menu_redraw_prompt (menu);
+  
+  return OP_NULL;
+}
+
 int mutt_menuLoop (MUTTMENU *menu)
 {
   int i = OP_NULL;
@@ -812,28 +863,8 @@ int mutt_menuLoop (MUTTMENU *menu)
     imap_keepalive ();
 #endif
 
-    /* See if all or part of the screen needs to be updated.  */
-    if (menu->redraw & REDRAW_FULL)
-    {
-      menu_redraw_full (menu);
-      /* allow the caller to do any local configuration */
-      return (OP_REDRAW);
-    }
-
-    if (!menu->dialog)
-      menu_check_recenter (menu);
-
-    if (menu->redraw & REDRAW_STATUS)
-      menu_redraw_status (menu);
-    if (menu->redraw & REDRAW_INDEX)
-      menu_redraw_index (menu);
-    else if (menu->redraw & (REDRAW_MOTION | REDRAW_MOTION_RESYNCH))
-      menu_redraw_motion (menu);
-    else if (menu->redraw == REDRAW_CURRENT)
-      menu_redraw_current (menu);
-
-    if (menu->dialog)
-      menu_redraw_prompt (menu);
+    if (menu_redraw (menu) == OP_REDRAW)
+      return OP_REDRAW;
     
     menu->oldcurrent = menu->current;
 
@@ -849,7 +880,7 @@ int mutt_menuLoop (MUTTMENU *menu)
       return i;
 		    
     i = km_dokey (menu->menu);
-    if (i == OP_TAG_PREFIX)
+    if (i == OP_TAG_PREFIX || i == OP_TAG_PREFIX_COND)
     {
       if (menu->tagged)
       {
@@ -859,11 +890,22 @@ int mutt_menuLoop (MUTTMENU *menu)
 	menu->tagprefix = 1;
 	CLEARLINE (LINES - 1);
       }
-      else 
+      else if (i == OP_TAG_PREFIX)
       {
 	mutt_error _("No tagged entries.");
 	i = -1;
       }
+      else /* None tagged, OP_TAG_PREFIX_COND */
+      {
+	event_t tmp;
+	while(UngetCount>0)
+	{
+	  tmp=mutt_getch();
+	  if(tmp.op==OP_END_COND)break;
+	}
+	mutt_message _("Nothing to do.");
+	i = -1;
+      }
     }
     else if (menu->tagged && option (OPTAUTOTAG))
       menu->tagprefix = 1;
@@ -878,6 +920,7 @@ int mutt_menuLoop (MUTTMENU *menu)
       mutt_resize_screen ();
       menu->redraw = REDRAW_FULL;
       SigWinch = 0;
+      clearok(stdscr,TRUE);/*force complete redraw*/
     }
 #endif
 
@@ -1008,6 +1051,10 @@ int mutt_menuLoop (MUTTMENU *menu)
 	MAYBE_REDRAW (menu->redraw);
 	break;
 
+      case OP_WHAT_KEY:
+	mutt_what_key ();
+	break;
+
       case OP_REDRAW:
 	clearok (stdscr, TRUE);
 	menu->redraw = REDRAW_FULL;
@@ -1022,6 +1069,9 @@ int mutt_menuLoop (MUTTMENU *menu)
 	km_error_key (menu->menu);
 	break;
 
+      case OP_END_COND:
+	break;
+
       default:
 	return (i);
     }
