@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
+ * Copyright (C) 2004 g10 Code GmbH
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #ifdef USE_SLANG_CURSES
@@ -36,11 +37,15 @@
 
 #else
 
+#ifdef HAVE_NCURSESW_NCURSES_H
+#include <ncursesw/ncurses.h>
+#else
 #ifdef HAVE_NCURSES_H
 #include <ncurses.h>
 #else
 #include <curses.h>
 #endif
+#endif
 
 #define M_ENTER_C '\n'
 #define M_ENTER_S "\n"
@@ -54,11 +59,9 @@
 #undef lines
 #endif /* lines */
 
-#include "mutt_regex.h"
-
 #define CLEARLINE(x) move(x,0), clrtoeol()
 #define CENTERLINE(x,y) move(y, (COLS-strlen(x))/2), addstr(x)
-#define BEEP if (option (OPTBEEP)) beep
+#define BEEP() do { if (option (OPTBEEP)) beep(); } while (0)
 
 #if ! (defined(USE_SLANG_CURSES) || defined(HAVE_CURS_SET))
 #define curs_set(x)
@@ -79,15 +82,20 @@ void mutt_curs_set (int);
 
 #define ctrl(c) ((c)-'@')
 
+#ifdef KEY_ENTER
 #define CI_is_return(c) ((c) == '\r' || (c) == '\n' || (c) == KEY_ENTER)
+#else
+#define CI_is_return(c) ((c) == '\r' || (c) == '\n')
+#endif
 
-int mutt_getch (void);
+event_t mutt_getch (void);
 
 void mutt_endwin (const char *);
 void mutt_flushinp (void);
 void mutt_refresh (void);
 void mutt_resize_screen (void);
-void mutt_ungetch (int);
+void mutt_ungetch (int, int);
+void mutt_need_hard_redraw (void);
 
 /* ----------------------------------------------------------------------------
  * Support for color
@@ -112,6 +120,7 @@ enum
   MT_COLOR_SEARCH,
   MT_COLOR_BOLD,
   MT_COLOR_UNDERLINE,
+  MT_COLOR_INDEX,
   MT_COLOR_MAX
 };
 
@@ -119,6 +128,8 @@ typedef struct color_line
 {
   regex_t rx;
   char *pattern;
+  pattern_t *color_pattern; /* compiled pattern to speed up index color
+                               calculation */
   short fg;
   short bg;
   int pair;
@@ -130,11 +141,13 @@ extern int ColorQuoteUsed;
 extern int ColorDefs[];
 extern COLOR_LINE *ColorHdrList;
 extern COLOR_LINE *ColorBodyList;
+extern COLOR_LINE *ColorIndexList;
 
 void ci_init_color (void);
 void ci_start_color (void);
 
 #define SETCOLOR(X) attrset(ColorDefs[X])
+#define ADDCOLOR(X) attron(ColorDefs[X])
 
 #define MAYBE_REDRAW(x) if (option (OPTNEEDREDRAW)) { unset_option (OPTNEEDREDRAW); x = REDRAW_FULL; }
 
