@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>, and others
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>, and others
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,10 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mapping.h"
 #include "keymap.h"
@@ -29,13 +33,7 @@
 #include <unistd.h>
 #include <stdarg.h>
 
-
-
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
-
-
+#include "mutt_crypt.h"
 
 static int eat_regexp (pattern_t *pat, BUFFER *, BUFFER *);
 static int eat_date (pattern_t *pat, BUFFER *, BUFFER *);
@@ -61,15 +59,12 @@ Flags[] =
   { 'E', M_EXPIRED,		0,		NULL },
   { 'f', M_FROM,		0,		eat_regexp },
   { 'F', M_FLAG,		0,		NULL },
-#ifdef HAVE_PGP
-  { 'g', M_PGP_SIGN, 		0, 		NULL },
-  { 'G', M_PGP_ENCRYPT, 	0, 		NULL },
-#endif
+  { 'g', M_CRYPT_SIGN, 		0, 		NULL },
+  { 'G', M_CRYPT_ENCRYPT, 	0, 		NULL },
   { 'h', M_HEADER,		M_FULL_MSG,	eat_regexp },
+  { 'H', M_HORMEL,		0,		eat_regexp },
   { 'i', M_ID,			0,		eat_regexp },
-#ifdef HAVE_PGP
   { 'k', M_PGP_KEY, 		0, 		NULL },
-#endif
   { 'L', M_ADDRESS,		0,		eat_regexp },
   { 'l', M_LIST,		0,		NULL },
   { 'm', M_MESSAGE,		0,		eat_range },
@@ -87,8 +82,12 @@ Flags[] =
   { 't', M_TO,			0,		eat_regexp },
   { 'U', M_UNREAD,		0,		NULL },
   { 'v', M_COLLAPSED,		0,		NULL },
+  { 'V', M_CRYPT_VERIFIED,      0,              NULL },
   { 'x', M_REFERENCE,		0,		eat_regexp },
+  { 'y', M_XLABEL,		0,		eat_regexp },
   { 'z', M_SIZE,		0,		eat_range },
+  { '=', M_DUPLICATED,		0,		NULL },
+  { '$', M_UNREFERENCED,	0,		NULL },
   { 0 }
 };
 
@@ -99,6 +98,17 @@ static char LastSearchExpn[LONG_STRING] = { 0 }; /* expanded version of
 
 #define M_MAXRANGE -1
 
+/* constants for parse_date_range() */
+#define M_PDR_NONE	0x0000
+#define M_PDR_MINUS	0x0001
+#define M_PDR_PLUS	0x0002
+#define M_PDR_WINDOW	0x0004
+#define M_PDR_ABSOLUTE	0x0008
+#define M_PDR_DONE	0x0010
+#define M_PDR_ERROR	0x0100
+#define M_PDR_ERRORDONE	(M_PDR_ERROR | M_PDR_DONE)
+
+
 int mutt_getvaluebychar (char ch, struct mapping_t *table)
 {
   int i;
@@ -158,10 +168,8 @@ msg_search (CONTEXT *ctx, regex_t *rx, char *buf, size_t blen, int op, int msgno
       {
 	mutt_parse_mime_message (ctx, h);
 
-
-
-#ifdef HAVE_PGP
-	if (h->pgp & PGPENCRYPT && !pgp_valid_passphrase())
+	if (WithCrypto && (h->security & ENCRYPT)
+            && !crypt_valid_passphrase(h->security))
 	{
 	  mx_close_message (&msg);
 	  if (fp)
@@ -171,9 +179,6 @@ msg_search (CONTEXT *ctx, regex_t *rx, char *buf, size_t blen, int op, int msgno
 	  }
 	  return (0);
 	}
-#endif
-
-
 
 	fseek (msg->fp, h->offset, 0);
 	mutt_body_handler (h->content, &s);
@@ -246,7 +251,7 @@ int eat_regexp (pattern_t *pat, BUFFER *s, BUFFER *err)
   {
     regerror (r, pat->rx, err->data, err->dsize);
     regfree (pat->rx);
-    safe_free ((void **) &pat->rx);
+    FREE (&pat->rx);
     return (-1);
   }
   return 0;
@@ -256,7 +261,17 @@ int eat_range (pattern_t *pat, BUFFER *s, BUFFER *err)
 {
   char *tmp;
   int do_exclusive = 0;
+  int skip_quote = 0;
   
+  /*
+   * If simple_search is set to "~m %s", the range will have double quotes 
+   * around it...
+   */
+  if (*s->dptr == '"')
+  {
+    s->dptr++;
+    skip_quote = 1;
+  }
   if (*s->dptr == '<')
     do_exclusive = 1;
   if ((*s->dptr != '-') && (*s->dptr != '<'))
@@ -269,12 +284,12 @@ int eat_range (pattern_t *pat, BUFFER *s, BUFFER *err)
     }
     else
       pat->min = strtol (s->dptr, &tmp, 0);
-    if (toupper (*tmp) == 'K') /* is there a prefix? */
+    if (toupper ((unsigned char) *tmp) == 'K') /* is there a prefix? */
     {
       pat->min *= 1024;
       tmp++;
     }
-    else if (toupper (*tmp) == 'M')
+    else if (toupper ((unsigned char) *tmp) == 'M')
     {
       pat->min *= 1048576;
       tmp++;
@@ -303,12 +318,12 @@ int eat_range (pattern_t *pat, BUFFER *s, BUFFER *err)
   {
     /* range maximum */
     pat->max = strtol (tmp, &tmp, 0);
-    if (toupper (*tmp) == 'K')
+    if (toupper ((unsigned char) *tmp) == 'K')
     {
       pat->max *= 1024;
       tmp++;
     }
-    else if (toupper (*tmp) == 'M')
+    else if (toupper ((unsigned char) *tmp) == 'M')
     {
       pat->max *= 1048576;
       tmp++;
@@ -319,6 +334,9 @@ int eat_range (pattern_t *pat, BUFFER *s, BUFFER *err)
   else
     pat->max = M_MAXRANGE;
 
+  if (skip_quote && *tmp == '"')
+    tmp++;
+
   SKIPWS (tmp);
   s->dptr = tmp;
   return 0;
@@ -330,7 +348,7 @@ static const char *getDate (const char *s, struct tm *t, BUFFER *err)
   time_t now = time (NULL);
   struct tm *tm = localtime (&now);
 
-  t->tm_mday = strtol (s, &p, 0);
+  t->tm_mday = strtol (s, &p, 10);
   if (t->tm_mday < 1 || t->tm_mday > 31)
   {
     snprintf (err->data, err->dsize, _("Invalid day of month: %s"), s);
@@ -344,7 +362,7 @@ static const char *getDate (const char *s, struct tm *t, BUFFER *err)
     return p;
   }
   p++;
-  t->tm_mon = strtol (p, &p, 0) - 1;
+  t->tm_mon = strtol (p, &p, 10) - 1;
   if (t->tm_mon < 0 || t->tm_mon > 11)
   {
     snprintf (err->data, err->dsize, _("Invalid month: %s"), p);
@@ -356,7 +374,7 @@ static const char *getDate (const char *s, struct tm *t, BUFFER *err)
     return p;
   }
   p++;
-  t->tm_year = strtol (p, &p, 0);
+  t->tm_year = strtol (p, &p, 10);
   if (t->tm_year < 70) /* year 2000+ */
     t->tm_year += 100;
   else if (t->tm_year > 1900)
@@ -368,30 +386,142 @@ static const char *getDate (const char *s, struct tm *t, BUFFER *err)
    Nm	months
    Nw	weeks
    Nd	days */
-static const char *get_offset (struct tm *tm, const char *s)
+static const char *get_offset (struct tm *tm, const char *s, int sign)
 {
   char *ps;
   int offset = strtol (s, &ps, 0);
+  if ((sign < 0 && offset > 0) || (sign > 0 && offset < 0))
+    offset = -offset;
 
   switch (*ps)
   {
     case 'y':
-      tm->tm_year -= offset;
+      tm->tm_year += offset;
       break;
     case 'm':
-      tm->tm_mon -= offset;
+      tm->tm_mon += offset;
       break;
     case 'w':
-      tm->tm_mday -= 7 * offset;
+      tm->tm_mday += 7 * offset;
       break;
     case 'd':
-      tm->tm_mday -= offset;
+      tm->tm_mday += offset;
       break;
+    default:
+      return s;
   }
   mutt_normalize_time (tm);
   return (ps + 1);
 }
 
+static void adjust_date_range (struct tm *min, struct tm *max)
+{
+  if (min->tm_year > max->tm_year
+      || (min->tm_year == max->tm_year && min->tm_mon > max->tm_mon)
+      || (min->tm_year == max->tm_year && min->tm_mon == max->tm_mon
+	&& min->tm_mday > max->tm_mday))
+  {
+    int tmp;
+    
+    tmp = min->tm_year;
+    min->tm_year = max->tm_year;
+    max->tm_year = tmp;
+      
+    tmp = min->tm_mon;
+    min->tm_mon = max->tm_mon;
+    max->tm_mon = tmp;
+      
+    tmp = min->tm_mday;
+    min->tm_mday = max->tm_mday;
+    max->tm_mday = tmp;
+    
+    min->tm_hour = min->tm_min = min->tm_sec = 0;
+    max->tm_hour = 23;
+    max->tm_min = max->tm_sec = 59;
+  }
+}
+
+static const char * parse_date_range (const char* pc, struct tm *min,
+    struct tm *max, int haveMin, struct tm *baseMin, BUFFER *err)
+{
+  int flag = M_PDR_NONE;	
+  while (*pc && ((flag & M_PDR_DONE) == 0))
+  {
+    const char *pt;
+    char ch = *pc++;
+    SKIPWS (pc);
+    switch (ch)
+    {
+      case '-':
+      {
+	/* try a range of absolute date minus offset of Ndwmy */
+	pt = get_offset (min, pc, -1);
+	if (pc == pt)
+	{
+	  if (flag == M_PDR_NONE)
+	  { /* nothing yet and no offset parsed => absolute date? */
+	    if (!getDate (pc, max, err))
+	      flag |= (M_PDR_ABSOLUTE | M_PDR_ERRORDONE);  /* done bad */
+	    else
+	    {
+	      /* reestablish initial base minimum if not specified */
+	      if (!haveMin)
+		memcpy (min, baseMin, sizeof(struct tm));
+	      flag |= (M_PDR_ABSOLUTE | M_PDR_DONE);  /* done good */
+	    }
+	  }
+	  else
+	    flag |= M_PDR_ERRORDONE;
+	}
+	else
+	{
+	  pc = pt;
+	  if (flag == M_PDR_NONE && !haveMin)
+	  { /* the very first "-3d" without a previous absolute date */
+	    max->tm_year = min->tm_year;
+	    max->tm_mon = min->tm_mon;
+	    max->tm_mday = min->tm_mday;
+	  }
+	  flag |= M_PDR_MINUS;
+	}
+      }
+      break;
+      case '+':
+      { /* enlarge plusRange */
+	pt = get_offset (max, pc, 1);
+	if (pc == pt)
+	  flag |= M_PDR_ERRORDONE;
+	else
+	{
+	  pc = pt;
+	  flag |= M_PDR_PLUS;
+	}
+      }
+      break;
+      case '*':
+      { /* enlarge window in both directions */
+	pt = get_offset (min, pc, -1);
+	if (pc == pt)
+	  flag |= M_PDR_ERRORDONE;
+	else
+	{
+	  pc = get_offset (max, pc, 1);
+	  flag |= M_PDR_WINDOW;
+	}
+      }
+      break;
+      default:
+	flag |= M_PDR_ERRORDONE;
+    }
+    SKIPWS (pc);
+  }
+  if ((flag & M_PDR_ERROR) && !(flag & M_PDR_ABSOLUTE))
+  { /* getDate has its own error message, don't overwrite it here */
+    snprintf (err->data, err->dsize, _("Invalid relative date: %s"), pc-1);
+  }
+  return ((flag & M_PDR_ERROR) ? NULL : pc);
+}
+
 static int eat_date (pattern_t *pat, BUFFER *s, BUFFER *err)
 {
   BUFFER buffer;
@@ -448,9 +578,10 @@ static int eat_date (pattern_t *pat, BUFFER *s, BUFFER *err)
 	exact++;
     }
     tm->tm_hour = 23;
-    tm->tm_min = max.tm_sec = 59;
+    tm->tm_min = tm->tm_sec = 59;
 
-    get_offset (tm, buffer.data + 1);
+    /* force negative offset */
+    get_offset (tm, buffer.data + 1, -1);
 
     if (exact)
     {
@@ -463,7 +594,9 @@ static int eat_date (pattern_t *pat, BUFFER *s, BUFFER *err)
   {
     const char *pc = buffer.data;
 
-    if (*pc != '-')
+    int haveMin = FALSE;
+    int untilNow = FALSE;
+    if (isdigit ((unsigned char)*pc))
     {
       /* mininum date specified */
       if ((pc = getDate (pc, &min, err)) == NULL)
@@ -471,29 +604,47 @@ static int eat_date (pattern_t *pat, BUFFER *s, BUFFER *err)
 	FREE (&buffer.data);
 	return (-1);
       }
+      haveMin = TRUE;
+      SKIPWS (pc);
+      if (*pc == '-')
+      {
+        const char *pt = pc + 1;
+	SKIPWS (pt);
+	untilNow = (*pt == '\0');
+      }
     }
 
-    if (*pc && *pc == '-')
-    {
-      /* max date */
-      pc++; /* skip the `-' */
-      SKIPWS (pc);
-      if (*pc)
-	if (!getDate (pc, &max, err))
-	{
-	  FREE (&buffer.data);
-	  return (-1);
-	}
-    }
-    else
-    {
-      /* search for messages on a specific day */
+    if (!untilNow)
+    { /* max date or relative range/window */
+
+      struct tm baseMin;
+
+      if (!haveMin)
+      { /* save base minimum and set current date, e.g. for "-3d+1d" */
+	time_t now = time (NULL);
+	struct tm *tm = localtime (&now);
+	memcpy (&baseMin, &min, sizeof(baseMin));
+	memcpy (&min, tm, sizeof (min));
+	min.tm_hour = min.tm_sec = min.tm_min = 0;
+      }
+      
+      /* preset max date for relative offsets,
+	 if nothing follows we search for messages on a specific day */
       max.tm_year = min.tm_year;
       max.tm_mon = min.tm_mon;
       max.tm_mday = min.tm_mday;
+
+      if (!parse_date_range (pc, &min, &max, haveMin, &baseMin, err))
+      { /* bail out on any parsing error */
+	FREE (&buffer.data);
+	return (-1);
+      }
     }
   }
 
+  /* Since we allow two dates to be specified we'll have to adjust that. */
+  adjust_date_range (&min, &max);
+
   pat->min = mutt_mktime (&min, 1);
   pat->max = mutt_mktime (&max, 1);
 
@@ -542,11 +693,11 @@ void mutt_pattern_free (pattern_t **pat)
     if (tmp->rx)
     {
       regfree (tmp->rx);
-      safe_free ((void **) &tmp->rx);
+      FREE (&tmp->rx);
     }
     if (tmp->child)
       mutt_pattern_free (&tmp->child);
-    safe_free ((void **) &tmp);
+    FREE (&tmp);
   }
 }
 
@@ -857,9 +1008,9 @@ mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx,
       return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS,
 					pat->alladdr, 1, h->env->cc));
     case M_SUBJECT:
-      return (pat->not ^ (h->env->subject && regexec (pat->rx, h->env->subject, 0, NULL, 0) == 0));
+      return (pat->not ^ (h->env && h->env->subject && regexec (pat->rx, h->env->subject, 0, NULL, 0) == 0));
     case M_ID:
-      return (pat->not ^ (h->env->message_id && regexec (pat->rx, h->env->message_id, 0, NULL, 0) == 0));
+      return (pat->not ^ (h->env && h->env->message_id && regexec (pat->rx, h->env->message_id, 0, NULL, 0) == 0));
     case M_SCORE:
       return (pat->not ^ (h->score >= pat->min && (pat->max == M_MAXRANGE ||
 						   h->score <= pat->max)));
@@ -868,28 +1019,44 @@ mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx,
     case M_REFERENCE:
       return (pat->not ^ match_reference (pat->rx, h->env->references));
     case M_ADDRESS:
-      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS,
+      return (pat->not ^ (h->env && match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS,
 					pat->alladdr, 4, h->env->from,
-					h->env->sender, h->env->to, h->env->cc));
+					h->env->sender, h->env->to, h->env->cc)));
     case M_RECIPIENT:
-      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS,
-					pat->alladdr, 2, h->env->to, h->env->cc));
+           return (pat->not ^ (h->env && match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS,
+					pat->alladdr, 2, h->env->to, h->env->cc)));
     case M_LIST:
-      return (pat->not ^ mutt_is_list_recipient (pat->alladdr, h->env->to, h->env->cc));
+      return (pat->not ^ (h->env && mutt_is_list_recipient (pat->alladdr, h->env->to, h->env->cc)));
     case M_PERSONAL_RECIP:
-      return (pat->not ^ match_user (pat->alladdr, h->env->to, h->env->cc));
+      return (pat->not ^ (h->env && match_user (pat->alladdr, h->env->to, h->env->cc)));
     case M_PERSONAL_FROM:
-      return (pat->not ^ match_user (pat->alladdr, h->env->from, NULL));
+      return (pat->not ^ (h->env && match_user (pat->alladdr, h->env->from, NULL)));
     case M_COLLAPSED:
       return (pat->not ^ (h->collapsed && h->num_hidden > 1));
-#ifdef HAVE_PGP
-   case M_PGP_SIGN:
-     return (pat->not ^ (h->pgp & PGPSIGN));
-   case M_PGP_ENCRYPT:
-     return (pat->not ^ (h->pgp & PGPENCRYPT));
+   case M_CRYPT_SIGN:
+     if (!WithCrypto)
+       break;
+     return (pat->not ^ ((h->security & SIGN) ? 1 : 0));
+   case M_CRYPT_VERIFIED:
+     if (!WithCrypto)
+       break;
+     return (pat->not ^ ((h->security & GOODSIGN) ? 1 : 0));
+   case M_CRYPT_ENCRYPT:
+     if (!WithCrypto)
+       break;
+     return (pat->not ^ ((h->security & ENCRYPT) ? 1 : 0));
    case M_PGP_KEY:
-     return (pat->not ^ (h->pgp & PGPKEY));
-#endif
+     if (!(WithCrypto & APPLICATION_PGP))
+       break;
+     return (pat->not ^ ((h->security & APPLICATION_PGP) && (h->security & PGPKEY)));
+    case M_XLABEL:
+      return (pat->not ^ (h->env->x_label && regexec (pat->rx, h->env->x_label, 0, NULL, 0) == 0));
+    case M_HORMEL:
+      return (pat->not ^ (h->env->spam && h->env->spam->data && regexec (pat->rx, h->env->spam->data, 0, NULL, 0) == 0));
+    case M_DUPLICATED:
+      return (pat->not ^ (h->thread && h->thread->duplicate_thread));
+    case M_UNREFERENCED:
+      return (pat->not ^ (h->thread && !h->thread->child));
   }
   mutt_error (_("error: unknown op %d (report this error)."), pat->op);
   return (-1);
@@ -900,7 +1067,7 @@ static void quote_simple(char *tmp, size_t len, const char *p)
   int i = 0;
   
   tmp[i++] = '"';
-  while (*p && i < len - 2)
+  while (*p && i < len - 3)
   {
     if (*p == '\\' || *p == '"')
       tmp[i++] = '\\';
@@ -915,27 +1082,31 @@ void mutt_check_simple (char *s, size_t len, const char *simple)
 {
   char tmp[LONG_STRING];
 
+  /* XXX - is ascii_strcasecmp() right here, or should we use locale's
+   * equivalences?
+   */
+  
   if (!strchr (s, '~')) /* yup, so spoof a real request */
   {
     /* convert old tokens into the new format */
-    if (mutt_strcasecmp ("all", s) == 0 ||
+    if (ascii_strcasecmp ("all", s) == 0 ||
 	!mutt_strcmp ("^", s) || !mutt_strcmp (".", s)) /* ~A is more efficient */
       strfcpy (s, "~A", len);
-    else if (mutt_strcasecmp ("del", s) == 0)
+    else if (ascii_strcasecmp ("del", s) == 0)
       strfcpy (s, "~D", len);
-    else if (mutt_strcasecmp ("flag", s) == 0)
+    else if (ascii_strcasecmp ("flag", s) == 0)
       strfcpy (s, "~F", len);
-    else if (mutt_strcasecmp ("new", s) == 0)
+    else if (ascii_strcasecmp ("new", s) == 0)
       strfcpy (s, "~N", len);
-    else if (mutt_strcasecmp ("old", s) == 0)
+    else if (ascii_strcasecmp ("old", s) == 0)
       strfcpy (s, "~O", len);
-    else if (mutt_strcasecmp ("repl", s) == 0)
+    else if (ascii_strcasecmp ("repl", s) == 0)
       strfcpy (s, "~Q", len);
-    else if (mutt_strcasecmp ("read", s) == 0)
+    else if (ascii_strcasecmp ("read", s) == 0)
       strfcpy (s, "~R", len);
-    else if (mutt_strcasecmp ("tag", s) == 0)
+    else if (ascii_strcasecmp ("tag", s) == 0)
       strfcpy (s, "~T", len);
-    else if (mutt_strcasecmp ("unread", s) == 0)
+    else if (ascii_strcasecmp ("unread", s) == 0)
       strfcpy (s, "~U", len);
     else
     {
@@ -1027,19 +1198,21 @@ int mutt_pattern_func (int op, char *prompt)
 
   if (op == M_LIMIT)
   {
-    safe_free ((void **) &Context->pattern);
+    FREE (&Context->pattern);
     if (Context->limit_pattern) 
       mutt_pattern_free (&Context->limit_pattern);
     if (!Context->vcount)
     {
-      Context->vcount = Context->msgcount;
       mutt_error _("No messages matched criteria.");
+#if 0
+      Context->vcount = Context->msgcount;
       /* restore full display */
       for (i = 0; i < Context->msgcount; i++)
       {
 	Context->hdrs[i]->virtual = i;
 	Context->v2r[i] = i;
       }
+#endif
     }
     else if (mutt_strncmp (buf, "~A", 2) != 0)
     {
@@ -1067,7 +1240,7 @@ int mutt_search_command (int cur, int op)
   {
     strfcpy (buf, LastSearch, sizeof (buf));
     if (mutt_get_field ((op == OP_SEARCH) ? _("Search for: ") :
-		      "Reverse search for: ", buf, sizeof (buf),
+		      _("Reverse search for: "), buf, sizeof (buf),
 		      M_CLEAR | M_PATTERN) != 0 || !buf[0])
       return (-1);
 
