@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 1996,1997 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 1999-2000 Thomas Roessler <roessler@does-not-exist.org>
+ * Copyright (C) 2004 g10 Code GmbH
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -42,8 +43,6 @@ int pgp_decrypt_mime (FILE *, FILE **, BODY *, BODY **);
 
 /* int pgp_string_matches_hint (const char *s, LIST * hints); */
 
-#define pgp_valid_passphrase() crypt_valid_passphrase(APPLICATION_PGP)
-
 /* pgp_key_t gpg_get_candidates (struct pgp_vinfo *, pgp_ring_t, LIST *); */
 pgp_key_t pgp_ask_for_key (char *, char *, short, pgp_ring_t);
 pgp_key_t pgp_get_candidates (pgp_ring_t, LIST *);
@@ -57,8 +56,7 @@ void pgp_application_pgp_handler (BODY *, STATE *);
 void pgp_encrypted_handler (BODY *, STATE *);
 void pgp_extract_keys_from_attachment_list (FILE * fp, int tag, BODY * top);
 void pgp_void_passphrase (void);
-
-
+int pgp_valid_passphrase (void);
 
 
 /* The PGP invocation interface - not really beautiful. */
@@ -102,5 +100,6 @@ BODY *pgp_traditional_encryptsign (BODY *, int, char *);
 BODY *pgp_encrypt_message (BODY *, char *, int);
 BODY *pgp_sign_message (BODY *);
 
+int pgp_send_menu (HEADER *msg, int *redraw);
 
 #endif /* CRYPT_BACKEND_CLASSIC_PGP */
