@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1997-2000 Thomas Roessler <roessler@guug.de>
+ * Copyright (C) 1997-2001 Thomas Roessler <roessler@does-not-exist.org>
  * 
  *     This program is free software; you can redistribute it
  *     and/or modify it under the terms of the GNU General Public
@@ -34,8 +34,9 @@
  *
  */
 
-
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,13 +46,18 @@
 #ifdef HAVE_GETOPT_H
 # include <getopt.h>
 #endif
+#include <errno.h>
 
 extern char *optarg;
 extern int optind;
 
-#include "sha.h"
+#include "sha1.h"
+#include "md5.h"
 #include "lib.h"
 #include "pgplib.h"
+#include "pgppacket.h"
+
+#define MD5_DIGEST_LENGTH  16
 
 #ifdef HAVE_FGETPOS
 #define FGETPOS(fp,pos) fgetpos((fp),&(pos))
@@ -61,13 +67,13 @@ extern int optind;
 #define FSETPOS(fp,pos) fseek((fp),(pos),SEEK_SET)
 #endif
 
-#define CHUNKSIZE 1024
 
-static unsigned char *pbuf = NULL;
-static size_t plen = 0;
+static short dump_signatures = 0;
+static short dump_fingerprints = 0;
+
 
 static void pgpring_find_candidates (char *ringfile, const char *hints[], int nhints);
-static void pgpring_dump_keyblock (pgp_key_t *p);
+static void pgpring_dump_keyblock (pgp_key_t p);
 
 int main (int argc, char * const argv[])
 {
@@ -82,10 +88,22 @@ int main (int argc, char * const argv[])
   char pgppath[_POSIX_PATH_MAX];
   char kring[_POSIX_PATH_MAX];
 
-  while ((c = getopt (argc, argv, "25sk:")) != EOF)
+  while ((c = getopt (argc, argv, "f25sk:S")) != EOF)
   {
     switch (c)
     {
+      case 'S':
+      {
+	dump_signatures = 1;
+	break;
+      }
+
+      case 'f':
+      {
+	dump_fingerprints = 1;
+	break;
+      }
+
       case 'k':
       {
 	_kring = optarg;
@@ -106,7 +124,7 @@ int main (int argc, char * const argv[])
     
       default:
       {
-	fprintf (stderr, "usage: %s [-k <key ring> | [-2 | -5] [ -s]] [hints]\n",
+	fprintf (stderr, "usage: %s [-k <key ring> | [-2 | -5] [ -s] [-S] [-f]] [hints]\n",
 		 argv[0]);
 	exit (1);
       }
@@ -141,247 +159,38 @@ int main (int argc, char * const argv[])
 
 /* The actual key ring parser */
 
-enum packet_tags
+static void pgp_make_pgp2_fingerprint (unsigned char *buff,
+                                       unsigned char *digest)
 {
-  PT_RES0 = 0,			/* reserved */
-  PT_ESK,			/* Encrypted Session Key */
-  PT_SIG,			/* Signature Packet */
-  PT_CESK,			/* Conventionally Encrypted Session Key Packet */
-  PT_OPS,			/* One-Pass Signature Packet */
-  PT_SECKEY,			/* Secret Key Packet */
-  PT_PUBKEY,			/* Public Key Packet */
-  PT_SUBSECKEY,			/* Secret Subkey Packet */
-  PT_COMPRESSED,		/* Compressed Data Packet */
-  PT_SKE,			/* Symmetrically Encrypted Data Packet */
-  PT_MARKER,			/* Marker Packet */
-  PT_LITERAL,			/* Literal Data Packet */
-  PT_TRUST,			/* Trust Packet */
-  PT_NAME,			/* Name Packet */
-  PT_SUBKEY,			/* Subkey Packet */
-  PT_RES15,			/* Reserved */
-  PT_COMMENT			/* Comment Packet */
-};
 
-/* 
- * These aren't displayed at all currently, but they may come quite
- * handy for debugging strings.
- * 
- * No need to translate them.
- *
- */
+  MD5_CTX context;
+  unsigned int size = 0;
 
-#if 0
 
-const char *pgp_packet_name[] =
+  MD5Init (&context);
+
+  size = (buff[0] << 8) + buff[1];
+  size = ((size + 7) / 8);
+  buff = &buff[2];
+
+  MD5Update (&context, buff, size);
+  buff = &buff[size];
+
+  size = (buff[0] << 8) + buff[1];
+  size = ((size + 7) / 8);
+  buff = &buff[2];
+
+  MD5Update (&context, buff, size);
+
+  MD5Final (digest, &context);
+
+} /* pgp_make_pgp2_fingerprint() */
+
+static pgp_key_t pgp_parse_pgp2_key (unsigned char *buff, size_t l)
 {
-  "reserved",
-  "Encrypted Session Key",
-  "Signature Packet",
-  "Conventionally Encrypted Session Key Packet",
-  "One-Pass Signature Packet",
-  "Secret Key Packet",
-  "Public Key Packet",
-  "Secret Subkey Packet",
-  "Compressed Data Packet",
-  "Symmetrically Encrypted Data Packet",
-  "Marker Packet",
-  "Literal Data Packet",
-  "Trust Packet",
-  "Name Packet",
-  "Subkey Packet",
-  "Reserved",
-  "Comment Packet"
-};
-
-#endif
-
-static int read_material (size_t material, size_t * used, FILE * fp)
-{
-  if (*used + material >= plen)
-  {
-    unsigned char *p;
-    size_t nplen;
-
-    nplen = *used + material + CHUNKSIZE;
-
-    if (!(p = realloc (pbuf, nplen)))
-    {
-      perror ("realloc");
-      return -1;
-    }
-    plen = nplen;
-    pbuf = p;
-  }
-
-  if (fread (pbuf + *used, 1, material, fp) < material)
-  {
-    perror ("fread");
-    return -1;
-  }
-
-  *used += material;
-  return 0;
-}
-
-static unsigned char *pgp_read_packet (FILE * fp, size_t * len)
-{
-  size_t used = 0;
-  long startpos;
-  unsigned char ctb;
-  unsigned char b;
-  size_t material;
-
-  startpos = ftell (fp);
-
-  if (!plen)
-  {
-    plen = CHUNKSIZE;
-    pbuf = safe_malloc (plen);
-  }
-
-  if (fread (&ctb, 1, 1, fp) < 1)
-  {
-    if (!feof (fp))
-      perror ("fread");
-    goto bail;
-  }
-
-  if (!(ctb & 0x80))
-  {
-    goto bail;
-  }
-
-  if (ctb & 0x40)		/* handle PGP 5.0 packets. */
-  {
-    int partial = 0;
-    pbuf[0] = ctb;
-    used++;
-
-    do
-    {
-      if (fread (&b, 1, 1, fp) < 1)
-      {
-	perror ("fread");
-	goto bail;
-      }
-
-      if (b < 192)
-      {
-	material = b;
-	partial = 0;
-	material -= 1;
-      }
-      else if (192 <= b && b <= 223)
-      {
-	material = (b - 192) * 256;
-	if (fread (&b, 1, 1, fp) < 1)
-	{
-	  perror ("fread");
-	  goto bail;
-	}
-	material += b + 192;
-	partial = 0;
-	material -= 2;
-      }
-      else if (b < 255)
-      {
-	material = 1 << (b & 0x1f);
-	partial = 1;
-	material -= 1;
-      }
-      else
-	/* b == 255 */
-      {
-	unsigned char buf[4];
-	if (fread (buf, 4, 1, fp) < 1)
-	{
-	  perror ("fread");
-	  goto bail;
-	}
-	/*assert( sizeof(material) >= 4 ); */
-	material = buf[0] << 24;
-	material |= buf[1] << 16;
-	material |= buf[2] << 8;
-	material |= buf[3];
-	partial = 0;
-	material -= 5;
-      }
-
-      if (read_material (material, &used, fp) == -1)
-	goto bail;
-
-    }
-    while (partial);
-  }
-  else
-    /* Old-Style PGP */
-  {
-    int bytes = 0;
-    pbuf[0] = 0x80 | ((ctb >> 2) & 0x0f);
-    used++;
-
-    switch (ctb & 0x03)
-    {
-      case 0:
-      {
-	if (fread (&b, 1, 1, fp) < 1)
-	{
-	  perror ("fread");
-	  goto bail;
-	}
-
-	material = b;
-	break;
-      }
-
-      case 1:
-      bytes = 2;
-
-      case 2:
-      {
-	int i;
-
-	if (!bytes)
-	  bytes = 4;
-
-	material = 0;
-
-	for (i = 0; i < bytes; i++)
-	{
-	  if (fread (&b, 1, 1, fp) < 1)
-	  {
-	    perror ("fread");
-	    goto bail;
-	  }
-
-	  material = (material << 8) + b;
-	}
-	break;
-      }
-
-      default:
-      goto bail;
-    }
-
-    if (read_material (material, &used, fp) == -1)
-      goto bail;
-  }
-
-  if (len)
-    *len = used;
-
-  return pbuf;
-
-bail:
-
-  fseek (fp, startpos, SEEK_SET);
-  return NULL;
-}
-
-static pgp_key_t *pgp_parse_pgp2_key (unsigned char *buff, size_t l)
-{
-  pgp_key_t *p;
+  pgp_key_t p;
   unsigned char alg;
+  unsigned char digest[MD5_DIGEST_LENGTH];
   size_t expl;
   unsigned long id;
   time_t gen_time = 0;
@@ -412,6 +221,16 @@ static pgp_key_t *pgp_parse_pgp2_key (unsigned char *buff, size_t l)
   p->algorithm = pgp_pkalgbytype (alg);
   p->flags |= pgp_get_abilities (alg);
 
+  if (dump_fingerprints)
+  {
+    /* j now points to the key material, which we need for the fingerprint */
+    p->fp_len = MD5_DIGEST_LENGTH;
+    pgp_make_pgp2_fingerprint (&buff[j], digest);
+    memcpy (p->fingerprint, digest, MD5_DIGEST_LENGTH);
+  }
+  else	/* just to be usre */
+    memset (p->fingerprint, 0, MD5_DIGEST_LENGTH);
+    
   expl = 0;
   for (i = 0; i < 2; i++)
     expl = (expl << 8) + buff[j++];
@@ -440,7 +259,7 @@ static pgp_key_t *pgp_parse_pgp2_key (unsigned char *buff, size_t l)
 
 bailout:
 
-  safe_free ((void **) &p);
+  FREE (&p);
   return NULL;
 }
 
@@ -448,7 +267,7 @@ static void pgp_make_pgp3_fingerprint (unsigned char *buff, size_t l,
 				       unsigned char *digest)
 {
   unsigned char dummy;
-  SHA_CTX context;
+  SHA1_CTX context;
 
   SHA1_Init (&context);
 
@@ -485,9 +304,9 @@ static void skip_bignum (unsigned char *buff, size_t l, size_t j,
 }
 
 
-static pgp_key_t *pgp_parse_pgp3_key (unsigned char *buff, size_t l)
+static pgp_key_t pgp_parse_pgp3_key (unsigned char *buff, size_t l)
 {
-  pgp_key_t *p;
+  pgp_key_t p;
   unsigned char alg;
   unsigned char digest[SHA_DIGEST_LENGTH];
   unsigned char scratch[LONG_STRING];
@@ -519,13 +338,15 @@ static pgp_key_t *pgp_parse_pgp3_key (unsigned char *buff, size_t l)
   len = (buff[j] << 8) + buff[j + 1];
   p->keylen = len;
 
+
   if (alg >= 1 && alg <= 3)
     skip_bignum (buff, l, j, &j, 2);
-  else if (alg == 17 || alg == 16)
+  else if (alg == 17 || alg == 16 || alg == 20)
     skip_bignum (buff, l, j, &j, 1);
 
   pgp_make_pgp3_fingerprint (buff, j, digest);
-
+  p->fp_len = SHA_DIGEST_LENGTH;
+  
   for (k = 0; k < 2; k++)
   {
     for (id = 0, i = SHA_DIGEST_LENGTH - 8 + k * 4;
@@ -540,7 +361,7 @@ static pgp_key_t *pgp_parse_pgp3_key (unsigned char *buff, size_t l)
   return p;
 }
 
-static pgp_key_t *pgp_parse_keyinfo (unsigned char *buff, size_t l)
+static pgp_key_t pgp_parse_keyinfo (unsigned char *buff, size_t l)
 {
   if (!buff || l < 2)
     return NULL;
@@ -557,11 +378,13 @@ static pgp_key_t *pgp_parse_keyinfo (unsigned char *buff, size_t l)
   }
 }
 
-static int pgp_parse_pgp2_sig (unsigned char *buff, size_t l, pgp_key_t * p)
+static int pgp_parse_pgp2_sig (unsigned char *buff, size_t l,
+                               pgp_key_t p, pgp_sig_t *s)
 {
   unsigned char sigtype;
   time_t sig_gen_time;
-  unsigned long signerid;
+  unsigned long signerid1;
+  unsigned long signerid2;
   size_t j;
   int i;
 
@@ -575,18 +398,29 @@ static int pgp_parse_pgp2_sig (unsigned char *buff, size_t l, pgp_key_t * p)
   for (i = 0; i < 4; i++)
     sig_gen_time = (sig_gen_time << 8) + buff[j++];
 
-  j += 4;
-  signerid = 0;
+  signerid1 = signerid2 = 0;
   for (i = 0; i < 4; i++)
-    signerid = (signerid << 8) + buff[j++];
+    signerid1 = (signerid1 << 8) + buff[j++];
 
+  for (i = 0; i < 4; i++)
+    signerid2 = (signerid2 << 8) + buff[j++];
+
+  
   if (sigtype == 0x20 || sigtype == 0x28)
     p->flags |= KEYFLAG_REVOKED;
 
+  if (s)
+  {
+    s->sigtype = sigtype;
+    s->sid1    = signerid1;
+    s->sid2    = signerid2;
+  }
+  
   return 0;
 }
 
-static int pgp_parse_pgp3_sig (unsigned char *buff, size_t l, pgp_key_t * p)
+static int pgp_parse_pgp3_sig (unsigned char *buff, size_t l,
+                               pgp_key_t p, pgp_sig_t *s)
 {
   unsigned char sigtype;
   unsigned char pkalg;
@@ -595,7 +429,8 @@ static int pgp_parse_pgp3_sig (unsigned char *buff, size_t l, pgp_key_t * p)
   time_t sig_gen_time = -1;
   long validity = -1;
   long key_validity = -1;
-  long signerid = 0;
+  unsigned long signerid1 = 0;
+  unsigned long signerid2 = 0;
   size_t ml;
   size_t j;
   int i;
@@ -678,10 +513,12 @@ static int pgp_parse_pgp3_sig (unsigned char *buff, size_t l, pgp_key_t * p)
 	{
 	  if (skl < 8)
 	    break;
-	  j += 4;
-	  signerid = 0;
+	  signerid2 = signerid1 = 0;
 	  for (i = 0; i < 4; i++)
-	    signerid = (signerid << 8) + buff[j++];
+	    signerid1 = (signerid1 << 8) + buff[j++];
+	  for (i = 0; i < 4; i++)
+	    signerid2 = (signerid2 << 8) + buff[j++];
+	  
 	  break;
 	}
 	case 10:			/* CMR key */
@@ -714,12 +551,21 @@ static int pgp_parse_pgp3_sig (unsigned char *buff, size_t l, pgp_key_t * p)
   if (have_critical_spks)
     p->flags |= KEYFLAG_CRITICAL;
 
+  if (s)
+  {
+    s->sigtype = sigtype;
+    s->sid1    = signerid1;
+    s->sid2    = signerid2;
+  }
+
+  
   return 0;
 
 }
 
 
-static int pgp_parse_sig (unsigned char *buff, size_t l, pgp_key_t * p)
+static int pgp_parse_sig (unsigned char *buff, size_t l,
+                          pgp_key_t p, pgp_sig_t *sig)
 {
   if (!buff || l < 2 || !p)
     return -1;
@@ -728,9 +574,9 @@ static int pgp_parse_sig (unsigned char *buff, size_t l, pgp_key_t * p)
   {
   case 2:
   case 3:
-    return pgp_parse_pgp2_sig (buff, l, p);
+    return pgp_parse_pgp2_sig (buff, l, p, sig);      
   case 4:
-    return pgp_parse_pgp3_sig (buff, l, p);
+    return pgp_parse_pgp3_sig (buff, l, p, sig);
   default:
     return -1;
   }
@@ -738,7 +584,7 @@ static int pgp_parse_sig (unsigned char *buff, size_t l, pgp_key_t * p)
 
 /* parse one key block, including all subkeys. */
 
-static pgp_key_t *pgp_parse_keyblock (FILE * fp)
+static pgp_key_t pgp_parse_keyblock (FILE * fp)
 {
   unsigned char *buff;
   unsigned char pt = 0;
@@ -752,11 +598,12 @@ static pgp_key_t *pgp_parse_keyblock (FILE * fp)
   long pos;
 #endif
 
-  pgp_key_t *root = NULL;
-  pgp_key_t **last = &root;
-  pgp_key_t *p = NULL;
+  pgp_key_t root = NULL;
+  pgp_key_t *last = &root;
+  pgp_key_t p = NULL;
   pgp_uid_t *uid = NULL;
   pgp_uid_t **addr = NULL;
+  pgp_sig_t **lsig = NULL;
 
   FGETPOS(fp,pos);
   
@@ -788,6 +635,7 @@ static pgp_key_t *pgp_parse_keyblock (FILE * fp)
 
 	last = &p->next;
 	addr = &p->address;
+	lsig = &p->sigs;
 	
 	if (pt == PT_SUBKEY || pt == PT_SUBSECKEY)
 	{
@@ -808,7 +656,14 @@ static pgp_key_t *pgp_parse_keyblock (FILE * fp)
 
       case PT_SIG:
       {
-	pgp_parse_sig (buff, l, p);
+	if (lsig)
+	{
+	  pgp_sig_t *signature = safe_calloc (sizeof (pgp_sig_t), 1);
+	  *lsig = signature;
+	  lsig = &signature->next;
+	  
+	  pgp_parse_sig (buff, l, p, signature);
+	}
 	break;
       }
 
@@ -846,7 +701,8 @@ static pgp_key_t *pgp_parse_keyblock (FILE * fp)
 	uid->parent = p;
 	uid->trust = 0;
 	addr = &uid->next;
-
+	lsig = &uid->sigs;
+	
 	/* the following tags are generated by
 	 * pgp 2.6.3in.
 	 */
@@ -907,7 +763,14 @@ static void pgpring_find_candidates (char *ringfile, const char *hints[], int nh
   
   if ((rfp = fopen (ringfile, "r")) == NULL)
   {
-    perror ("fopen");
+    char *error_buf;
+    size_t error_buf_len;
+
+    error_buf_len = sizeof ("fopen: ") - 1 + strlen (ringfile) + 1;
+    error_buf = safe_malloc (error_buf_len);
+    snprintf (error_buf, error_buf_len, "fopen: %s", ringfile);
+    perror (error_buf);
+    FREE (&error_buf);
     return;
   }
 
@@ -936,7 +799,7 @@ static void pgpring_find_candidates (char *ringfile, const char *hints[], int nh
 
       if (pgpring_string_matches_hint (tmp, hints, nhints))
       {
-	pgp_key_t *p;
+	pgp_key_t p;
 
 	FSETPOS(rfp, keypos);
 
@@ -949,7 +812,7 @@ static void pgpring_find_candidates (char *ringfile, const char *hints[], int nh
 	pgp_free_key (&p);
       }
 
-      safe_free ((void **) &tmp);
+      FREE (&tmp);
     }
 
     FGETPOS(rfp,pos);
@@ -966,10 +829,37 @@ static void print_userid (const char *id)
     if (*id >= ' ' && *id <= 'z' && *id != ':')
       putchar (*id);
     else
-      printf ("\\x%02x", *id);
+      printf ("\\x%02x", (*id) & 0xff);
   }
 }
 
+static void print_fingerprint (pgp_key_t p) 
+{
+  int i = 0;
+
+  printf ("fpr:::::::::");
+  for (i = 0; i < p->fp_len; i++)
+    printf ("%02X", p->fingerprint[i]);
+  printf (":\n");
+
+} /* print_fingerprint() */
+
+
+static void pgpring_dump_signatures (pgp_sig_t *sig)
+{
+  for (; sig; sig = sig->next)
+  {
+    if (sig->sigtype == 0x10 || sig->sigtype == 0x11 ||
+	sig->sigtype == 0x12 || sig->sigtype == 0x13)
+      printf ("sig::::%08lX%08lX::::::%X:\n",
+	      sig->sid1, sig->sid2, sig->sigtype);
+    else if (sig->sigtype == 0x20)
+      printf ("rev::::%08lX%08lX::::::%X:\n",
+	      sig->sid1, sig->sid2, sig->sigtype);
+  }
+}
+
+
 static char gnupg_trustletter (int t)
 {
   switch (t)
@@ -981,7 +871,7 @@ static char gnupg_trustletter (int t)
   return 'q';
 }
 
-static void pgpring_dump_keyblock (pgp_key_t *p)
+static void pgpring_dump_keyblock (pgp_key_t p)
 {
   pgp_uid_t *uid;
   short first;
@@ -1036,9 +926,35 @@ static void pgpring_dump_keyblock (pgp_key_t *p)
 		1900 + tp->tm_year, tp->tm_mon + 1, tp->tm_mday);
 	
 	print_userid (uid->addr);
+	printf ("::");
+
+	if(pgp_canencrypt(p->numalg))
+	  putchar ('e');
+	if(pgp_cansign(p->numalg))
+	  putchar ('s');
+	if (p->flags & KEYFLAG_DISABLED)
+	  putchar ('D');
 	printf (":\n");
+
+	if (dump_fingerprints) 
+          print_fingerprint (p);
+      }
+      
+      if (dump_signatures)
+      {
+	if (first) pgpring_dump_signatures (p->sigs);
+	pgpring_dump_signatures (uid->sigs);
       }
     }
   }
 }
-	
+
+/*
+ * The mutt_gettext () defined in gettext.c requires iconv,
+ * so we do without charset conversion here.
+ */
+
+char *mutt_gettext (const char *message)
+{
+  return (char *)message;
+}
