@@ -16,6 +16,10 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mx.h"
 #include "url.h"
@@ -270,7 +274,7 @@ int pop_open_connection (POP_DATA *pop_data)
     return -2;
   }
 
-#if defined(USE_SSL) && !defined(USE_NSS)
+#if (defined(USE_SSL) || defined(USE_GNUTLS)) && !defined(USE_NSS)
   /* Attempt STLS if available and desired. */
   if (pop_data->cmd_stls && !pop_data->conn->ssf)
   {
@@ -295,7 +299,11 @@ int pop_open_connection (POP_DATA *pop_data)
 	mutt_error ("%s", pop_data->err_msg);
 	mutt_sleep (2);
       }
+#ifdef USE_SSL
       else if (mutt_ssl_starttls (pop_data->conn))
+#elif USE_GNUTLS
+      else if (mutt_gnutls_starttls (pop_data->conn))
+#endif
       {
 	mutt_error (_("Could not negotiate TLS connection"));
 	mutt_sleep (2);
