@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
- * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
+ * Copyright (C) 1999-2000 Thomas Roessler <roessler@does-not-exist.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,16 +17,22 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_menu.h"
 #include "rfc1524.h"
 #include "mime.h"
 #include "mailbox.h"
 #include "mapping.h"
+#include "sort.h"
 #ifdef USE_IMAP
 #include "mx.h"
 #include "imap.h"
 #endif
+#include "mutt_crypt.h"
 
 #include <ctype.h>
 #include <unistd.h>
@@ -43,11 +49,6 @@ static struct mapping_t PostponeHelp[] = {
 
 
 
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif /* HAVE_PGP */
-
-
 static short PostCount = 0;
 static CONTEXT *PostContext = NULL;
 static short UpdateNumPostponed = 0;
@@ -86,10 +87,16 @@ int mutt_num_postponed (int force)
   {
     if (force)
     {
-      PostCount = imap_mailbox_check (Postponed, 0);
-      PostCount = (PostCount < 0) ? 0 : PostCount;
-      dprint(2, (debugfile,
-        "mutt_num_postponed: %d postponed IMAP messages found.\n", PostCount));
+      short newpc;
+
+      newpc = imap_mailbox_check (Postponed, 0);
+      if (newpc >= 0)
+      {
+	PostCount = newpc;
+	dprint (2, (debugfile, "mutt_num_postponed: %d postponed IMAP messages found.\n", PostCount));
+      }
+      else
+	dprint (2, (debugfile, "mutt_num_postponed: using old IMAP postponed count.\n"));
     }
     return PostCount;
   }
@@ -151,6 +158,7 @@ static HEADER *select_msg (void)
   MUTTMENU *menu;
   int i, done=0, r=-1;
   char helpstr[SHORT_STRING];
+  short orig_sort;
 
   menu = mutt_new_menu ();
   menu->make_entry = post_entry;
@@ -160,6 +168,12 @@ static HEADER *select_msg (void)
   menu->data = PostContext;
   menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_POST, PostponeHelp);
 
+  /* The postponed mailbox is setup to have sorting disabled, but the global
+   * Sort variable may indicate something different.   Sorting has to be
+   * disabled while the postpone menu is being displayed. */
+  orig_sort = Sort;
+  Sort = SORT_ORDER;
+  
   while (!done)
   {
     switch (i = mutt_menuLoop (menu))
@@ -195,6 +209,7 @@ static HEADER *select_msg (void)
     }
   }
 
+  Sort = orig_sort;
   mutt_menuDestroy (&menu);
   return (r > -1 ? PostContext->hdrs[r] : NULL);
 }
@@ -222,28 +237,10 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   LIST *next;
   char *p;
   int opt_delete;
-#ifdef USE_IMAP
-  char curpath[LONG_STRING];
-  int need_reopen = 0;
-#endif
 
   if (!Postponed)
     return (-1);
 
-#ifdef USE_IMAP
-  /* if we're in an IMAP folder and the postponed folder is also IMAP, we may
-   * need to take steps to avoid opening an additional connection to the same
-   * server. */
-  if ((ctx && ctx->magic == M_IMAP) && mx_is_imap (Postponed))
-  { 
-    strfcpy (curpath, ctx->path, sizeof (curpath));
-    if (imap_select_mailbox (ctx, Postponed) < 0)
-      return -1;
-    need_reopen = 1;
-    PostContext = ctx;
-  }
-  else
-#endif
   if ((PostContext = mx_open_mailbox (Postponed, M_NOSORT, NULL)) == NULL)
   {
     PostCount = 0;
@@ -255,12 +252,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   {
     PostCount = 0;
     mx_close_mailbox (PostContext, NULL);
-#ifdef USE_IMAP
-  if (need_reopen)
-    ctx = mx_open_mailbox (curpath, 0, PostContext);
-  else
-#endif
-    safe_free ((void **) &PostContext);
+    FREE (&PostContext);
     mutt_error _("No postponed messages.");
     return (-1);
   }
@@ -273,24 +265,14 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   else if ((h = select_msg ()) == NULL)
   {
     mx_close_mailbox (PostContext, NULL);
-#ifdef USE_IMAP
-  if (need_reopen)
-    ctx = mx_open_mailbox (curpath, 0, PostContext);
-  else
-#endif
-    safe_free ((void **) &PostContext);
+    FREE (&PostContext);
     return (-1);
   }
 
   if (mutt_prepare_template (NULL, PostContext, hdr, h, 0) < 0)
   {
     mx_fastclose_mailbox (PostContext);
-#ifdef USE_IMAP
-    if (need_reopen)
-      ctx = mx_open_mailbox (curpath, 0, NULL);
-    else
-#endif
-    safe_free ((void **) &PostContext);
+    FREE (&PostContext);
     return (-1);
   }
 
@@ -306,16 +288,11 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   mx_close_mailbox (PostContext, NULL);
   set_quadoption (OPT_DELETE, opt_delete);
 
-#ifdef USE_IMAP
-  if (need_reopen)
-    ctx = mx_open_mailbox (curpath, 0, PostContext);
-  else
-#endif
-  safe_free ((void **) &PostContext);
+  FREE (&PostContext);
 
   for (tmp = hdr->env->userhdrs; tmp; )
   {
-    if (mutt_strncasecmp ("X-Mutt-References:", tmp->data, 18) == 0)
+    if (ascii_strncasecmp ("X-Mutt-References:", tmp->data, 18) == 0)
     {
       if (ctx)
       {
@@ -323,6 +300,8 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
 	   the user attempted to reply to is in this mailbox */
 	p = tmp->data + 18;
 	SKIPWS (p);
+	if (!ctx->id_hash)
+	  ctx->id_hash = mutt_make_id_hash (ctx);
 	*cur = hash_find (ctx->id_hash, p);
       }
 
@@ -338,7 +317,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
       if (*cur)
 	code |= SENDREPLY;
     }
-    else if (mutt_strncasecmp ("X-Mutt-Fcc:", tmp->data, 11) == 0)
+    else if (ascii_strncasecmp ("X-Mutt-Fcc:", tmp->data, 11) == 0)
     {
       p = tmp->data + 11;
       SKIPWS (p);
@@ -355,16 +334,14 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
       mutt_free_list (&tmp);
       tmp = next;
     }
-
-
-
-#ifdef HAVE_PGP
-    else if (mutt_strncmp ("Pgp:", tmp->data, 4) == 0 /* this is generated
+    else if ((WithCrypto & APPLICATION_PGP)
+             && (mutt_strncmp ("Pgp:", tmp->data, 4) == 0 /* this is generated
 						       * by old mutt versions
 						       */
-	     || mutt_strncmp ("X-Mutt-PGP:", tmp->data, 11) == 0)
+                 || mutt_strncmp ("X-Mutt-PGP:", tmp->data, 11) == 0))
     {
-      hdr->pgp = mutt_parse_pgp_hdr (strchr (tmp->data, ':') + 1, 1);
+      hdr->security = mutt_parse_crypt_hdr (strchr (tmp->data, ':') + 1, 1);
+      hdr->security |= APPLICATION_PGP;
        
       /* remove the pgp field */
       next = tmp->next;
@@ -376,7 +353,22 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
       mutt_free_list (&tmp);
       tmp = next;
     }
-#endif /* HAVE_PGP */
+    else if ((WithCrypto & APPLICATION_SMIME)
+             && mutt_strncmp ("X-Mutt-SMIME:", tmp->data, 13) == 0)
+    {
+      hdr->security = mutt_parse_crypt_hdr (strchr (tmp->data, ':') + 1, 1);
+      hdr->security |= APPLICATION_SMIME;
+       
+      /* remove the smime field */
+      next = tmp->next;
+      if (last)
+	last->next = tmp->next;
+      else
+	hdr->env->userhdrs = tmp->next;
+      tmp->next = NULL;
+      mutt_free_list (&tmp);
+      tmp = next;
+    }
 
 #ifdef MIXMASTER
     else if (mutt_strncmp ("X-Mutt-Mix:", tmp->data, 11) == 0)
@@ -413,13 +405,14 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
 
 
 
-#ifdef HAVE_PGP
-
-int mutt_parse_pgp_hdr (char *p, int set_signas)
+int mutt_parse_crypt_hdr (char *p, int set_signas)
 {
   int pgp = 0;
   char pgp_sign_as[LONG_STRING] = "\0", *q;
-  char pgp_sign_micalg[LONG_STRING] = "\0";
+  char smime_cryptalg[LONG_STRING] = "\0";
+
+  if (!WithCrypto)
+    return 0;
    
   SKIPWS (p);
   for (; *p; p++)
@@ -429,12 +422,12 @@ int mutt_parse_pgp_hdr (char *p, int set_signas)
     {
       case 'e':
       case 'E':
-        pgp |= PGPENCRYPT;
+        pgp |= ENCRYPT;
         break;
 
       case 's':    
       case 'S':
-        pgp |= PGPSIGN;
+        pgp |= SIGN;
         q = pgp_sign_as;
       
         if (*(p+1) == '<')
@@ -454,16 +447,17 @@ int mutt_parse_pgp_hdr (char *p, int set_signas)
         *q = '\0';
         break;
 
+      /* This used to be the micalg parameter.
+       * 
+       * It's no longer needed, so we just skip the parameter in order
+       * to be able to recall old messages.
+       */
       case 'm':
       case 'M':
-   	q = pgp_sign_micalg;
-	
         if(*(p+1) == '<')
-	{
-	  for(p += 2; *p && *p != '>' && q < pgp_sign_micalg + sizeof(pgp_sign_micalg) - 1;
-	      *q++ = *p++)
+        {
+	  for (p += 2; *p && *p != '>'; p++)
 	    ;
-	  
 	  if(*p != '>')
 	  {
 	    mutt_error _("Illegal PGP header");
@@ -471,9 +465,34 @@ int mutt_parse_pgp_hdr (char *p, int set_signas)
 	  }
 	}
 
-	*q = '\0';
 	break;
 	  
+	  
+      case 'c':
+      case 'C':
+   	q = smime_cryptalg;
+	
+        if(*(p+1) == '<')
+	{
+	  for(p += 2; *p && *p != '>' && q < smime_cryptalg + sizeof(smime_cryptalg) - 1;
+	      *q++ = *p++)
+	    ;
+	  
+	  if(*p != '>')
+	  {
+	    mutt_error _("Illegal S/MIME header");
+	    return 0;
+	  }
+	}
+
+	*q = '\0';
+	break;
+
+      case 'i':
+      case 'I':
+	pgp |= INLINE;
+	break;
+
       default:
         mutt_error _("Illegal PGP header");
         return 0;
@@ -481,26 +500,31 @@ int mutt_parse_pgp_hdr (char *p, int set_signas)
      
   }
  
-  if (set_signas || *pgp_sign_as)
-    mutt_str_replace (&PgpSignAs, pgp_sign_as);
+  /* the cryptalg field must not be empty */
+  if ((WithCrypto & APPLICATION_SMIME) && *smime_cryptalg)
+    mutt_str_replace (&SmimeCryptAlg, smime_cryptalg);
 
-  /* the micalg field must not be empty */
-  if (set_signas && *pgp_sign_micalg)
-    mutt_str_replace (&PgpSignMicalg, pgp_sign_micalg);
+  if ((WithCrypto & APPLICATION_PGP) && (set_signas || *pgp_sign_as))
+    mutt_str_replace (&PgpSignAs, pgp_sign_as);
 
   return pgp;
 }
-#endif /* HAVE_PGP */
+
+
 
 int mutt_prepare_template (FILE *fp, CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
 			       short weed)
 {
   MESSAGE *msg = NULL;
   char file[_POSIX_PATH_MAX];
-  LIST *p, **q;
   BODY *b;
   FILE *bfp;
-
+  
+  int rv = -1;
+  STATE s;
+  
+  memset (&s, 0, sizeof (s));
+  
   if (!fp && (msg = mx_open_message (ctx, hdr->msgno)) == NULL)
     return (-1);
 
@@ -516,43 +540,29 @@ int mutt_prepare_template (FILE *fp, CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
   newhdr->content->length = hdr->content->length;
   mutt_parse_part (fp, newhdr->content);
 
-  /* weed user-agent, x-mailer - we don't want them here */
-  p = newhdr->env->userhdrs; 
-  q = &newhdr->env->userhdrs;
+  FREE (&newhdr->env->message_id);
+  FREE (&newhdr->env->mail_followup_to); /* really? */
 
-  while (p)
-  {
-    if (!strncasecmp (p->data, "x-mailer:", 9) || !strncasecmp (p->data, "user-agent:", 11))
-    {
-      *q = p->next;
-      p->next = NULL;
-      mutt_free_list (&p);
-    }
-    else
-      q = &p->next;
-
-    p = *q;
-  }
-
-  safe_free ((void **) &newhdr->env->message_id);
-  safe_free ((void **) &newhdr->env->mail_followup_to);
-
-#ifdef HAVE_PGP
   /* decrypt pgp/mime encoded messages */
-  if ((hdr->pgp & PGPENCRYPT) && 
-      mutt_is_multipart_encrypted (newhdr->content))
+
+  if ((WithCrypto & (APPLICATION_PGP|APPLICATION_SMIME) & hdr->security)
+      && mutt_is_multipart_encrypted (newhdr->content))
   {
-    newhdr->pgp |= PGPENCRYPT;
-    if (!pgp_valid_passphrase())
+    int ccap = WithCrypto & (APPLICATION_PGP|APPLICATION_SMIME) & hdr->security;
+    newhdr->security |= ENCRYPT | ccap;
+    if (!crypt_valid_passphrase (ccap))
       goto err;
 
-    mutt_message _("Invoking PGP...");
-    if (pgp_decrypt_mime (fp, &bfp, newhdr->content, &b) == -1)
+    mutt_message _("Decrypting message...");
+    if (((ccap & APPLICATION_PGP) && crypt_pgp_decrypt_mime (fp, &bfp, newhdr->content, &b) == -1) 
+	|| ((ccap & APPLICATION_SMIME) && crypt_smime_decrypt_mime (fp, &bfp, newhdr->content, &b) == -1) 
+	|| b == NULL)
     {
  err:
       mx_close_message (&msg);
       mutt_free_envelope (&newhdr->env);
       mutt_free_body (&newhdr->content);
+      mutt_error _("Decryption failed.");
       return -1;
     }
 
@@ -567,15 +577,20 @@ int mutt_prepare_template (FILE *fp, CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
    * resending messages 
    */
   
-  if (mutt_is_multipart_signed (newhdr->content))
+  if (WithCrypto && mutt_is_multipart_signed (newhdr->content))
   {
-    newhdr->pgp |= PGPSIGN;
+    newhdr->security |= SIGN;
+    if ((WithCrypto & APPLICATION_PGP)
+        && ascii_strcasecmp (mutt_get_parameter ("protocol", newhdr->content->parameter), "application/pgp-signature") == 0)
+      newhdr->security |= APPLICATION_PGP;
+    else if ((WithCrypto & APPLICATION_SMIME))
+      newhdr->security |= APPLICATION_SMIME;
     
     /* destroy the signature */
     mutt_free_body (&newhdr->content->parts->next);
     newhdr->content = mutt_remove_multipart (newhdr->content);
   }
-#endif
+
 
   /* 
    * We don't need no primary multipart.
@@ -590,6 +605,8 @@ int mutt_prepare_template (FILE *fp, CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
   if (newhdr->content->type == TYPEMULTIPART)
     newhdr->content = mutt_remove_multipart (newhdr->content);
 
+  s.fpin = bfp;
+  
   /* create temporary files for all attachments */
   for (b = newhdr->content; b; b = b->next)
   {
@@ -605,34 +622,91 @@ int mutt_prepare_template (FILE *fp, CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
       b->d_filename = safe_strdup (b->filename);
     }
     else
+    {
       /* avoid Content-Disposition: header with temporary filename */
       b->use_disp = 0;
+    }
+
+    /* set up state flags */
+
+    s.flags = 0;
+
+    if (b->type == TYPETEXT)
+    {
+      if (!ascii_strcasecmp ("yes", mutt_get_parameter ("x-mutt-noconv", b->parameter)))
+	b->noconv = 1;
+      else
+      {
+	s.flags |= M_CHARCONV;
+	b->noconv = 0;
+      }
+
+      mutt_delete_parameter ("x-mutt-noconv", &b->parameter);
+    }
 
     mutt_adv_mktemp (file, sizeof(file));
-    if (mutt_save_attachment (bfp, b, file, 0, NULL) == -1)
-    {
-      mutt_free_envelope (&newhdr->env);
-      mutt_free_body (&newhdr->content);
-      if (bfp != fp) fclose (bfp);
-      if (msg) mx_close_message (&msg);
-      return -1;
-    }
+    if ((s.fpout = safe_fopen (file, "w")) == NULL)
+      goto bail;
+
     
+    if ((WithCrypto & APPLICATION_PGP) 
+	&& (mutt_is_application_pgp (b) & (ENCRYPT|SIGN)))
+    {
+      
+      mutt_body_handler (b, &s);
+
+      newhdr->security |= mutt_is_application_pgp (newhdr->content);
+
+      b->type = TYPETEXT;
+      mutt_str_replace (&b->subtype, "plain");
+      mutt_delete_parameter ("x-action", &b->parameter);
+    }
+    else
+      mutt_decode_attachment (b, &s);
+
+    if (safe_fclose (&s.fpout) != 0)
+      goto bail;
+
     mutt_str_replace (&b->filename, file);
     b->unlink = 1;
 
-    if (mutt_is_text_type (b->type, b->subtype))
-      b->noconv = 1;
-      
     mutt_stamp_attachment (b);
 
     mutt_free_body (&b->parts);
     if (b->hdr) b->hdr->content = NULL; /* avoid dangling pointer */
   }
 
+  /* Fix encryption flags. */
+  
+  /* No inline if multipart. */
+  if (WithCrypto && (newhdr->security & INLINE) && newhdr->content->next)
+    newhdr->security &= ~INLINE;
+  
+  /* Do we even support multiple mechanisms? */
+  newhdr->security &= WithCrypto | ~(APPLICATION_PGP|APPLICATION_SMIME);
+  
+  /* Theoretically, both could be set. Take the one the user wants to set by default. */
+  if ((newhdr->security & APPLICATION_PGP) && (newhdr->security & APPLICATION_SMIME))
+  {
+    if (option (OPTSMIMEISDEFAULT))
+      newhdr->security &= ~APPLICATION_PGP;
+    else
+      newhdr->security &= ~APPLICATION_SMIME;
+  }
+
+  rv = 0;
+  
+  bail:
+  
   /* that's it. */
   if (bfp != fp) fclose (bfp);
   if (msg) mx_close_message (&msg);
-
-  return 0;
+  
+  if (rv == -1)
+  {
+    mutt_free_envelope (&newhdr->env);
+    mutt_free_body (&newhdr->content);
+  }
+  
+  return rv;
 }
