@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -16,8 +16,13 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_menu.h"
+#include "mutt_idna.h"
 #include "mapping.h"
 #include "sort.h"
 
@@ -57,14 +62,15 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf);
 
 static ADDRESS *result_to_addr (QUERY *r)
 {
-  static ADDRESS tmp;
+  static ADDRESS *tmp;
   
-  tmp = *r->addr;
+  tmp = rfc822_cpy_adr (r->addr);
   
-  if(!tmp.next && !tmp.personal)
-    tmp.personal = r->name;
+  if(!tmp->next && !tmp->personal)
+    tmp->personal = safe_strdup (r->name);
   
-  return &tmp;
+  mutt_addrlist_to_idna (tmp, NULL);
+  return tmp;
 }
 
 static QUERY *run_query (char *s, int quiet)
@@ -73,7 +79,9 @@ static QUERY *run_query (char *s, int quiet)
   QUERY *first = NULL;
   QUERY *cur = NULL;
   char cmd[_POSIX_PATH_MAX];
-  char buf[STRING];
+  char *buf = NULL;
+  size_t buflen;
+  int dummy = 0;
   char msg[STRING];
   char *p;
   pid_t thepid;
@@ -89,8 +97,10 @@ static QUERY *run_query (char *s, int quiet)
   }
   if (!quiet)
     mutt_message _("Waiting for response...");
-  fgets (msg, sizeof (msg) - 1, fp);
-  while (fgets(buf, sizeof (buf) - 1, fp))
+  fgets (msg, sizeof (msg), fp);
+  if ((p = strrchr (msg, '\n')))
+    *p = '\0';
+  while ((buf = mutt_read_line (buf, &buflen, fp, &dummy)) != NULL)
   {
     if ((p = strtok(buf, "\t\n")))
     {
@@ -107,7 +117,7 @@ static QUERY *run_query (char *s, int quiet)
 	cur = cur->next;
       }
 
-      l = mutt_strlen (p);
+      l = mutt_strwidth (p);
       if (l > SecondColumn)
 	SecondColumn = l;
 	
@@ -115,7 +125,7 @@ static QUERY *run_query (char *s, int quiet)
       p = strtok(NULL, "\t\n");
       if (p)
       {
-	l = mutt_strlen (p);
+	l = mutt_strwidth (p);
 	if (l > FirstColumn)
 	  FirstColumn = l;
 	cur->name = safe_strdup (p);
@@ -127,6 +137,7 @@ static QUERY *run_query (char *s, int quiet)
       }
     }
   }
+  FREE (&buf);
   fclose (fp);
   if (mutt_wait_filter (thepid))
   {
@@ -146,45 +157,78 @@ static int query_search (MUTTMENU *m, regex_t *re, int n)
 {
   ENTRY *table = (ENTRY *) m->data;
 
-  return (regexec (re, table[n].data->name, 0, NULL, 0));
+  if (table[n].data->name && !regexec (re, table[n].data->name, 0, NULL, 0))
+    return 0;
+  if (table[n].data->other && !regexec (re, table[n].data->other, 0, NULL, 0))
+    return 0;
+  if (table[n].data->addr)
+  {
+    if (table[n].data->addr->personal && 
+	!regexec (re, table[n].data->addr->personal, 0, NULL, 0))
+      return 0;
+    if (table[n].data->addr->mailbox &&
+	!regexec (re, table[n].data->addr->mailbox, 0, NULL, 0))
+      return 0;
+#ifdef EXACT_ADDRESS
+    if (table[n].data->addr->val &&
+	!regexec (re, table[n].data->addr->val, 0, NULL, 0))
+      return 0;
+#endif
+  }
+  
+  return REG_NOMATCH;
 }
 
 /* This is the callback routine from mutt_menuLoop() which is used to generate
  * a menu entry for the requested item number.
  */
+#define QUERY_MIN_COLUMN_LENGHT 20 /* Must be < 70/2 */
 static void query_entry (char *s, size_t slen, MUTTMENU *m, int num)
 {
   ENTRY *table = (ENTRY *) m->data;
-  char buf[SHORT_STRING] = "";
-  
+  char buf2[SHORT_STRING], buf[SHORT_STRING] = "";
+
+  /* need a query format ... hard coded constants are not good */
   while (FirstColumn + SecondColumn > 70)
   {
     FirstColumn = FirstColumn * 3 / 4;
     SecondColumn = SecondColumn * 3 / 4;
+    if (FirstColumn < QUERY_MIN_COLUMN_LENGHT)
+      FirstColumn = QUERY_MIN_COLUMN_LENGHT;
+    if (SecondColumn < QUERY_MIN_COLUMN_LENGHT)
+      SecondColumn = QUERY_MIN_COLUMN_LENGHT;
   }
 
-  rfc822_write_address (buf, sizeof (buf), table[num].data->addr);
+  rfc822_write_address (buf, sizeof (buf), table[num].data->addr, 1);
 
-  snprintf (s, slen, " %c %3d %-*.*s %-*.*s %s", 
+  mutt_format_string (buf2, sizeof (buf2),
+		      FirstColumn + 2, FirstColumn + 2,
+		      0, ' ', table[num].data->name,
+		      mutt_strlen (table[num].data->name), 0);
+
+  snprintf (s, slen, " %c %3d %s %-*.*s %s", 
 	    table[num].tagged ? '*':' ',
 	    num+1,
-	    FirstColumn+2,
-	    FirstColumn+2,
-	    table[num].data->name,
+	    buf2,
 	    SecondColumn+2,
 	    SecondColumn+2,
 	    buf,
-	    table[num].data->other);
+	    NONULL (table[num].data->other));
 }
 
-static int query_tag (MUTTMENU *menu, int n)
+static int query_tag (MUTTMENU *menu, int n, int m)
 {
-  return ((((ENTRY *) menu->data)[n].tagged = !((ENTRY *) menu->data)[n].tagged) ? 1 : -1);
+  ENTRY *cur = &((ENTRY *) menu->data)[n];
+  int ot = cur->tagged;
+  
+  cur->tagged = m >= 0 ? m : !cur->tagged;
+  return cur->tagged - ot;
 }
 
 int mutt_query_complete (char *buf, size_t buflen)
 {
   QUERY *results = NULL;
+  ADDRESS *tmpa;
 
   if (!QueryCmd)
   {
@@ -198,8 +242,11 @@ int mutt_query_complete (char *buf, size_t buflen)
     /* only one response? */
     if (results->next == NULL)
     {
+      tmpa = result_to_addr (results);
+      mutt_addrlist_to_local (tmpa);
       buf[0] = '\0';
-      rfc822_write_address (buf, buflen, result_to_addr(results));
+      rfc822_write_address (buf, buflen, tmpa, 0);
+      rfc822_free_address (&tmpa);
       mutt_clear_error ();
       return (0);
     }
@@ -261,9 +308,6 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 
   if (results)
   {
-    /* tell whoever called me to redraw the screen when I return */
-    set_option (OPTNEEDREDRAW);
-
     snprintf (title, sizeof (title), _("Query '%s'"), buf);
 
     /* count the number of results */
@@ -298,14 +342,14 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 		while (queryp)
 		{
 		  rfc822_free_address (&queryp->addr);
-		  safe_free ((void **)&queryp->name);
-		  safe_free ((void **)&queryp->other);
+		  FREE (&queryp->name);
+		  FREE (&queryp->other);
 		  results = queryp->next;
-		  safe_free ((void **)&queryp);
+		  FREE (&queryp);
 		  queryp = results;
 		}
 		results = newresults;
-		safe_free ((void **) &QueryTable);
+		FREE (&QueryTable);
 	      }
 	      else
 	      {
@@ -344,7 +388,7 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 		int clear = 0;
 
 		/* append */
-		safe_realloc ((void **)&QueryTable, menu->max * sizeof (ENTRY));
+		safe_realloc (&QueryTable, menu->max * sizeof (ENTRY));
 
 		menu->data = QueryTable;
 
@@ -371,13 +415,19 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 
 	    for (i = 0; i < menu->max; i++)
 	      if (QueryTable[i].tagged)
-		rfc822_append (&naddr, result_to_addr(QueryTable[i].data));
+	      {
+		ADDRESS *a = result_to_addr(QueryTable[i].data);
+		rfc822_append (&naddr, a);
+		rfc822_free_address (&a);
+	      }
 
 	    mutt_create_alias (NULL, naddr);
 	  }
 	  else
 	  {
-	    mutt_create_alias (NULL, result_to_addr(QueryTable[menu->current].data));
+	    ADDRESS *a = result_to_addr(QueryTable[menu->current].data);
+	    mutt_create_alias (NULL, a);
+	    rfc822_free_address (&a);
 	  }
 	  break;
 
@@ -394,14 +444,17 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 	  msg->env = mutt_new_envelope ();
 	  if (!menu->tagprefix)
 	  {
-	    msg->env->to = 
-	      rfc822_cpy_adr (result_to_addr(QueryTable[menu->current].data));
+	    msg->env->to = result_to_addr(QueryTable[menu->current].data);
 	  }
 	  else
 	  {
 	    for (i = 0; i < menu->max; i++)
 	      if (QueryTable[i].tagged)
-		rfc822_append (&msg->env->to, result_to_addr(QueryTable[i].data));
+	      {
+		ADDRESS *a = result_to_addr(QueryTable[i].data);
+		rfc822_append (&msg->env->to, a);
+		rfc822_free_address (&a);
+	      }
 	  }
 	  ci_send_message (0, msg, NULL, Context, NULL);
 	  menu->redraw = REDRAW_FULL;
@@ -428,37 +481,50 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 	{
 	  if (curpos == 0)
 	  {
+	    ADDRESS *tmpa = result_to_addr (QueryTable[i].data);
+	    mutt_addrlist_to_local (tmpa);
 	    tagged = 1;
-	    rfc822_write_address (buf, buflen, result_to_addr(QueryTable[i].data));
+	    rfc822_write_address (buf, buflen, tmpa, 0);
 	    curpos = mutt_strlen (buf);
+	    rfc822_free_address (&tmpa);
 	  }
 	  else if (curpos + 2 < buflen)
 	  {
-	    strcat (buf, ", ");
+	    ADDRESS *tmpa = result_to_addr (QueryTable[i].data);
+	    mutt_addrlist_to_local (tmpa);
+	    strcat (buf, ", ");	/* __STRCAT_CHECKED__ */
 	    rfc822_write_address ((char *) buf + curpos + 1, buflen - curpos - 1,
-				  result_to_addr(QueryTable[i].data));
+				  tmpa, 0);
 	    curpos = mutt_strlen (buf);
+	    rfc822_free_address (&tmpa);
 	  }
 	}
       }
       /* then enter current message */
       if (!tagged)
       {
-	rfc822_write_address (buf, buflen, result_to_addr(QueryTable[menu->current].data));
+	ADDRESS *tmpa = result_to_addr (QueryTable[menu->current].data);
+	mutt_addrlist_to_local (tmpa);
+	rfc822_write_address (buf, buflen, tmpa, 0);
+	rfc822_free_address (&tmpa);
       }
+      
     }
 
     queryp = results;
     while (queryp)
     {
       rfc822_free_address (&queryp->addr);
-      safe_free ((void **)&queryp->name);
-      safe_free ((void **)&queryp->other);
+      FREE (&queryp->name);
+      FREE (&queryp->other);
       results = queryp->next;
-      safe_free ((void **)&queryp);
+      FREE (&queryp);
       queryp = results;
     }
-    safe_free ((void **) &QueryTable);
+    FREE (&QueryTable);
+    
+    /* tell whoever called me to redraw the screen when I return */
+    set_option (OPTNEEDREDRAW);
   }
 
   mutt_menuDestroy (&menu);
