@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
- * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
+ * Copyright (C) 1999-2000 Thomas Roessler <roessler@does-not-exist.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,10 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mutt_menu.h"
@@ -27,14 +31,7 @@
 #include "mapping.h"
 #include "mx.h"
 #include "copy.h"
-
-
-
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
-
-
+#include "mutt_crypt.h"
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -111,36 +108,40 @@ ATTACHPTR **mutt_gen_attach_list (BODY *m,
 				  int compose)
 {
   ATTACHPTR *new;
-
+  int i;
+  
   for (; m; m = m->next)
   {
     if (*idxlen == *idxmax)
-      safe_realloc ((void **) &idx, sizeof (ATTACHPTR *) * (*idxmax += 5));
+    {
+      safe_realloc (&idx, sizeof (ATTACHPTR *) * ((*idxmax) += 5));
+      for (i = *idxlen; i < *idxmax; i++)
+	idx[i] = NULL;
+    }
 
     if (m->type == TYPEMULTIPART && m->parts
-	&& (compose || (parent_type == -1 && mutt_strcasecmp ("alternative", m->subtype)))
-#ifdef HAVE_PGP
-	&& !mutt_is_multipart_encrypted(m)
-#endif
+	&& (compose || (parent_type == -1 && ascii_strcasecmp ("alternative", m->subtype)))
+        && (!(WithCrypto & APPLICATION_PGP) || !mutt_is_multipart_encrypted(m))
 	)
     {
       idx = mutt_gen_attach_list (m->parts, m->type, idx, idxlen, idxmax, level, compose);
     }
     else
     {
-      new = idx[(*idxlen)++] = (ATTACHPTR *) safe_calloc (1, sizeof (ATTACHPTR));
+      if (!idx[*idxlen])
+	idx[*idxlen] = (ATTACHPTR *) safe_calloc (1, sizeof (ATTACHPTR));
+
+      new = idx[(*idxlen)++];
       new->content = m;
+      m->aptr = new;
       new->parent_type = parent_type;
       new->level = level;
-      /* called when creating new menu, so clear the tagged indicator */
-      m->tagged = 0;
 
       /* We don't support multipart messages in the compose menu yet */
-      if (!compose && 
+      if (!compose && !m->collapsed && 
 	  ((m->type == TYPEMULTIPART
-#ifdef HAVE_PGP
-	    && !mutt_is_multipart_encrypted (m)
-#endif
+            && (!(WithCrypto & APPLICATION_PGP)
+                || !mutt_is_multipart_encrypted (m))
 	    )
 	   || mutt_is_message_type(m->type, m->subtype)))
       {
@@ -163,6 +164,7 @@ ATTACHPTR **mutt_gen_attach_list (BODY *m,
  * %f = filename
  * %I = content-disposition, either I (inline) or A (attachment)
  * %t = tagged flag
+ * %T = tree chars
  * %m = major MIME type
  * %M = MIME subtype
  * %n = attachment number
@@ -191,15 +193,14 @@ const char *mutt_attach_fmt (char *dest,
     case 'C':
       if (!optional)
       {
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	if (mutt_is_text_type (aptr->content->type, aptr->content->subtype) &&
-	    mutt_get_send_charset (charset, sizeof (charset), aptr->content, 0))
-	  snprintf (dest, destlen, fmt, charset);
+	if (mutt_is_text_part (aptr->content) &&
+	    mutt_get_body_charset (charset, sizeof (charset), aptr->content))
+	  mutt_format_s (dest, destlen, prefix, charset);
 	else
-	  snprintf (dest, destlen, fmt, "");
+	  mutt_format_s (dest, destlen, prefix, "");
       }
-      else if (!mutt_is_text_type (aptr->content->type, aptr->content->subtype) ||
-	       !mutt_get_send_charset (charset, sizeof (charset), aptr->content, 0))
+      else if (!mutt_is_text_part (aptr->content) ||
+	       !mutt_get_body_charset (charset, sizeof (charset), aptr->content))
         optional = 0;
       break;
     case 'c':
@@ -216,10 +217,9 @@ const char *mutt_attach_fmt (char *dest,
     case 'd':
       if(!optional)
       {
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
 	if (aptr->content->description)
 	{
-	  snprintf (dest, destlen, fmt, aptr->content->description);
+	  mutt_format_s (dest, destlen, prefix, aptr->content->description);
 	  break;
 	}
 	if (mutt_is_message_type(aptr->content->type, aptr->content->subtype) &&
@@ -230,13 +230,13 @@ const char *mutt_attach_fmt (char *dest,
 			     M_FORMAT_FORCESUBJ | M_FORMAT_MAKEPRINT | M_FORMAT_ARROWCURSOR);
 	  if (*s)
 	  {
-	    snprintf (dest, destlen, fmt, s);
+	    mutt_format_s (dest, destlen, prefix, s);
 	    break;
 	  }
 	}
 	if (!aptr->content->filename)
 	{
-	  snprintf (dest, destlen, fmt, "<no description>");
+	  mutt_format_s (dest, destlen, prefix, "<no description>");
 	  break;
 	}
       }
@@ -248,17 +248,16 @@ const char *mutt_attach_fmt (char *dest,
     case 'f':
       if(!optional)
       {
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
 	if (aptr->content->filename && *aptr->content->filename == '/')
 	{
 	  char path[_POSIX_PATH_MAX];
 	  
 	  strfcpy (path, aptr->content->filename, sizeof (path));
 	  mutt_pretty_mailbox (path);
-	  snprintf (dest, destlen, fmt, path);
+	  mutt_format_s (dest, destlen, prefix, path);
 	}
 	else
-	  snprintf (dest, destlen, fmt, NONULL (aptr->content->filename));
+	  mutt_format_s (dest, destlen, prefix, NONULL (aptr->content->filename));
       }
       else if(!aptr->content->filename)
         optional = 0;
@@ -271,10 +270,8 @@ const char *mutt_attach_fmt (char *dest,
       break;
     case 'e':
       if(!optional)
-      {
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, ENCODING (aptr->content->encoding));
-      }
+	mutt_format_s (dest, destlen, prefix,
+		      ENCODING (aptr->content->encoding));
       break;
     case 'I':
       if (!optional)
@@ -285,17 +282,11 @@ const char *mutt_attach_fmt (char *dest,
       break;
     case 'm':
       if(!optional)
-      {
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, TYPE (aptr->content));
-      }
+	mutt_format_s (dest, destlen, prefix, TYPE (aptr->content));
       break;
     case 'M':
       if(!optional)
-      {
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, aptr->content->subtype);
-      } 
+	mutt_format_s (dest, destlen, prefix, aptr->content->subtype);
       else if(!aptr->content->subtype)
         optional = 0;
       break;
@@ -319,8 +310,7 @@ const char *mutt_attach_fmt (char *dest,
       if(!optional)
       {
 	mutt_pretty_size (tmp, sizeof(tmp), l);
-	snprintf (fmt, sizeof(fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, tmp);
+	mutt_format_s (dest, destlen, prefix, tmp);
       }
       else if (l == 0)
         optional = 0;
@@ -334,10 +324,7 @@ const char *mutt_attach_fmt (char *dest,
       break;
     case 'T':
       if(!optional)
-      {
-	snprintf (fmt, sizeof(fmt), "%%%ss", prefix);
-        snprintf (dest, destlen, fmt, NONULL (aptr->tree));
-      } 
+	mutt_format_s_tree (dest, destlen, prefix, NONULL (aptr->tree));
       else if (!aptr->tree)
         optional = 0;
       break;
@@ -363,9 +350,13 @@ void attach_entry (char *b, size_t blen, MUTTMENU *menu, int num)
   mutt_FormatString (b, blen, NONULL (AttachFormat), mutt_attach_fmt, (unsigned long) (((ATTACHPTR **)menu->data)[num]), M_FORMAT_ARROWCURSOR);
 }
 
-int mutt_tag_attach (MUTTMENU *menu, int n)
+int mutt_tag_attach (MUTTMENU *menu, int n, int m)
 {
-  return ((((ATTACHPTR **) menu->data)[n]->content->tagged = !((ATTACHPTR **) menu->data)[n]->content->tagged) ? 1 : -1);
+  BODY *cur = ((ATTACHPTR **) menu->data)[n]->content;
+  int ot = cur->tagged;
+  
+  cur->tagged = (m >= 0 ? m : !cur->tagged);
+  return cur->tagged - ot;
 }
 
 int mutt_is_message_type (int type, const char *subtype)
@@ -374,17 +365,24 @@ int mutt_is_message_type (int type, const char *subtype)
     return 0;
 
   subtype = NONULL(subtype);
-  return (mutt_strcasecmp (subtype, "rfc822") == 0 || mutt_strcasecmp (subtype, "news") == 0);
+  return (ascii_strcasecmp (subtype, "rfc822") == 0 || ascii_strcasecmp (subtype, "news") == 0);
 }
 
-static int mutt_query_save_attachment (FILE *fp, BODY *body, HEADER *hdr)
+static int mutt_query_save_attachment (FILE *fp, BODY *body, HEADER *hdr, char **directory)
 {
+  char *prompt;
   char buf[_POSIX_PATH_MAX], tfile[_POSIX_PATH_MAX];
   int is_message;
   int append = 0;
-
-  if (body->filename)
-    strfcpy (buf, body->filename, sizeof (buf));
+  int rc;
+  
+  if (body->filename) 
+  {
+    if (directory && *directory)
+      mutt_concat_path (buf, *directory, mutt_basename (body->filename), sizeof (buf));
+    else
+      strfcpy (buf, body->filename, sizeof (buf));
+  }
   else if(body->hdr &&
 	  body->encoding != ENCBASE64 &&
 	  body->encoding != ENCQUOTEDPRINTABLE &&
@@ -393,48 +391,68 @@ static int mutt_query_save_attachment (FILE *fp, BODY *body, HEADER *hdr)
   else
     buf[0] = 0;
 
-  if (mutt_get_field (_("Save to file: "), buf, sizeof (buf), M_FILE | M_CLEAR) != 0
-      || !buf[0])
-    return -1;
-
-  mutt_expand_path (buf, sizeof (buf));
-
-  is_message = (fp && 
-      body->hdr && 
-      body->encoding != ENCBASE64 && 
-      body->encoding != ENCQUOTEDPRINTABLE && 
-      mutt_is_message_type (body->type, body->subtype));
-  
-  if (is_message)
+  prompt = _("Save to file: ");
+  while (prompt)
   {
-    struct stat st;
+    if (mutt_get_field (prompt, buf, sizeof (buf), M_FILE | M_CLEAR) != 0
+	|| !buf[0])
+      return -1;
     
-    /* check to make sure that this file is really the one the user wants */
-    if (!mutt_save_confirm (buf, &st))
+    prompt = NULL;
+    mutt_expand_path (buf, sizeof (buf));
+    
+    is_message = (fp && 
+		  body->hdr && 
+		  body->encoding != ENCBASE64 && 
+		  body->encoding != ENCQUOTEDPRINTABLE && 
+		  mutt_is_message_type (body->type, body->subtype));
+    
+    if (is_message)
     {
-      CLEARLINE (LINES-1);
-      return -1;
+      struct stat st;
+      
+      /* check to make sure that this file is really the one the user wants */
+      if ((rc = mutt_save_confirm (buf, &st)) == 1)
+      {
+	prompt = _("Save to file: ");
+	continue;
+      } 
+      else if (rc == -1)
+	return -1;
+      strfcpy(tfile, buf, sizeof(tfile));
+    }
+    else
+    {
+      if ((rc = mutt_check_overwrite (body->filename, buf, tfile, sizeof (tfile), &append, directory)) == -1)
+	return -1;
+      else if (rc == 1)
+      {
+	prompt = _("Save to file: ");
+	continue;
+      }
+    }
+    
+    mutt_message _("Saving...");
+    if (mutt_save_attachment (fp, body, tfile, append, (hdr || !is_message) ? hdr : body->hdr) == 0)
+    {
+      mutt_message _("Attachment saved.");
+      return 0;
+    }
+    else
+    {
+      prompt = _("Save to file: ");
+      continue;
     }
-    strfcpy(tfile, buf, sizeof(tfile));
   }
-  else
-    if (mutt_check_overwrite (body->filename, buf, tfile, sizeof (tfile), &append))
-      return -1;
-
-  mutt_message _("Saving...");
-  if (mutt_save_attachment (fp, body, tfile, append, (hdr || !is_message) ? hdr : body->hdr) == 0)
-  {
-    mutt_message _("Attachment saved.");
-    return 0;
-  }
-  else
-    return -1;
+  return 0;
 }
-
-void mutt_save_attachment_list (FILE *fp, int tag, BODY *top, HEADER *hdr)
+    
+void mutt_save_attachment_list (FILE *fp, int tag, BODY *top, HEADER *hdr, MUTTMENU *menu)
 {
   char buf[_POSIX_PATH_MAX], tfile[_POSIX_PATH_MAX];
+  char *directory = NULL;
   int rc = 1;
+  int last = menu ? menu->current : -1;
   FILE *fpout;
 
   buf[0] = 0;
@@ -455,7 +473,7 @@ void mutt_save_attachment_list (FILE *fp, int tag, BODY *top, HEADER *hdr)
 	    return;
 	  mutt_expand_path (buf, sizeof (buf));
 	  if (mutt_check_overwrite (top->filename, buf, tfile,
-				    sizeof (tfile), &append))
+				    sizeof (tfile), &append, NULL))
 	    return;
 	  rc = mutt_save_attachment (fp, top, tfile, append, hdr);
 	  if (rc == 0 && AttachSep && (fpout = fopen (tfile,"a")) != NULL)
@@ -474,17 +492,39 @@ void mutt_save_attachment_list (FILE *fp, int tag, BODY *top, HEADER *hdr)
 	  }
 	}
       }
-      else
-	mutt_query_save_attachment (fp, top, hdr);
+      else 
+      {
+	if (tag && menu && top->aptr)
+	{
+	  menu->oldcurrent = menu->current;
+	  menu->current = top->aptr->num;
+	  menu_check_recenter (menu);
+	  menu->redraw |= REDRAW_MOTION;
+
+	  menu_redraw (menu);
+	}
+	if (mutt_query_save_attachment (fp, top, hdr, &directory) == -1)
+	  break;
+      }
     }
     else if (top->parts)
-      mutt_save_attachment_list (fp, 1, top->parts, hdr);
+      mutt_save_attachment_list (fp, 1, top->parts, hdr, menu);
     if (!tag)
-      return;
+      break;
   }
 
+  FREE (&directory);
+
+  if (tag && menu)
+  {
+    menu->oldcurrent = menu->current;
+    menu->current = last;
+    menu_check_recenter (menu);
+    menu->redraw |= REDRAW_MOTION;
+  }
+  
   if (!option (OPTATTACHSPLIT) && (rc == 0))
-    mutt_message _("Attachment saved");
+    mutt_message _("Attachment saved.");
 }
 
 static void
@@ -589,7 +629,7 @@ void mutt_pipe_attachment_list (FILE *fp, int tag, BODY *top, int filter)
 
   if (!filter && !option (OPTATTACHSPLIT))
   {
-    endwin ();
+    mutt_endwin (NULL);
     thepid = mutt_create_filter (buf, &state.fpout, NULL, NULL);
     pipe_attachment_list (buf, fp, tag, top, filter, &state);
     fclose (state.fpout);
@@ -611,8 +651,8 @@ static int can_print (BODY *top, int tag)
     {
       if (!rfc1524_mailcap_lookup (top, type, NULL, M_PRINT))
       {
-	if (mutt_strcasecmp ("text/plain", top->subtype) &&
-	    mutt_strcasecmp ("application/postscript", top->subtype))
+	if (ascii_strcasecmp ("text/plain", top->subtype) &&
+	    ascii_strcasecmp ("application/postscript", top->subtype))
 	{
 	  if (!mutt_can_decode (top))
 	  {
@@ -642,8 +682,8 @@ static void print_attachment_list (FILE *fp, int tag, BODY *top, STATE *state)
       snprintf (type, sizeof (type), "%s/%s", TYPE (top), top->subtype);
       if (!option (OPTATTACHSPLIT) && !rfc1524_mailcap_lookup (top, type, NULL, M_PRINT))
       {
-	if (!mutt_strcasecmp ("text/plain", top->subtype) ||
-	    !mutt_strcasecmp ("application/postscript", top->subtype))
+	if (!ascii_strcasecmp ("text/plain", top->subtype) ||
+	    !ascii_strcasecmp ("application/postscript", top->subtype))
 	  pipe_attachment (fp, top, state);
 	else if (mutt_can_decode (top))
 	{
@@ -653,7 +693,7 @@ static void print_attachment_list (FILE *fp, int tag, BODY *top, STATE *state)
 	  FILE *ifp;
 
 	  mutt_mktemp (newfile);
-	  if (mutt_decode_save_attachment (fp, top, newfile, 0, 0) == 0)
+	  if (mutt_decode_save_attachment (fp, top, newfile, M_PRINTING, 0) == 0)
 	  {
 	    if ((ifp = fopen (newfile, "r")) != NULL)
 	    {
@@ -688,7 +728,7 @@ void mutt_print_attachment_list (FILE *fp, int tag, BODY *top)
   {
     if (!can_print (top, tag))
       return;
-    endwin ();
+    mutt_endwin (NULL);
     memset (&state, 0, sizeof (STATE));
     thepid = mutt_create_filter (NONULL (PrintCmd), &state.fpout, NULL, NULL);
     print_attachment_list (fp, tag, top, &state);
@@ -701,9 +741,34 @@ void mutt_print_attachment_list (FILE *fp, int tag, BODY *top)
 }
 
 void
-mutt_attach_display_loop (MUTTMENU *menu, int op, FILE *fp, HEADER *hdr, 
-			  ATTACHPTR **idx, short idxlen)
+mutt_update_attach_index (BODY *cur, ATTACHPTR ***idxp,
+				      short *idxlen, short *idxmax,
+				      MUTTMENU *menu)
 {
+  ATTACHPTR **idx = *idxp;
+  while (--(*idxlen) >= 0)
+    idx[(*idxlen)]->content = NULL;
+  *idxlen = 0;
+
+  idx = *idxp = mutt_gen_attach_list (cur, -1, idx, idxlen, idxmax, 0, 0);
+  
+  menu->max  = *idxlen;
+  menu->data = *idxp;
+
+  if (menu->current >= menu->max)
+    menu->current = menu->max - 1;
+  menu_check_recenter (menu);
+  menu->redraw |= REDRAW_INDEX;
+  
+}
+
+
+int
+mutt_attach_display_loop (MUTTMENU *menu, int op, FILE *fp, HEADER *hdr,
+			  BODY *cur, ATTACHPTR ***idxp, short *idxlen, short *idxmax,
+			  int recv)
+{
+  ATTACHPTR **idx = *idxp;
 #if 0
   int old_optweed = option (OPTWEED);
   set_option (OPTWEED);
@@ -719,7 +784,7 @@ mutt_attach_display_loop (MUTTMENU *menu, int op, FILE *fp, HEADER *hdr,
 
       case OP_VIEW_ATTACH:
 	op = mutt_view_attachment (fp, idx[menu->current]->content, M_REGULAR,
-				   hdr, idx, idxlen);
+				   hdr, idx, *idxlen);
 	break;
 
       case OP_NEXT_ENTRY:
@@ -745,9 +810,25 @@ mutt_attach_display_loop (MUTTMENU *menu, int op, FILE *fp, HEADER *hdr,
       case OP_EDIT_TYPE:
 	/* when we edit the content-type, we should redisplay the attachment
 	   immediately */
-	mutt_edit_content_type (hdr, idx[menu->current]->content);
-	op = OP_VIEW_ATTACH;
+	mutt_edit_content_type (hdr, idx[menu->current]->content, fp);
+        if (idxmax)
+        {
+	  mutt_update_attach_index (cur, idxp, idxlen, idxmax, menu);
+	  idx = *idxp;
+	}
+        op = OP_VIEW_ATTACH;
 	break;
+      /* functions which are passed through from the pager */
+      case OP_CHECK_TRADITIONAL:
+        if (!(WithCrypto & APPLICATION_PGP) || (hdr && hdr->security & PGP_TRADITIONAL_CHECKED))
+        {
+          op = OP_NULL;
+          break;
+        }
+        /* fall through */
+      case OP_ATTACH_COLLAPSE:
+        if (recv)
+          return op;
       default:
 	op = OP_NULL;
     }
@@ -758,8 +839,36 @@ mutt_attach_display_loop (MUTTMENU *menu, int op, FILE *fp, HEADER *hdr,
   if (option (OPTWEED) != old_optweed)
     toggle_option (OPTWEED);
 #endif
+  return op;
 }
 
+static void attach_collapse (BODY *b, short collapse, short init, short just_one)
+{
+  short i;
+  for (; b; b = b->next)
+  {
+    i = init || b->collapsed;
+    if (i && option (OPTDIGESTCOLLAPSE) && b->type == TYPEMULTIPART
+	&& !ascii_strcasecmp (b->subtype, "digest"))
+      attach_collapse (b->parts, 1, 1, 0);
+    else if (b->type == TYPEMULTIPART || mutt_is_message_type (b->type, b->subtype))
+      attach_collapse (b->parts, collapse, i, 0);
+    b->collapsed = collapse;
+    if (just_one)
+      return;
+  }
+}
+
+void mutt_attach_init (BODY *b)
+{
+  for (; b; b = b->next)
+  {
+    b->tagged = 0;
+    b->collapsed = 0;
+    if (b->parts) 
+      mutt_attach_init (b->parts);
+  }
+}
 
 static const char *Function_not_permitted = N_("Function not permitted in attach-message mode.");
 
@@ -770,16 +879,13 @@ static const char *Function_not_permitted = N_("Function not permitted in attach
 			break; \
 		     }
 
+
+
+
 void mutt_view_attachments (HEADER *hdr)
 {
-
-
-
-#ifdef HAVE_PGP
-  int pgp = 0;
-#endif
-
-
+  int secured = 0;
+  int need_secured = 0;
 
   char helpstr[SHORT_STRING];
   MUTTMENU *menu;
@@ -790,60 +896,92 @@ void mutt_view_attachments (HEADER *hdr)
   short idxlen = 0;
   short idxmax = 0;
   int flags = 0;
-  int op;
+  int op = OP_NULL;
   
   /* make sure we have parsed this message */
   mutt_parse_mime_message (Context, hdr);
 
+  mutt_message_hook (Context, hdr, M_MESSAGEHOOK);
+  
   if ((msg = mx_open_message (Context, hdr->msgno)) == NULL)
     return;
 
 
+  if (WithCrypto && ((hdr->security & ENCRYPT) ||
+                     (mutt_is_application_smime(hdr->content) & SMIMEOPAQUE)))
+  {
+    need_secured  = 1;
 
-#ifdef HAVE_PGP
-  if((hdr->pgp & PGPENCRYPT) && !pgp_valid_passphrase())
-  {
-    mx_close_message (&msg);
-    return;
-  }
-  
-  if ((hdr->pgp & PGPENCRYPT) && mutt_is_multipart_encrypted(hdr->content))
-  {
-    if (pgp_decrypt_mime (msg->fp, &fp, hdr->content, &cur))
+    if ((hdr->security & ENCRYPT) && !crypt_valid_passphrase(hdr->security))
     {
       mx_close_message (&msg);
       return;
     }
-    pgp = 1;
+    if ((WithCrypto & APPLICATION_SMIME) && (hdr->security & APPLICATION_SMIME))
+    {
+      if (hdr->env)
+	  crypt_smime_getkeys (hdr->env);
+
+      if (mutt_is_application_smime(hdr->content))
+      {
+	secured = ! crypt_smime_decrypt_mime (msg->fp, &fp,
+                                              hdr->content, &cur);
+	
+	/* S/MIME nesting */
+	if ((mutt_is_application_smime (cur) & SMIMEOPAQUE))
+	{
+	  BODY *_cur = cur;
+	  FILE *_fp = fp;
+	  
+	  fp = NULL; cur = NULL;
+	  secured = !crypt_smime_decrypt_mime (_fp, &fp, _cur, &cur);
+	  
+	  mutt_free_body (&_cur);
+	  safe_fclose (&_fp);
+	}
+      }
+      else
+	need_secured = 0;
+    }
+    if ((WithCrypto & APPLICATION_PGP) && (hdr->security & APPLICATION_PGP))
+    {
+      if (mutt_is_multipart_encrypted(hdr->content))
+	secured = !crypt_pgp_decrypt_mime (msg->fp, &fp, hdr->content, &cur);
+      else
+	need_secured = 0;
+    }
+
+    if (need_secured && !secured)
+    {
+      mx_close_message (&msg);
+      mutt_error _("Can't decrypt encrypted message!");
+      return;
+    }
   }
-  else
-#endif /* HAVE_PGP */
+  
+  if (!WithCrypto || !need_secured)
   {
     fp = msg->fp;
     cur = hdr->content;
   }
 
-  idx = mutt_gen_attach_list (cur, -1, idx, &idxlen, &idxmax, 0, 0);
-
   menu = mutt_new_menu ();
-  menu->max = idxlen;
-  menu->make_entry = attach_entry;
-  menu->tag = mutt_tag_attach;
   menu->menu = MENU_ATTACH;
   menu->title = _("Attachments");
-  menu->data = idx;
+  menu->make_entry = attach_entry;
+  menu->tag = mutt_tag_attach;
   menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_ATTACH, AttachHelp);
 
+  mutt_attach_init (cur);
+  attach_collapse (cur, 0, 1, 0);
+  mutt_update_attach_index (cur, &idx, &idxlen, &idxmax, menu);
+
   FOREVER
   {
-    switch (op = mutt_menuLoop (menu))
+    if (op == OP_NULL)
+      op = mutt_menuLoop (menu);
+    switch (op)
     {
-      case OP_DISPLAY_HEADERS:
-      case OP_VIEW_ATTACH:
-	mutt_attach_display_loop (menu, op, fp, hdr, idx, idxlen);
-	menu->redraw = REDRAW_FULL;
-	break;
-
       case OP_ATTACH_VIEW_MAILCAP:
 	mutt_view_attachment (fp, idx[menu->current]->content, M_MAILCAP,
 			      hdr, idx, idxlen);
@@ -856,17 +994,48 @@ void mutt_view_attachments (HEADER *hdr)
 	menu->redraw = REDRAW_FULL;
 	break;
 
-
-
-#ifdef HAVE_PGP
-      case OP_EXTRACT_KEYS:
-        pgp_extract_keys_from_attachment_list (fp, menu->tagprefix, 
-		  menu->tagprefix ? cur : idx[menu->current]->content);
+      case OP_DISPLAY_HEADERS:
+      case OP_VIEW_ATTACH:
+        op = mutt_attach_display_loop (menu, op, fp, hdr, cur, &idx, &idxlen, &idxmax, 1);
         menu->redraw = REDRAW_FULL;
-        break;
-#endif
-      
+        continue;
 
+      case OP_ATTACH_COLLAPSE:
+        if (!idx[menu->current]->content->parts)
+        {
+	  mutt_error _("There are no subparts to show!");
+	  break;
+	}
+        if (!idx[menu->current]->content->collapsed)
+	  attach_collapse (idx[menu->current]->content, 1, 0, 1);
+        else
+	  attach_collapse (idx[menu->current]->content, 0, 1, 1);
+        mutt_update_attach_index (cur, &idx, &idxlen, &idxmax, menu);
+        break;
+      
+      case OP_FORGET_PASSPHRASE:
+        crypt_forget_passphrase ();
+        break;
+
+      case OP_EXTRACT_KEYS:
+        if ((WithCrypto & APPLICATION_PGP))
+        {
+          crypt_pgp_extract_keys_from_attachment_list (fp, menu->tagprefix, 
+		    menu->tagprefix ? cur : idx[menu->current]->content);
+          menu->redraw = REDRAW_FULL;
+        }
+        break;
+      
+      case OP_CHECK_TRADITIONAL:
+        if ((WithCrypto & APPLICATION_PGP)
+            && crypt_pgp_check_traditional (fp, menu->tagprefix ? cur
+                                              : idx[menu->current]->content,
+                                      menu->tagprefix))
+        {
+	  hdr->security = crypt_query (cur);
+	  menu->redraw = REDRAW_FULL;
+	}
+        break;
 
       case OP_PRINT:
 	mutt_print_attachment_list (fp, menu->tagprefix, 
@@ -880,9 +1049,9 @@ void mutt_view_attachments (HEADER *hdr)
 
       case OP_SAVE:
 	mutt_save_attachment_list (fp, menu->tagprefix, 
-		  menu->tagprefix ?  cur : idx[menu->current]->content, hdr);
+		  menu->tagprefix ?  cur : idx[menu->current]->content, hdr, menu);
 
-        if (option (OPTRESOLVE) && menu->current < menu->max - 1)
+        if (!menu->tagprefix && option (OPTRESOLVE) && menu->current < menu->max - 1)
 	  menu->current++;
       
         menu->redraw = REDRAW_MOTION_RESYNCH | REDRAW_FULL;
@@ -891,16 +1060,21 @@ void mutt_view_attachments (HEADER *hdr)
       case OP_DELETE:
 	CHECK_READONLY;
 
+#ifdef USE_POP
+	if (Context->magic == M_POP)
+	{
+	  mutt_flushinp ();
+	  mutt_error _("Can't delete attachment from POP server.");
+	  break;
+	}
+#endif
 
-
-#ifdef HAVE_PGP
-        if (hdr->pgp)
+        if (WithCrypto && hdr->security)
         {
 	  mutt_message _(
-	    "Deletion of attachments from PGP messages is unsupported.");
+	    "Deletion of attachments from encrypted messages is unsupported.");
 	}
         else
-#endif
         {
 	  if (!menu->tagprefix)
 	  {
@@ -1005,40 +1179,40 @@ void mutt_view_attachments (HEADER *hdr)
 	break;
 
       case OP_EDIT_TYPE:
-	mutt_edit_content_type (hdr, idx[menu->current]->content);
-	menu->redraw = REDRAW_CURRENT;
+	mutt_edit_content_type (hdr, idx[menu->current]->content, fp);
+        mutt_update_attach_index (cur, &idx, &idxlen, &idxmax, menu);
 	break;
 
       case OP_EXIT:
 	mx_close_message (&msg);
 	hdr->attach_del = 0;
-	while (idxlen-- > 0)
+	while (idxmax-- > 0)
 	{
-	  if (idx[idxlen]->content->deleted)
+	  if (!idx[idxmax])
+	    continue;
+	  if (idx[idxmax]->content && idx[idxmax]->content->deleted)
 	    hdr->attach_del = 1;
-	  safe_free ((void **) &idx[idxlen]->tree);
-	  safe_free ((void **) &idx[idxlen]);
+	  if (idx[idxmax]->content)
+	    idx[idxmax]->content->aptr = NULL;
+	  FREE (&idx[idxmax]->tree);
+	  FREE (&idx[idxmax]);
 	}
 	if (hdr->attach_del)
 	  hdr->changed = 1;
-	safe_free ((void **) &idx);
+	FREE (&idx);
 	idxmax = 0;
 
-
-
-#ifdef HAVE_PGP
-	if (pgp)
+        if (WithCrypto && need_secured && secured)
 	{
 	  fclose (fp);
 	  mutt_free_body (&cur);
 	}
-#endif /* HAVE_PGP */
-
-
 
 	mutt_menuDestroy  (&menu);
 	return;
     }
+
+    op = OP_NULL;
   }
 
   /* not reached */
