@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
+ * Copyright (C) 1999-2000 Thomas Roessler <roessler@does-not-exist.org>
  * 
  *     This program is free software; you can redistribute it
  *     and/or modify it under the terms of the GNU General Public
@@ -19,6 +19,10 @@
  *     Boston, MA  02111, USA. 
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mutt_menu.h"
@@ -29,7 +33,7 @@
 #include "mapping.h"
 #include "mx.h"
 #include "copy.h"
-
+#include "mutt_idna.h"
 
 /* some helper functions to verify that we are exclusively operating
  * on message/rfc822 attachments
@@ -126,17 +130,20 @@ void mutt_attach_bounce (FILE * fp, HEADER * hdr,
 	   ATTACHPTR ** idx, short idxlen, BODY * cur)
 {
   short i;
-  short ntagged;
   char prompt[STRING];
   char buf[HUGE_STRING];
+  char *err = NULL;
   ADDRESS *adr = NULL;
+  int ret = 0;
+  int p   = 0;
 
   if (check_all_msg (idx, idxlen, cur, 1) == -1)
     return;
 
-  ntagged = count_tagged (idx, idxlen);
-  
-  if (cur || ntagged == 1)
+  /* one or more messages? */
+  p = (cur || count_tagged (idx, idxlen) == 1);
+
+  if (p)
     strfcpy (prompt, _("Bounce message to: "), sizeof (prompt));
   else
     strfcpy (prompt, _("Bounce tagged messages to: "), sizeof (prompt));
@@ -146,32 +153,68 @@ void mutt_attach_bounce (FILE * fp, HEADER * hdr,
       || buf[0] == '\0')
     return;
 
-  adr = rfc822_parse_adrlist (adr, buf);
+  if (!(adr = rfc822_parse_adrlist (adr, buf)))
+  {
+    mutt_error _("Error parsing address!");
+    return;
+  }
+
   adr = mutt_expand_aliases (adr);
+  
+  if (mutt_addrlist_to_idna (adr, &err) < 0)
+  {
+    mutt_error (_("Bad IDN: '%s'"), err);
+    FREE (&err);
+    rfc822_free_address (&adr);
+    return;
+  }
+  
   buf[0] = 0;
-  rfc822_write_address (buf, sizeof (buf), adr);
+  rfc822_write_address (buf, sizeof (buf), adr, 1);
 
-  snprintf (prompt, sizeof (prompt), 
-	    cur ? _("Bounce message to %s...?") :  _("Bounce messages to %s...?"), buf);
-
-  if (mutt_yesorno (prompt, 1) != 1)
-    goto bail;
+#define extra_space (15+7+2)
+  /*
+   * See commands.c.
+   */
+  snprintf (prompt, sizeof (prompt) - 4, 
+   (p ? _("Bounce message to %s") : _("Bounce messages to %s")), buf);
+  
+  if (mutt_strwidth (prompt) > COLS - extra_space)
+  {
+    mutt_format_string (prompt, sizeof (prompt) - 4,
+			0, COLS-extra_space, 0, 0,
+			prompt, sizeof (prompt), 0);
+    safe_strcat (prompt, sizeof (prompt), "...?");
+  }
+  else
+    safe_strcat (prompt, sizeof (prompt), "?");
 
+  if (query_quadoption (OPT_BOUNCE, prompt) != M_YES)
+  {
+    rfc822_free_address (&adr);
+    CLEARLINE (LINES - 1);
+    mutt_message (p ? _("Message not bounced.") : _("Messages not bounced."));
+    return;
+  }
+  
+  CLEARLINE (LINES - 1);
+  
   if (cur)
-    mutt_bounce_message (fp, cur->hdr, adr);
+    ret = mutt_bounce_message (fp, cur->hdr, adr);
   else
   {
     for (i = 0; i < idxlen; i++)
     {
       if (idx[i]->content->tagged)
-	mutt_bounce_message (fp, idx[i]->content->hdr, adr);
+	if (mutt_bounce_message (fp, idx[i]->content->hdr, adr))
+	  ret = 1;
     }
   }
 
-bail:
-
-  rfc822_free_address (&adr);
-  CLEARLINE (LINES - 1);
+  if (!ret)
+    mutt_message (p ? _("Message bounced.") : _("Messages bounced."));
+  else
+    mutt_error (p ? _("Error bouncing message!") : _("Error bouncing messages!"));
 }
 
 
@@ -290,9 +333,11 @@ static void include_header (int quote, FILE * ifp,
   {
     if (_prefix)
       strfcpy (prefix, _prefix, sizeof (prefix));
-    else
+    else if (!option (OPTTEXTFLOWED))
       _mutt_make_string (prefix, sizeof (prefix), NONULL (Prefix), 
 			 Context, hdr, 0);
+    else
+      strfcpy (prefix, ">", sizeof (prefix));
 
     chflags |= CH_PREFIX;
   }
@@ -378,9 +423,14 @@ static void attach_forward_bodies (FILE * fp, HEADER * hdr,
   /* prepare the prefix here since we'll need it later. */
 
   if (option (OPTFORWQUOTE))
-    _mutt_make_string (prefix, sizeof (prefix), NONULL (Prefix), Context,
-		       parent, 0);
-
+  {
+    if (!option (OPTTEXTFLOWED))
+      _mutt_make_string (prefix, sizeof (prefix), NONULL (Prefix), Context,
+			 parent, 0);
+    else
+      strfcpy (prefix, ">", sizeof (prefix));
+  }
+    
   include_header (option (OPTFORWQUOTE), fp, parent,
 		  tmpfp, prefix);
 
@@ -395,7 +445,7 @@ static void attach_forward_bodies (FILE * fp, HEADER * hdr,
 
   if ((!cur || mutt_can_decode (cur)) &&
       (rc = query_quadoption (OPT_MIMEFWD, 
-			      _("Forward MIME encapsulated?"))) == M_YES)
+			      _("Forward as attachments?"))) == M_YES)
     mime_fwd_all = 1;
   else if (rc == -1)
     goto bail;
@@ -566,6 +616,7 @@ static void attach_forward_msgs (FILE * fp, HEADER * hdr,
     
     if (cur)
     {
+      /* mutt_message_hook (cur->hdr, M_MESSAGEHOOK); */ 
       mutt_forward_intro (tmpfp, cur->hdr);
       _mutt_copy_message (tmpfp, fp, cur->hdr, cur->hdr->content, cmflags, chflags);
       mutt_forward_trailer (tmpfp);
@@ -576,6 +627,7 @@ static void attach_forward_msgs (FILE * fp, HEADER * hdr,
       {
 	if (idx[i]->content->tagged)
 	{
+	  /* mutt_message_hook (idx[i]->content->hdr, M_MESSAGEHOOK); */ 
 	  mutt_forward_intro (tmpfp, idx[i]->content->hdr);
 	  _mutt_copy_message (tmpfp, fp, idx[i]->content->hdr,
 			      idx[i]->content->hdr->content, cmflags, chflags);
@@ -700,23 +752,17 @@ attach_reply_envelope_defaults (ENVELOPE *env, ATTACHPTR **idx, short idxlen,
   
   mutt_fix_reply_recipients (env);
   mutt_make_misc_reply_headers (env, Context, curhdr, curenv);
-  
+
   if (parent)
-    env->references = mutt_make_references (curenv);
+    mutt_add_to_reference_headers (env, curenv, NULL, NULL);
   else
   {
-    LIST **p;
-
-    env->references = NULL;
-    p = &env->references;
-
+    LIST **p = NULL, **q = NULL;
+    
     for (i = 0; i < idxlen; i++)
     {
       if (idx[i]->content->tagged)
-      {
-	while (*p) p = &(*p)->next;
-	*p = mutt_make_references (idx[i]->content->hdr->env);
-      }
+	mutt_add_to_reference_headers (env, idx[i]->content->hdr->env, &p, &q);
     }
   }
   
@@ -731,6 +777,8 @@ static void attach_include_reply (FILE *fp, FILE *tmpfp, HEADER *cur, int flags)
   int cmflags = M_CM_PREFIX | M_CM_DECODE | M_CM_CHARCONV;
   int chflags = CH_DECODE;
 
+  /* mutt_message_hook (cur, M_MESSAGEHOOK); */ 
+  
   mutt_make_attribution (Context, cur, tmpfp);
   
   if (!option (OPTHEADER))
@@ -785,7 +833,7 @@ void mutt_attach_reply (FILE * fp, HEADER * hdr,
   tmphdr->env = mutt_new_envelope ();
 
   if (attach_reply_envelope_defaults (tmphdr->env, idx, idxlen, 
-				      parent ? parent : cur->hdr, flags) == -1)
+				      parent ? parent : (cur ? cur->hdr : NULL), flags) == -1)
   {
     mutt_free_header (&tmphdr);
     return;
@@ -820,8 +868,11 @@ void mutt_attach_reply (FILE * fp, HEADER * hdr,
     st.fpin = fp;
     st.fpout = tmpfp;
 
-    _mutt_make_string (prefix, sizeof (prefix), NONULL (Prefix), 
-		       Context, parent, 0);
+    if (!option (OPTTEXTFLOWED))
+      _mutt_make_string (prefix, sizeof (prefix), NONULL (Prefix), 
+			 Context, parent, 0);
+    else
+      strfcpy (prefix, ">", sizeof (prefix));
 
     st.prefix = prefix;
     st.flags  = M_CHARCONV;
