@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 /* 
@@ -26,14 +26,21 @@
  * rfc1524.
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "rfc1524.h"
 
-#include <ctype.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <sys/wait.h>
 #include <string.h>
+#include <stdlib.h>
+#include <ctype.h>
+
+#include <sys/stat.h>
+#include <sys/wait.h>
+#include <errno.h>
+#include <unistd.h>
 
 /* The command semantics include the following:
  * %s is the filename that contains the mail body data
@@ -73,46 +80,30 @@ int rfc1524_expand_command (BODY *a, char *filename, char *_type,
       if (command[x] == '{') 
       {
 	char param[STRING];
-	char pvalue[LONG_STRING];
+	char pvalue[STRING];
+	char *_pvalue;
 	int z = 0;
-	char *ret = NULL;
-	char *pv;
 
 	x++;
 	while (command[x] && command[x] != '}' && z<sizeof(param))
 	  param[z++] = command[x++];
 	param[z] = '\0';
-	dprint(2,(debugfile,"Parameter: %s  Returns: %s\n",param,ret));
-	pv = mutt_get_parameter (param, a->parameter);
-	strfcpy (pvalue, NONULL(pv), sizeof (pvalue));
-	if (option (OPTMAILCAPSANITIZE)) 
+	
+	_pvalue = mutt_get_parameter (param, a->parameter);
+	strfcpy (pvalue, NONULL(_pvalue), sizeof (pvalue));
+	if (option (OPTMAILCAPSANITIZE))
 	  mutt_sanitize_filename (pvalue, 0);
-	ret = mutt_quote_filename (pvalue);
-	dprint(2,(debugfile,"Parameter: %s  Returns: %s\n",param,ret));
-	z = 0;
-	while (ret && ret[z] && y<sizeof(buf))
-	  buf[y++] = ret[z++];
-	FREE(&ret);
+	
+	y += mutt_quote_filename (buf + y, sizeof (buf) - y, pvalue);
       }
       else if (command[x] == 's' && filename != NULL)
       {
-	char *fn = mutt_quote_filename(filename);
-	int i;
-	
-	for(i = 0; fn[i] && y < sizeof(buf); i++)
-	  buf[y++] = fn[i];
-	
-	FREE(&fn);
+	y += mutt_quote_filename (buf + y, sizeof (buf) - y, filename);
 	needspipe = FALSE;
       }
       else if (command[x] == 't')
       {
-	char *t = mutt_quote_filename (type);
-	char *s;
-	
-	for (s = t; *s && y < sizeof (buf);)
-	  buf[y++] = *s++;
-	FREE (&t);
+	y += mutt_quote_filename (buf + y, sizeof (buf) - y, type);
       }
       x++;
     }
@@ -163,8 +154,7 @@ static int get_field_text (char *field, char **entry,
     {
       field++;
       field = mutt_skip_whitespace (field);
-      safe_free ((void **) entry);
-      *entry = safe_strdup (field);
+      mutt_str_replace (entry, field);
     }
     return 1;
   }
@@ -222,8 +212,8 @@ static int rfc1524_mailcap_parse (BODY *a,
 
       /* check type */
       ch = get_field (buf);
-      if (mutt_strcasecmp (buf, type) &&
-	  (mutt_strncasecmp (buf, type, btlen) ||
+      if (ascii_strcasecmp (buf, type) &&
+	  (ascii_strncasecmp (buf, type, btlen) ||
 	   (buf[btlen] != 0 &&			/* implicit wild */
 	    mutt_strcmp (buf + btlen, "/*"))))	/* wildsubtype */
 	continue;
@@ -247,53 +237,53 @@ static int rfc1524_mailcap_parse (BODY *a,
 	ch = get_field (ch);
 	dprint (2, (debugfile, "field: %s\n", field));
 
-	if (!mutt_strcasecmp (field, "needsterminal"))
+	if (!ascii_strcasecmp (field, "needsterminal"))
 	{
 	  if (entry)
 	    entry->needsterminal = TRUE;
 	}
-	else if (!mutt_strcasecmp (field, "copiousoutput"))
+	else if (!ascii_strcasecmp (field, "copiousoutput"))
 	{
 	  copiousoutput = TRUE;
 	  if (entry)
 	    entry->copiousoutput = TRUE;
 	}
-	else if (!mutt_strncasecmp (field, "composetyped", 12))
+	else if (!ascii_strncasecmp (field, "composetyped", 12))
 	{
 	  /* this compare most occur before compose to match correctly */
 	  if (get_field_text (field + 12, entry ? &entry->composetypecommand : NULL,
 			      type, filename, line))
 	    composecommand = TRUE;
 	}
-	else if (!mutt_strncasecmp (field, "compose", 7))
+	else if (!ascii_strncasecmp (field, "compose", 7))
 	{
 	  if (get_field_text (field + 7, entry ? &entry->composecommand : NULL,
 			      type, filename, line))
 	    composecommand = TRUE;
 	}
-	else if (!mutt_strncasecmp (field, "print", 5))
+	else if (!ascii_strncasecmp (field, "print", 5))
 	{
 	  if (get_field_text (field + 5, entry ? &entry->printcommand : NULL,
 			      type, filename, line))
 	    printcommand = TRUE;
 	}
-	else if (!mutt_strncasecmp (field, "edit", 4))
+	else if (!ascii_strncasecmp (field, "edit", 4))
 	{
 	  if (get_field_text (field + 4, entry ? &entry->editcommand : NULL,
 			      type, filename, line))
 	    editcommand = TRUE;
 	}
-	else if (!mutt_strncasecmp (field, "nametemplate", 12))
+	else if (!ascii_strncasecmp (field, "nametemplate", 12))
 	{
 	  get_field_text (field + 12, entry ? &entry->nametemplate : NULL,
 			  type, filename, line);
 	}
-	else if (!mutt_strncasecmp (field, "x-convert", 9))
+	else if (!ascii_strncasecmp (field, "x-convert", 9))
 	{
 	  get_field_text (field + 9, entry ? &entry->convert : NULL,
 			  type, filename, line);
 	}
-	else if (!mutt_strncasecmp (field, "test", 4))
+	else if (!ascii_strncasecmp (field, "test", 4))
 	{
 	  /* 
 	   * This routine executes the given test command to determine
@@ -306,8 +296,8 @@ static int rfc1524_mailcap_parse (BODY *a,
 	      && test_command)
 	  {
 	    len = mutt_strlen (test_command) + STRING;
-	    safe_realloc ((void **) &test_command, len);
-	    rfc1524_expand_command (a, NULL, type, test_command, len);
+	    safe_realloc (&test_command, len);
+	    rfc1524_expand_command (a, a->filename, type, test_command, len);
 	    if (mutt_system (test_command))
 	    {
 	      /* a non-zero exit code means test failed */
@@ -344,13 +334,13 @@ static int rfc1524_mailcap_parse (BODY *a,
 	/* reset */
 	if (entry)
 	{
-	  safe_free ((void **) &entry->command);
-	  safe_free ((void **) &entry->composecommand);
-	  safe_free ((void **) &entry->composetypecommand);
-	  safe_free ((void **) &entry->editcommand);
-	  safe_free ((void **) &entry->printcommand);
-	  safe_free ((void **) &entry->nametemplate);
-	  safe_free ((void **) &entry->convert);
+	  FREE (&entry->command);
+	  FREE (&entry->composecommand);
+	  FREE (&entry->composetypecommand);
+	  FREE (&entry->editcommand);
+	  FREE (&entry->printcommand);
+	  FREE (&entry->nametemplate);
+	  FREE (&entry->convert);
 	  entry->needsterminal = 0;
 	  entry->copiousoutput = 0;
 	}
@@ -358,7 +348,7 @@ static int rfc1524_mailcap_parse (BODY *a,
     } /* while (!found && (buf = mutt_read_line ())) */
     fclose (fp);
   } /* if ((fp = fopen ())) */
-  safe_free ((void **) &buf);
+  FREE (&buf);
   return found;
 }
 
@@ -371,14 +361,14 @@ void rfc1524_free_entry(rfc1524_entry **entry)
 {
   rfc1524_entry *p = *entry;
 
-  safe_free((void **)&p->command);
-  safe_free((void **)&p->testcommand);
-  safe_free((void **)&p->composecommand);
-  safe_free((void **)&p->composetypecommand);
-  safe_free((void **)&p->editcommand);
-  safe_free((void **)&p->printcommand);
-  safe_free((void **)&p->nametemplate);
-  safe_free((void **)entry);
+  FREE (&p->command);
+  FREE (&p->testcommand);
+  FREE (&p->composecommand);
+  FREE (&p->composetypecommand);
+  FREE (&p->editcommand);
+  FREE (&p->printcommand);
+  FREE (&p->nametemplate);
+  FREE (entry);
 }
 
 /*
@@ -406,6 +396,8 @@ int rfc1524_mailcap_lookup (BODY *a, char *type, rfc1524_entry *entry, int opt)
     return 0;
   }
 
+  mutt_check_lookup_list (a, type, SHORT_STRING);
+
   while (!found && *curr)
   {
     x = 0;
@@ -433,44 +425,6 @@ int rfc1524_mailcap_lookup (BODY *a, char *type, rfc1524_entry *entry, int opt)
   return found;
 }
 
-/* Modified by blong to accept a "suggestion" for file name.  If
- * that file exists, then construct one with unique name but 
- * keep any extension.  This might fail, I guess.
- * Renamed to mutt_adv_mktemp so I only have to change where it's
- * called, and not all possible cases.
- */
-void mutt_adv_mktemp (char *s, size_t l)
-{
-  char buf[_POSIX_PATH_MAX];
-  char tmp[_POSIX_PATH_MAX];
-  char *period;
-  size_t sl;
-  
-  strfcpy (buf, NONULL (Tempdir), sizeof (buf));
-  mutt_expand_path (buf, sizeof (buf));
-  if (s[0] == '\0')
-  {
-    snprintf (s, l, "%s/muttXXXXXX", buf);
-    mktemp (s);
-  }
-  else
-  {
-    strfcpy (tmp, s, sizeof (tmp));
-    snprintf (s, l, "%s/%s", buf, tmp);
-    if (access (s, F_OK) != 0)
-      return;
-    if ((period = strrchr (tmp, '.')) != NULL)
-      *period = 0;
-    snprintf (s, l, "%s/%s.XXXXXX", buf, tmp);
-    mktemp (s);
-    if (period != NULL)
-    {
-      *period = '.';
-      sl = mutt_strlen(s);
-      strfcpy(s + sl, period, l - sl);
-    }
-  }
-}
 
 /* This routine will create a _temporary_ filename matching the
  * name template given if this needs to be done.
@@ -519,7 +473,7 @@ int rfc1524_expand_filename (char *nametemplate,
   }
   else if (!oldfile)
   {
-    snprintf (newfile, nflen, nametemplate, "mutt");
+    mutt_expand_fmt (newfile, nflen, nametemplate, "mutt");
   }
   else /* oldfile && nametemplate */
   {
@@ -610,6 +564,11 @@ int rfc1524_expand_filename (char *nametemplate,
  * This function returns 0 on successful move, 1 on old file doesn't exist,
  * 2 on new file already exists, and 3 on other failure.
  */
+
+/* note on access(2) use: No dangling symlink problems here due to
+ * safe_fopen().
+ */
+
 int mutt_rename_file (char *oldfile, char *newfile)
 {
   FILE *ofp, *nfp;
