@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -16,8 +16,13 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
-void rfc2047_encode_string (char *, size_t, const unsigned char *);
-void rfc2047_encode_adrlist (ADDRESS *);
+char *mutt_choose_charset (const char *fromcode, const char *charsets,
+		      char *u, size_t ulen, char **d, size_t *dlen);
 
-void rfc2047_decode (char *, const char *, size_t);
+void _rfc2047_encode_string (char **, int, int);
+void rfc2047_encode_adrlist (ADDRESS *, const char *);
+
+#define rfc2047_encode_string(a) _rfc2047_encode_string (a, 0, 32);
+
+void rfc2047_decode (char **);
 void rfc2047_decode_adrlist (ADDRESS *);
